@@ -2,8 +2,12 @@
 
 require 5.000;
 
+#Note: mkdir() must have 2 arguments as late as 5.005.
+
 use strict;
 use English;
+use Fcntl;
+use File::Basename;
 
 my ($TRUE, $FALSE) = (1,0);
 
@@ -102,6 +106,8 @@ sub getPrefix() {
     my $default;
     if ($OSNAME eq "cygwin") {
         $default = "/usr/local";
+    } elsif ($ENV{OSTYPE} && $ENV{OSTYPE} eq "msdosdjgpp") {
+        $default = "/dev/env/DJDIR";
     } else {
         $default = "/usr/local/netpbm";
     }
@@ -187,9 +193,9 @@ sub getBinDir($) {
 
 
 
-sub installProgram($$) {
+sub installProgram($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $bindirR) = @_;
 
     my $binDir = getBinDir($prefix);
 
@@ -203,6 +209,7 @@ sub installProgram($$) {
     } else {
         print("Done.\n");
     }
+    $$bindirR = $binDir;
 }
 
 
@@ -252,6 +259,17 @@ execLdconfig() {
 #  from Ldconfig.  And we want to tread as lightly as possible on the 
 #  system -- we don't want creating symlinks that have nothing to do with
 #  Netpbm to be a hidden side effect of installing Netpbm.
+#
+#  Note that this Ldconfig works only if the user installed the Netpbm
+#  library in a standard directory that Ldconfig searches.  Note that on
+#  OpenBSD, Ldconfig is hardcoded to search only /usr/lib ever.  We could
+#  also do 'ldconfig DIR' to scan the particular directory in which we
+#  installed the Netpbm library.  But 1) the effects of this would disappear
+#  the next time the user rebuilds the cache file; and 2) on OpenBSD, this
+#  causes the cache file to be rebuilt from ONLY that directory.  On OpenBSD,
+#  you can add the -m option to cause it to ADD the contents of DIR to the
+#  existing cache file.
+#  
 #-----------------------------------------------------------------------------
 # Implementation note:  We've seen varying completion codes and varying
 # error messages from different versions of Ldconfig when it fails.
@@ -327,10 +345,9 @@ doLdconfig() {
 
 
 
+sub installSharedLib($$$) {
 
-sub installSharedLib($$) {
-
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $libdirR) = @_;
 
     if (-d("$pkgdir/lib")) {
         my $libDir = getLibDir($prefix);
@@ -347,6 +364,7 @@ sub installSharedLib($$) {
             print("\n");
             doLdconfig();
         }
+        $$libdirR = $libDir;
     } else {
         print("You did not build a shared library, so I will not " .
               "install one.\n");
@@ -390,9 +408,9 @@ sub getLinkDir($) {
 
 
 
-sub installStaticLib($$) {
+sub installStaticLib($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $linkdirR) = @_;
 
     if (-d("$pkgdir/link")) {
         my $linkDir = getLinkDir($prefix);
@@ -407,6 +425,7 @@ sub installStaticLib($$) {
         } else {
             print("done.\n");
         }
+        $$linkdirR = $linkDir;
     } else {
         print("You did not build a static library, so I will not " .
               "install one \n");
@@ -483,9 +502,9 @@ sub getHdrDir($) {
 
 
 
-sub installDataFile($$) {
+sub installDataFile($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $datadirR) = @_;
 
     my $dataDir = getDataDir($prefix);
 
@@ -498,15 +517,16 @@ sub installDataFile($$) {
               "failed.\n");
         print("cp exit code is $rc\n");
     } else {
+        $$datadirR = $dataDir;
         print("done.\n");
     }
 }
 
 
 
-sub installHeader($$) {
+sub installHeader($$$) {
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $includedirR) = @_;
 
     my $hdrDir = getHdrDir($prefix);
 
@@ -521,6 +541,7 @@ sub installHeader($$) {
     } else {
         print("done.\n");
     }
+    $$includedirR = $hdrDir;
 }
 
 
@@ -612,9 +633,15 @@ sub tryToCreateManwebConf($) {
 
 
 sub getWebdir($) {
-
     my ($manweb_conf_filename) = @_;
-
+#-----------------------------------------------------------------------------
+#  Return the value of the Manweb "web directory," as indicated by the
+#  Manweb configuration file $manweb_conf_filename.
+#
+#  If that file doesn't exist, or doesn't have a 'webdir' value, or
+#  the 'webdir' value is a chain of directories instead of just one,
+#  we return an undefined value.
+#-----------------------------------------------------------------------------
     my $webdir;
 
     my $success = open(MANWEB_CONF, "<$manweb_conf_filename");
@@ -632,7 +659,11 @@ sub getWebdir($) {
                 #It looks like "keyword=value"
                 my ($keyword, $value) = ($1, $2);
                 if ($keyword eq "webdir") {
-                    $webdir = $value;
+                    # We can't handle a multi-directory path; we're looking
+                    # only for a webdir statement naming a sole directory.
+                    if ($value !~ m{:}) {
+                        $webdir = $value;
+                    }
                 }
             }
         }
@@ -689,6 +720,7 @@ sub makeInManwebPath($) {
                 if (userWantsManwebSymlink($webdir, $netpbmWebdir)) {
                     my $old = "$netpbmWebdir/netpbm.url";
                     my $new = "$webdir/netpbm.url";
+                    mkdir($webdir, 0777);
                     my $success = symlink($old, $new);
                     if (!$success) {
                         print("Failed to create symbolic link from $new to " .
@@ -702,12 +734,12 @@ sub makeInManwebPath($) {
 
 
 
-sub installManPage($$) {
+sub installManPage($$$) {
 
 
 # Note: This installs the pointer man pages and the netpbm.url file for Manweb.
 
-    my ($pkgdir, $prefix) = @_;
+    my ($pkgdir, $prefix, $mandirR) = @_;
 
     my $manDir = getManDir($prefix);
 
@@ -727,11 +759,109 @@ sub installManPage($$) {
     removeObsoleteManPage($manDir);
 
     makeInManwebPath("$manDir/web");
-
+    
+    $$mandirR = $manDir;
 }
 
 
 
+sub 
+processTemplate($$$$$$$$$) {
+    my ($templateR, $version, $bindir, $libdir, $linkdir, $datadir, 
+        $includedir, $mandir, $outputR) = @_;
+
+    my @output;
+
+    foreach (@{$templateR}) {
+        if (m{^@}) {
+            # Comment -- ignore it.
+        } else {
+            if (defined($version)) {
+                s/\@VERSION\@/$version/;
+            }
+            if (defined($bindir)) {
+                s/\@BINDIR@/$bindir/;
+            }
+            if (defined($libdir)) {
+                s/\@LIBDIR@/$libdir/;
+            }
+            if (defined($linkdir)) {
+                s/\@LINKDIR@/$linkdir/;
+            }
+            if (defined($datadir)) {
+                s/\@DATADIR@/$datadir/;
+            }
+            if (defined($includedir)) {
+                s/\@INCLUDEDIR@/$includedir/;
+            }
+            if (defined($mandir)) {
+                s/\@MANDIR@/$mandir/;
+            }
+            push(@output, $_);
+        }
+    }
+    $$outputR = \@output;
+}
+
+
+
+sub
+installConfig($$$$$$$) {
+    my ($pkgdir, 
+        $bindir, $libdir, $linkdir, $datadir, $includedir, $mandir) = @_;
+
+    my $error;
+
+    my $configTemplateFilename = dirname($0) . "/config_template";
+
+    my $templateOpened = open(TEMPLATE, "<", $configTemplateFilename);
+    if (!$templateOpened) {
+        $error = "Can't open template file '$configTemplateFilename'.\n";
+    } else {
+        my @template = <TEMPLATE>;
+
+        close(TEMPLATE);
+
+        my $versionOpened = open(VERSION, "<", "$pkgdir/VERSION");
+
+        my $version;
+        if (!$versionOpened) {
+            $error = "Unable to open $pkgdir/VERSION for reading.  " .
+                "Errno=$ERRNO\n";
+        } else {
+            $version = <VERSION>;
+            chomp($version);
+            close(VERSION);
+
+            processTemplate(\@template, $version, $bindir, $libdir,
+                            $linkdir, $datadir, $includedir, $mandir,
+                            \my $fileContentsR);
+
+            # TODO: Really, this ought to go in an independent directory,
+            # because you might want to have the Netpbm executables in
+            # some place not in the PATH and use this program, via the
+            # PATH, to find them.
+            
+            my $filename = "$bindir/netpbm-config";
+            
+            my $success = open(NETPBM_CONFIG, ">", $filename);
+            if ($success) {
+                chmod(0755, $filename);
+                foreach (@{$fileContentsR}) { print NETPBM_CONFIG; }
+                close(NETPBM_CONFIG);
+            } else {
+                $error = "Unable to open the file " .
+                    "'$filename' for writing.  Errno=$ERRNO\n";
+            }
+        }
+    }
+    if ($error) {
+        print(STDERR "Failed to create the Netpbm configuration program.  " .
+              "$error\n");
+    }
+}
+
+
 
 #******************************************************************************
 #
@@ -754,24 +884,27 @@ my $prefix = getPrefix();
 
 $cpCommand = getCpCommand();
 
-installProgram($pkgdir, $prefix);
+installProgram($pkgdir, $prefix, \my $bindir);
 print("\n");
 
-installSharedLib($pkgdir, $prefix);
+installSharedLib($pkgdir, $prefix, \my $libdir);
 print("\n");
 
-installStaticLib($pkgdir, $prefix);
+installStaticLib($pkgdir, $prefix, \my $linkdir);
 print("\n");
 
-installDataFile($pkgdir, $prefix);
+installDataFile($pkgdir, $prefix, \my $datadir);
 print("\n");
 
-installHeader($pkgdir, $prefix);
+installHeader($pkgdir, $prefix, \my $includedir);
 print("\n");
 
-installManPage($pkgdir, $prefix);
+installManPage($pkgdir, $prefix, \my $mandir);
 print("\n");
 
+installConfig($pkgdir, 
+              $bindir, $libdir, $linkdir, $datadir, $includedir, $mandir);
+
 print("Installation is complete (except where previous error messages have\n");
 print("indicated otherwise).\n");
 
