@@ -2,11 +2,15 @@
                                  libopt
 ------------------------------------------------------------------------------
   This is a program to convert link library filepaths to linker options
-  that select them.  E.g. ../pbm/libpbm.so becomes -L../pbm -lpbm   .
+  that select them.  E.g. ../lib/libnetpbm.so becomes -L../lib -lnetpbm   .
 
   Each argument is a library filepath.  The option string to identify
   all of those library filepaths goes to Standard Output.
 
+  If there is no slash in the library filepath, we assume it is just a
+  filename to be searched for in the linker's default search path, and
+  generate a -l option, but no -L.
+
   If an argument doesn't make sense as a library filespec, it is
   copied verbatim, blank delimited, to the output string.
 
@@ -21,40 +25,49 @@
   arguments.  -R tells the buildtime linker to include the specified
   library in the search path that the runtime linker uses to find
   dynamically linked libraries.
+
+  But if you compile this program with the EXPLICIT macro defined, and
+  a library filepath contains a slash, then it skips all that -L/-l
+  nonsense and just outputs the input library filepath verbatim (plus
+  any -R option).
 ------------------------------------------------------------------------------
   Why would you want to use this?
 
   On some systems, the -L/-l output of this program has exactly the
   same effect as the filepath input when used in the arguments to a
   link command.  A GNU/Linux system, for example.  On others (Solaris,
-  for example), if you include /tmp/lib/libpbm.so in the link as a
+  for example), if you include /tmp/lib/libnetpbm.so in the link as a
   link object, the executable gets built in such a way that the system
-  accesses the shared library /tmp/lib/libpbm.so at run time.  On the
-  other hand, if you instead put the options -L/tmp/lib -lpbm on the
+  accesses the shared library /tmp/lib/libnetpbm.so at run time.  On the
+  other hand, if you instead put the options -L/tmp/lib -lnetpbm on the
   link command, the executable gets built so that the system accesses
-  libpbm.so in its actual installed directory at runtime (that
+  libnetpbm.so in its actual installed directory at runtime (that
   location might be determined by a --rpath linker option or a
   LD_LIBRARY_PATH environment variable at run time).
 
   In a make file, it is nice to use the same variable as the
-  dependency of rule that builds an executable and as the thing that
+  dependency of a rule that builds an executable and as the thing that
   the rule's command uses to identify its input.  Here is an example
   of using libopt for that:
 
-     PBMLIB=../pbm/libpbm.so
+     NETPBMLIB=../lib/libnetpbm.so
      ...
      pbmmake: pbmmake.o $(PBMLIB)
              ld -o pbmmake pbmmake.o `libopt $(PBMLIB)` --rpath=/lib/netpbm
 
-  Caveat: "-L../pbm -lpbm" is NOT exactly the same as
-  "../pbm/libpbm.so" on any system.  All of the -l libraries are
+  Caveat: "-L../lib -lnetpbm" is NOT exactly the same as
+  "../pbm/libnetpbm.so" on any system.  All of the -l libraries are
   searched for in all of the -L directories.  So you just might get a
   different library with the -L/-l version than if you specify the
   library file explicitly.
 
+  That's why you should compile with -DEXPLICIT if your linker can
+  handle explicit file names.
+
 -----------------------------------------------------------------------------*/
-#define _BSD_SOURCE 1
-    /* Make sure strdup() is in stdio.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in stdio.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #define MAX_PREFIXES 10
 
 /* Here's how to use SHLIBPREFIXLIST:  Use a -D compile option to pass in
@@ -62,7 +75,7 @@
 
    It's a blank-delimited list of prefixes that library names might
    have.  "lib" is traditionally the only prefix, as in libc or
-   libpbm.  However, on Windows there is a convention of using
+   libnetpbm.  However, on Windows there is a convention of using
    different prefixes to distinguish different co-existent versions of
    the same library (kind of like a major number in some unices). 
    E.g. the value "cyg lib" is appropriate for a Cygwin system.
@@ -76,8 +89,32 @@
 #include <stdio.h>
 
 typedef unsigned char bool;
+#ifndef TRUE
 #define TRUE (1)
+#endif
+#ifndef FALSE
 #define FALSE (0)
+#endif
+
+#ifdef DLLVERSTR
+static const char * dllverstr = DLLVERSTR;
+#else
+static const char * dllverstr = "";
+#endif
+
+bool const explicit = 
+#ifdef EXPLICIT
+TRUE
+#else
+FALSE
+#endif
+;
+
+static void
+strfree(const char * const arg) {
+    free((void*) arg);
+}
+
 
 static void
 parse_prefixlist(const char * const prefixlist, 
@@ -228,11 +265,11 @@ parse_prefix(const char * const filename,
 
 
 static void
-parse_filename(const char * const filename,
-               char ** const libname_p,
-               bool * const valid_library_p,
-               bool * const static_p,
-               bool * const error_p) {
+parse_filename(const char *  const filename,
+               const char ** const libname_p,
+               bool *        const valid_library_p,
+               bool *        const static_p,
+               bool *        const error_p) {
 /*----------------------------------------------------------------------------
    Extract the library name root component of the filename 'filename'.  This
    is just a filename, not a whole pathname.
@@ -241,16 +278,16 @@ parse_filename(const char * const filename,
    
    E.g. for "libxyz.so", return "xyz".
 
-   return *valid_library == TRUE iff 'filename' validly names a library
+   return *valid_library_p == TRUE iff 'filename' validly names a library
    that can be expressed in a -l linker option.
 
-   return *valid_library == TRUE iff 'filename' indicates a static library.
-   (but undefined if *valid_library != TRUE).
+   return *static_p == TRUE iff 'filename' indicates a static library.
+   (but undefined if *valid_library_p != TRUE).
 
    return *error_p == TRUE iff some error such as out of memory prevents
    parsing.
 
-   Do not allocate any memory if *error_p == TRUE or *valid_library == FALSE.
+   Do not allocate any memory if *error_p == TRUE or *valid_library_p == FALSE.
 -----------------------------------------------------------------------------*/
     char *lastdot;  
     /* Pointer to last period in 'filename'.  Null if none */
@@ -286,34 +323,39 @@ parse_filename(const char * const filename,
                 /* Extract everything between <prefix> and "." as 
                    the library name root. 
                 */
-                *libname_p = strdup(filename + prefix_length);
-                if (*libname_p == NULL)
+                char * libname;
+
+                libname = strdup(filename + prefix_length);
+                if (libname == NULL)
                     *error_p = TRUE;
                 else {
-                    (*libname_p)[lastdot - filename - prefix_length] = '\0';
-                
-#ifdef DLLVERSTR
-                    if (p = strstr(*libname_p,DLLVERSTR)) {
-                        if (*libname_p + strlen(*libname_p) - strlen(DLLVERSTR) == p) {
-                            *p = '\0';
+                    libname[lastdot - filename - prefix_length] = '\0';
+                    if (strlen(dllverstr) > 0) {
+                        p = strstr(libname, dllverstr);
+                        if (p) {
+                            if (libname + strlen(libname) 
+                                - strlen(dllverstr) == p) {
+                                *p = '\0';
+                            }
                         }
                     }
-#endif
-                    if (strlen(*libname_p) == 0) {
+                    if (strlen(libname) == 0) {
                         *valid_library_p = FALSE;
-                        free(*libname_p);
+                        strfree(libname);
                     } else
                         *valid_library_p = TRUE;
                 }
+                *libname_p = libname;
             }
         }
     }
 }   
 
 static void
-parse_filepath(const char * const filepath,
-               char ** const directory_p, char ** const filename_p,
-               bool * const error_p) {
+parse_filepath(const char *  const filepath,
+               const char ** const directory_p, 
+               const char ** const filename_p,
+               bool *        const error_p) {
 /*----------------------------------------------------------------------------
    Extract the directory and filename components of the filepath 
    'filepath' and return them in newly malloc'ed storage pointed to by
@@ -321,6 +363,7 @@ parse_filepath(const char * const filepath,
 
    If there is no directory component, return a null string for it.
 -----------------------------------------------------------------------------*/
+    char *directory;
     char *lastslash; /* Pointer to last slash in 'filepath', or null if none */
 
     lastslash = strrchr(filepath, '/');
@@ -333,10 +376,10 @@ parse_filepath(const char * const filepath,
         if (*filename_p == NULL)
             *error_p = TRUE;
         else {
-            *directory_p = strdup("");
-            if (*directory_p == NULL) {
+            directory = strdup("");
+            if (directory == NULL) {
                 *error_p = TRUE;
-                free(*filename_p);
+                strfree(*filename_p);
             } else
                 *error_p = FALSE;
         }
@@ -348,62 +391,102 @@ parse_filepath(const char * const filepath,
         if (*filename_p == NULL)
             *error_p = TRUE;
         else {
-            *directory_p = strdup(filepath);
-            if (*directory_p == NULL) {
+            directory = strdup(filepath);
+            if (directory == NULL) {
                 *error_p = TRUE;
-                free(*filename_p);
+                strfree(*filename_p);
             } else {
                 *error_p = FALSE;
-                (*directory_p)[lastslash - filepath] = '\0';
+                directory[lastslash - filepath] = '\0';
             }
         }
     }
+    *directory_p = directory;
 }
 
 
 
 static void
-processOneLibrary(const char * const filepath, 
-                  bool         const runtime, 
-                  bool *       const errorP) {
+doOptions(const char *  const filepath, 
+          const char *  const directory,
+          const char *  const libname,
+          bool          const runtime,
+          bool          const explicit,
+          bool          const staticlib,
+          const char ** const optionsP) {
+
+    char * options;
+    char * linkopt;
+
+    if (strlen(directory) == 0) {
+        linkopt = malloc(strlen(libname) + 10);
+        sprintf(linkopt, "-l%s", libname);
+    } else {
+        if (explicit)
+            linkopt = strdup(filepath);
+        else {
+            linkopt = malloc(strlen(directory) + strlen(libname) + 10);
+            sprintf(linkopt, "-L%s -l%s", directory, libname);
+        }
+    }
+    if (runtime && !staticlib && strlen(directory) > 0) {
+        options = malloc(strlen(linkopt) + strlen(directory) + 10);
+        sprintf(options, "%s -R%s", linkopt, directory); 
+    } else
+        options = strdup(linkopt);
+    
+    strfree(linkopt);
+    
+    *optionsP = options;
+}
+
+
+
+static void
+processOneLibrary(const char *  const filepath, 
+                  bool          const runtime, 
+                  bool          const explicit,
+                  const char ** const optionsP,
+                  bool *        const errorP) {
 /*----------------------------------------------------------------------------
-   Process the library with filepath 'filepath' and write the resulting
-   linker options to standard output.
+   Process the library with filepath 'filepath'.  Return the resulting
+   linker option string as a newly malloced null-terminated string at
+   *optionsP.
 -----------------------------------------------------------------------------*/
-    bool valid_library;  
-        /* Our argument is a valid library filepath that can be converted to
-           -l/-L notation.
-        */
-    bool staticlib;
-        /* Our argument appears to name a static library. */
-    char *directory = NULL;
-    /* Directory component of 'filepath' */
-    char *filename = NULL;
-    /* Filename component of 'filepath' */
-    char *libname = NULL;
-    /* Library name component of 'filename'.  e.g. xyz in libxyz.so */
+    unsigned int const MAX_OPT_SIZE = 1024;
+
+    const char *directory;
+        /* Directory component of 'filepath' */
+    const char *filename;
+        /* Filename component of 'filepath' */
 
     parse_filepath(filepath, &directory, &filename, errorP);
     if (!*errorP) {
-        parse_filename(filename, &libname, &valid_library, &staticlib, errorP);
+        const char *libname;
+            /* Library name component of 'filename'.  e.g. xyz in
+               libxyz.so 
+            */
+        bool valid_library;  
+            /* Our argument is a valid library filepath that can be
+               converted to -l/-L notation.  
+            */
+        bool staticlib;
+            /* Our argument appears to name a static library. */
 
+        parse_filename(filename, 
+                       &libname, &valid_library, &staticlib, errorP);
+        
         if (!*errorP) {
             if (valid_library) {
-                if (strlen(directory) == 0)
-                    fprintf(stdout, "-L. -l%s ", libname);
-                else {
-                    if (runtime && !staticlib)
-                        fprintf(stdout, "-L%s -l%s -R%s ", 
-                                directory, libname, directory);
-                    else
-                        fprintf(stdout, "-L%s -l%s ", directory, libname);
-                }
-                free(libname);
+                doOptions(filepath, directory, libname, 
+                          runtime, explicit, staticlib, optionsP);
+
+                strfree(libname);
             } else
-                fprintf(stdout, "%s ", filepath);
+                *optionsP = strdup(filepath);
         }
-        free(directory); 
-        free(filename);
+        strfree(directory); 
+        strfree(filename);
     }
 }
 
@@ -416,21 +499,42 @@ main(int argc, char **argv) {
     bool runtime;  /* -runtime option has been seen */
     int retval;
     unsigned int arg;  /* Index into argv[] of argument we're processing */
+    char outputLine[1024];
 
-    runtime = FALSE;
+    strcpy(outputLine, "");  /* initial value */
+    runtime = FALSE;  /* initial value */
     error = FALSE;  /* no error yet */
     for (arg = 1; arg < argc && !error; arg++) {
         if (strcmp(argv[arg], "-runtime") == 0)
             runtime = TRUE;
-        else 
-            processOneLibrary(argv[arg], runtime, &error);
+        else {
+            if (strlen(argv[arg]) > 200)
+                error = TRUE;
+            else {
+                const char * options;
+                processOneLibrary(argv[arg], runtime, explicit, 
+                                  &options, &error);
+                if (!error) {
+                    if (strlen(outputLine) + strlen(options) + 1 + 1 > 
+                        sizeof(outputLine))
+                        error = TRUE;
+                    else {
+                        strcat(outputLine, " ");
+                        strcat(outputLine, options);
+                    }
+                    strfree(options);
+                }
+            }
+        }
     }
     if (error) {
         fprintf(stderr, "serious libopt error prevented parsing library "
                 "names.  Invalid input to libopt is NOT the problem.\n");
         retval = 10;
-    } else 
+    } else {
+        fputs(outputLine, stdout);
+        fprintf(stderr, "libopt results: '%s'\n", outputLine);
         retval = 0;
-
+    }
     return retval;
 }
