@@ -138,6 +138,11 @@ struct gifScreen {
     unsigned int    Height;
     gifColorMap     ColorMap;
     unsigned int    ColorMapSize;
+        /* Base 2 logarithm of number of colors in the color map.
+           Note that the color map portion must be a power of two slots in 
+           size.  Each color takes 3 bytes (1 R, 1G, 1B), so 
+           e.g. 3 means 8 colors = 24 bytes of color map.
+        */
     unsigned int    ColorResolution;
     unsigned int    Background;
     unsigned int    AspectRatio;
@@ -877,6 +882,7 @@ readImageData(FILE * const ifP,
               int    const len, 
               int    const height, 
               gifColorMap  cmap, 
+              unsigned int const cmapSize,
               bool   const interlace,
               int    const transparentIndex,
               bit ** const alphabits) {
@@ -899,6 +905,11 @@ readImageData(FILE * const ifP,
                  "right after an image separator; no "
                  "image data follows.");
 
+    if (lzwMinCodeSize > MAX_LZW_BITS)
+        pm_error("Invalid minimum code size value in image data: %u.  "
+                 "Maximum allowable code size in GIF is %u", 
+                 lzwMinCodeSize, MAX_LZW_BITS);
+
     lzwInit(&decomp, ifP, lzwMinCodeSize);
 
     if (verbose)
@@ -906,6 +917,10 @@ readImageData(FILE * const ifP,
                    len, height, interlace ? " interlaced" : "" );
 
     while ((v = lzwReadByte(&decomp)) >= 0 ) {
+        if (v >= cmapSize)
+            pm_error("Invalid color index %u in an image that has only "
+                     "%u colors in the color map.", v, cmapSize);
+
         PPM_ASSIGN(xels[ypos][xpos], 
                    cmap[CM_RED][v], cmap[CM_GRN][v], cmap[CM_BLU][v]);
 
@@ -1185,7 +1200,7 @@ convertImage(FILE *           const ifP,
         readColorMap(ifP, localColorMapSize, localColorMap, 
                      &hasGray, &hasColor);
         transparencyMessage(gif89.transparent, localColorMap);
-        readImageData(ifP, xels, cols, rows, localColorMap, 
+        readImageData(ifP, xels, cols, rows, localColorMap, localColorMapSize,
                       BitSet(buf[8], INTERLACE), gif89.transparent,
                       alphabits);
         if (!skipIt) {
@@ -1194,7 +1209,8 @@ convertImage(FILE *           const ifP,
         }
     } else {
         transparencyMessage(gif89.transparent, gifScreen.ColorMap);
-        readImageData(ifP, xels, cols, rows, gifScreen.ColorMap, 
+        readImageData(ifP, xels, cols, rows,
+                      gifScreen.ColorMap, gifScreen.ColorMapSize,
                       BitSet(buf[8], INTERLACE), gif89.transparent,
                       alphabits);
         if (!skipIt) {
