@@ -48,20 +48,23 @@
     
 *****************************************************************************/
 
-#define _BSD_SOURCE 1
-    /* Make sure strdup() is in string.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
 #include <ctype.h>		/* to declare isprint() */
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
+#include <assert.h>
 /* Note: jpeglib.h prerequires stdlib.h and ctype.h.  It should include them
    itself, but doesn't.
 */
 #include <jpeglib.h>
 #include "pnm.h"
 #include "shhopt.h"
+#include "mallocvar.h"
+#include "nstring.h"
 #include "exif.h"
 #include "jpegdatasource.h"
 
@@ -97,15 +100,15 @@ struct cmdlineInfo {
         /* Filespec in which to save EXIF information.  NULL means don't
            save.  "-" means standard output
         */
-    int verbose;
-    int nosmooth;
+    unsigned int verbose;
+    unsigned int nosmooth;
     J_DCT_METHOD dct_method;
-    int grayscale;
     long int max_memory_to_use;
     unsigned int trace_level;
     enum inklevel inklevel;
-    int comments;
-    int dumpexif;
+    unsigned int comments;
+    unsigned int dumpexif;
+    unsigned int multiple;
 };
 
 
@@ -113,15 +116,16 @@ static bool displayComments;
     /* User wants comments from the JPEG to be displayed */
 
 static void 
-interpret_maxmemory (const char * const maxmemory, 
-                     long int * const max_memory_to_use_p) { 
+interpret_maxmemory (bool         const maxmemorySpec,
+                     const char * const maxmemory, 
+                     long int *   const max_memory_to_use_p) { 
 /*----------------------------------------------------------------------------
    Interpret the "maxmemory" command line option.
 -----------------------------------------------------------------------------*/
     long int lval;
     char ch;
     
-    if (maxmemory == NULL) {
+    if (!maxmemorySpec) {
         *max_memory_to_use_p = -1;  /* unspecified */
     } else if (sscanf(maxmemory, "%ld%c", &lval, &ch) < 1) {
         pm_error("Invalid value for --maxmemory option: '%s'.", maxmemory);
@@ -164,60 +168,63 @@ parse_command_line(const int argc, char ** argv,
    On the other hand, unlike other option processing functions, we do
    not change argv at all.
 -----------------------------------------------------------------------------*/
+    optEntry *option_def;
+        /* Instructions to optParseOptions3 on how to parse our options.
+         */
+    optStruct3 opt;
 
     int i;  /* local loop variable */
 
-    char *dctval;
     char *maxmemory;
-    int adobe, notadobe;
-
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions2 on how to parse our options.
-         */
-    optStruct2 opt;
+    char *dctval;
+    unsigned int adobe, notadobe;
 
+    unsigned int tracelevelSpec, exifSpec, dctvalSpec, maxmemorySpec;
     unsigned int option_def_index;
 
     int argc_parse;       /* argc, except we modify it as we parse */
-    char ** const argv_parse = malloc(argc*sizeof(char *));
+    char ** argv_parse;
+
+    MALLOCARRAY_NOFAIL(option_def, 100);
+    MALLOCARRAY_NOFAIL(argv_parse, argc);
+    
     /* argv, except we modify it as we parse */
 
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY(0, "verbose",     OPT_FLAG,   &cmdlineP->verbose,        0);
-    OPTENTRY(0, "dct",         OPT_STRING, &dctval,                    0);
-    OPTENTRY(0, "maxmemory",   OPT_STRING, &maxmemory,                 0); 
-    OPTENTRY(0, "nosmooth",    OPT_FLAG,   &cmdlineP->nosmooth,       0);
-    OPTENTRY(0, "tracelevel",  OPT_UINT,   &cmdlineP->trace_level,    0);
-    OPTENTRY(0, "adobe",       OPT_FLAG,   &adobe,                     0);
-    OPTENTRY(0, "notadobe",    OPT_FLAG,   &notadobe,                  0);
-    OPTENTRY(0, "comments",    OPT_FLAG,   &cmdlineP->comments,       0);
-    OPTENTRY(0, "exif",        OPT_STRING, &cmdlineP->exif_filespec,  0);
-    OPTENTRY(0, "dumpexif",    OPT_FLAG,   &cmdlineP->dumpexif,       0);
+    OPTENT3(0, "verbose",     OPT_FLAG,   NULL, &cmdlineP->verbose,       0);
+    OPTENT3(0, "dct",         OPT_STRING, &dctval,
+            &dctvalSpec, 0);
+    OPTENT3(0, "maxmemory",   OPT_STRING, &maxmemory,
+            &maxmemorySpec, 0); 
+    OPTENT3(0, "nosmooth",    OPT_FLAG,   NULL, &cmdlineP->nosmooth,      0);
+    OPTENT3(0, "tracelevel",  OPT_UINT,   &cmdlineP->trace_level,   
+            &tracelevelSpec, 0);
+    OPTENT3(0, "adobe",       OPT_FLAG,   NULL, &adobe,                   0);
+    OPTENT3(0, "notadobe",    OPT_FLAG,   NULL, &notadobe,                0);
+    OPTENT3(0, "comments",    OPT_FLAG,   NULL, &cmdlineP->comments,      0);
+    OPTENT3(0, "exif",        OPT_STRING, &cmdlineP->exif_filespec, 
+            &exifSpec, 0);
+    OPTENT3(0, "dumpexif",    OPT_FLAG,   NULL, &cmdlineP->dumpexif,      0);
+    OPTENT3(0, "multiple",    OPT_FLAG,   NULL, &cmdlineP->multiple,      0);
 
-    /* Set the defaults */
-    cmdlineP->verbose = FALSE;
-    cmdlineP->nosmooth = FALSE;
-    dctval = NULL;
-    maxmemory = NULL;
-    cmdlineP->trace_level = 0;
-    adobe = FALSE;
-    notadobe = FALSE;
-    cmdlineP->comments = FALSE;
-    cmdlineP->dumpexif = FALSE;
-    cmdlineP->exif_filespec = NULL;
-    
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We may have parms that are negative numbers */
 
-  /* Make private copy of arguments for optParseOptions to corrupt */
+    /* Make private copy of arguments for optParseOptions to corrupt */
     argc_parse = argc;
     for (i=0; i < argc; i++) argv_parse[i] = argv[i];
 
-    optParseOptions2(&argc_parse, argv_parse, opt, 0);
+    optParseOptions3( &argc_parse, argv_parse, opt, sizeof(opt), 0);
         /* Uses and sets argc_parse, argv_parse, 
            and some of *cmdlineP and others. */
 
+    if (!tracelevelSpec)
+        cmdlineP->trace_level = 0;
+
+    if (!exifSpec)
+        cmdlineP->exif_filespec = NULL;
+
     if (argc_parse - 1 == 0)
         cmdlineP->input_filespec = strdup("-");  /* he wants stdin */
     else if (argc_parse - 1 == 1)
@@ -226,7 +233,7 @@ parse_command_line(const int argc, char ** argv,
         pm_error("Too many arguments.  The only argument accepted "
                  "is the input file specificaton");
 
-    if (dctval == NULL)
+    if (!dctvalSpec)
         cmdlineP->dct_method = JDCT_DEFAULT;
     else {
         if (strcmp(dctval, "int") == 0)
@@ -238,7 +245,8 @@ parse_command_line(const int argc, char ** argv,
         else pm_error("Invalid value for the --dct option: '%s'.", dctval);
     }
 
-    interpret_maxmemory(maxmemory, &cmdlineP->max_memory_to_use);
+    interpret_maxmemory(maxmemorySpec, maxmemory, 
+                        &cmdlineP->max_memory_to_use);
 
     interpret_adobe(adobe, notadobe, &cmdlineP->inklevel);
 
@@ -637,11 +645,18 @@ print_exif_info(struct jpeg_marker_struct const marker) {
    marker 'marker', assuming it is an Exif header.
 -----------------------------------------------------------------------------*/
     ImageInfo_t imageInfo;
+    const char * error;
 
-    process_EXIF(marker.data, marker.data_length, &imageInfo, FALSE);
+    assert(marker.data_length >= 6);
 
-    ShowImageInfo(&imageInfo);
+    process_EXIF(marker.data+6, marker.data_length-6, 
+                 &imageInfo, FALSE, &error);
 
+    if (error) {
+        pm_message("EXIF header is invalid.  %s", error);
+        strfree(error);
+    } else
+        ShowImageInfo(&imageInfo);
 }
 
 
@@ -655,7 +670,7 @@ is_exif(struct jpeg_marker_struct const marker) {
     boolean retval;
     
     if (marker.marker == JPEG_APP0+1) {
-        if (memcmp(marker.data, "Exif", 4) == 0)
+        if (marker.data_length >=6 && memcmp(marker.data, "Exif", 4) == 0)
             retval = TRUE;
         else retval = FALSE;
     }
@@ -876,6 +891,29 @@ saveMarkers(struct jpeg_decompress_struct * const cinfoP) {
 
 
 
+static void
+convertImages(FILE *                          const ofP,
+              struct cmdlineInfo              const cmdline,
+              struct jpeg_decompress_struct * const cinfoP,
+              struct sourceManager *          const sourceManagerP) {
+              
+    if (cmdline.multiple) {
+        unsigned int imageSequence;
+        for (imageSequence = 0; dsDataLeft(sourceManagerP); ++imageSequence) {
+            if (cmdline.verbose)
+                pm_message("Reading Image %u", imageSequence);
+            convertImage(ofP, cmdline, cinfoP);
+        }
+    } else {
+        if (dsDataLeft(sourceManagerP))
+            convertImage(ofP, cmdline, cinfoP);
+        else
+            pm_error("Input stream is empty");
+    }
+}
+
+
+
 int
 main(int argc, char **argv) {
     FILE * ofP;
@@ -911,9 +949,8 @@ main(int argc, char **argv) {
 
     cinfo.src = dsJpegSourceMgr(sourceManagerP);
 
-    while (dsDataLeft(sourceManagerP)) {
-        convertImage(ofP, cmdline, &cinfo);
-    }
+    convertImages(ofP, cmdline, &cinfo, sourceManagerP);
+
     jpeg_destroy_decompress(&cinfo);
 
     if (ofP) {
