@@ -14,6 +14,7 @@
 */
 
 #define _BSD_SOURCE  /* Make sure string.h contains strdup() */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 #include <ctype.h>
 #include <string.h>
 #include "pam.h"
@@ -340,7 +341,7 @@ writeRaster(struct pam *  const pamP,
         printf("    '");
         for (col = 0; col < pamP->width; ++col) {
             int colorIndex;
-            bool found;
+            int found;
             if ((col * charspp) % 70 == 0 && col > 0)
                 printf( "\\\n" );       /* line continuation */
             pnm_lookuptuple(pamP, cht, tuples[row][col], &found, &colorIndex);
@@ -402,8 +403,7 @@ main(int argc, char *argv[]) {
     parseCommandLine(argc, argv, &cmdline);
 
     ifP = pm_openr(cmdline.inputFilespec);
-
-    tuples = pnm_readpam(ifP, &pam, sizeof(pam));
+    tuples = pnm_readpam(ifP, &pam, PAM_STRUCT_SIZE(tuple_type));
     pm_close(ifP);
 
     pm_message("computing colormap...");
