@@ -39,6 +39,9 @@
 #define VERSION "2.37.4 (5 December 1999) +netpbm"
 
 #include "pnm.h"
+#include "mallocvar.h"
+#include "nstring.h"
+#include "shhopt.h"
 
 typedef struct _jmpbuf_wrapper {
   jmp_buf jmpbuf;
@@ -62,11 +65,25 @@ typedef struct _jmpbuf_wrapper {
 #  define NONE 0
 #endif
 
+enum alpha_handling {none, alpha_only, mix};
+
+struct cmdlineInfo {
+    /* All the information the user supplied in the command line,
+       in a form easy for the program to use.
+    */
+    const char *inputFilespec;  /* '-' if stdin */
+    unsigned int verbose;
+    enum alpha_handling alpha;
+    const char * background;
+    float gamma;
+    const char * text;
+    unsigned int time;
+};
+
+
 /* function prototypes */
 #ifdef __STDC__
 static png_uint_16 _get_png_val (png_byte **pp, int bit_depth);
-static void store_pixel (xel *pix, png_uint_16 r, png_uint_16 g, png_uint_16 b,
-                         png_uint_16 a);
 static int iscolor (png_color c);
 static void save_text (png_info *info_ptr, FILE *tfp);
 static void show_time (png_info *info_ptr);
@@ -74,22 +91,94 @@ static void pngtopnm_error_handler (png_structp png_ptr, png_const_charp msg);
 int main (int argc, char *argv[]);
 #endif
 
-enum alpha_handling
-  {none, alpha_only, mix};
-
 static png_uint_16 maxval, maxmaxval;
 static png_uint_16 bgr, bgg, bgb; /* background colors */
 static int verbose = FALSE;
 static enum alpha_handling alpha = none;
-static int background = -1;
-static char *backstring;
-static float displaygamma = -1.0; /* display gamma */
+const char *background;
+static float displaygamma; /* display gamma */
 static float totalgamma = -1.0;
-static int text = FALSE;
-static char *text_file;
-static int mtime = FALSE;
+static int mtime;
 static jmpbuf_wrapper pngtopnm_jmpbuf_struct;
 
+
+static void
+parseCommandLine(int                 argc, 
+                 char **             argv,
+                 struct cmdlineInfo *cmdlineP ) {
+/*----------------------------------------------------------------------------
+   Parse program command line described in Unix standard form by argc
+   and argv.  Return the information in the options as *cmdlineP.  
+
+   If command line is internally inconsistent (invalid options, etc.),
+   issue error message to stderr and abort program.
+
+   Note that the strings we return are stored in the storage that
+   was passed to us as the argv array.  We also trash *argv.
+-----------------------------------------------------------------------------*/
+    optEntry *option_def = malloc(100*sizeof(optEntry));
+        /* Instructions to optParseOptions3 on how to parse our options.
+         */
+    optStruct3 opt;
+
+    unsigned int option_def_index;
+
+    unsigned int alphaSpec, mixSpec, backgroundSpec, gammaSpec, textSpec;
+
+    option_def_index = 0;   /* incremented by OPTENT3 */
+    OPTENT3(0, "verbose",     OPT_FLAG,   NULL,                  
+            &cmdlineP->verbose,       0);
+    OPTENT3(0, "alpha",       OPT_FLAG,   NULL,                  
+            &alphaSpec,               0);
+    OPTENT3(0, "mix",         OPT_FLAG,   NULL,                  
+            &mixSpec,                 0);
+    OPTENT3(0, "background",  OPT_STRING, &cmdlineP->background,
+            &backgroundSpec,          0);
+    OPTENT3(0, "gamma",       OPT_FLOAT,  &cmdlineP->gamma,
+            &gammaSpec,               0);
+    OPTENT3(0, "text",        OPT_STRING, &cmdlineP->text,
+            &textSpec,                0);
+    OPTENT3(0, "time",        OPT_FLAG,   NULL,                  
+            &cmdlineP->time,          0);
+
+    opt.opt_table = option_def;
+    opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
+    opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
+
+    optParseOptions3( &argc, argv, opt, sizeof(opt), 0);
+        /* Uses and sets argc, argv, and some of *cmdlineP and others. */
+
+
+    if (alphaSpec && mixSpec)
+        pm_error("You cannot specify both -alpha and -mix");
+    else if (alphaSpec)
+        cmdlineP->alpha = alpha_only;
+    else if (mixSpec)
+        cmdlineP->alpha = mix;
+    else
+        cmdlineP->alpha = none;
+
+    if (!backgroundSpec)
+        cmdlineP->background = NULL;
+
+    if (!gammaSpec)
+        cmdlineP->gamma = -1.0;
+
+    if (!textSpec)
+        cmdlineP->text = NULL;
+
+    if (argc-1 < 1)
+        cmdlineP->inputFilespec = "-";
+    else if (argc-1 == 1)
+        cmdlineP->inputFilespec = argv[1];
+    else
+        pm_error("Program takes at most one argument: input file name.  "
+            "you specified %d", argc-1);
+}
+
+
+
+
 #define get_png_val(p) _get_png_val (&(p), info_ptr->bit_depth)
 
 #ifdef __STDC__
@@ -113,24 +202,29 @@ int bit_depth;
   return c;
 }
 
-#ifdef __STDC__
-static void store_pixel (xel *pix, png_uint_16 r, png_uint_16 g, png_uint_16 b, png_uint_16 a)
-#else
-static void store_pixel (pix, r, g, b, a)
-xel *pix;
-png_uint_16 r, g, b, a;
-#endif
-{
-  if (alpha == alpha_only) {
-    PNM_ASSIGN1 (*pix, a);
-  } else {
-    if ((alpha == mix) && (a != maxval)) {
-      r = r * (double)a / maxval + ((1.0 - (double)a / maxval) * bgr);
-      g = g * (double)a / maxval + ((1.0 - (double)a / maxval) * bgg);
-      b = b * (double)a / maxval + ((1.0 - (double)a / maxval) * bgb);
+
+
+static void 
+setXel(xel * const xelP, 
+       png_uint_16 const r, 
+       png_uint_16 const g, 
+       png_uint_16 const b, 
+       png_uint_16 const a) {
+
+    if (alpha == alpha_only) {
+        PNM_ASSIGN1(*xelP, a);
+    } else {
+        if ((alpha == mix) && (a != maxval)) {
+            png_uint_16 const mixR = 
+                r * (double)a / maxval + ((1.0 - (double)a / maxval) * bgr);
+            png_uint_16 const mixG = 
+                g * (double)a / maxval + ((1.0 - (double)a / maxval) * bgg);
+            png_uint_16 const mixB = 
+                b * (double)a / maxval + ((1.0 - (double)a / maxval) * bgb);
+            PPM_ASSIGN(*xelP, mixR, mixG, mixB);
+        } else
+            PPM_ASSIGN(*xelP, r, g, b);
     }
-    PPM_ASSIGN (*pix, r, g, b);
-  }
 }
 
 #ifdef __STDC__
@@ -246,6 +340,214 @@ png_const_charp msg;
   longjmp(jmpbuf_ptr->jmpbuf, 1);
 }
 
+
+
+static void
+dump_png_info(png_info *info_ptr) {
+
+    const char *type_string;
+    const char *filter_string;
+
+    switch (info_ptr->color_type) {
+      case PNG_COLOR_TYPE_GRAY:
+        type_string = "gray";
+        break;
+
+      case PNG_COLOR_TYPE_GRAY_ALPHA:
+        type_string = "gray+alpha";
+        break;
+
+      case PNG_COLOR_TYPE_PALETTE:
+        type_string = "palette";
+        break;
+
+      case PNG_COLOR_TYPE_RGB:
+        type_string = "truecolor";
+        break;
+
+      case PNG_COLOR_TYPE_RGB_ALPHA:
+        type_string = "truecolor+alpha";
+        break;
+    }
+
+    switch (info_ptr->filter_type) {
+    case PNG_FILTER_TYPE_BASE:
+        asprintfN(&filter_string, "base filter");
+        break;
+    default:
+        asprintfN(&filter_string, "unknown filter type %d", 
+                  info_ptr->filter_type);
+    }
+
+    pm_message("reading a %ldw x %ldh image, %d bit%s",
+               info_ptr->width, info_ptr->height,
+               info_ptr->bit_depth, info_ptr->bit_depth > 1 ? "s" : "");
+    pm_message("%s, %s, %s",
+               type_string,
+               info_ptr->interlace_type ? 
+               "Adam7 interlaced" : "not interlaced",
+               filter_string);
+    pm_message("background {index, gray, red, green, blue} = "
+               "{%d, %d, %d, %d, %d}",
+               info_ptr->background.index,
+               info_ptr->background.gray,
+               info_ptr->background.red,
+               info_ptr->background.green,
+               info_ptr->background.blue);
+
+    strfree(filter_string);
+
+    if (info_ptr->valid & PNG_INFO_tRNS)
+        pm_message("tRNS chunk (transparency): present");
+    else
+        pm_message("tRNS chunk (transparency): not present");
+
+    if (info_ptr->valid & PNG_INFO_gAMA)
+        pm_message("gAMA chunk (image gamma): gamma = %4.2f", info_ptr->gamma);
+    else
+        pm_message("gAMA chunk (image gamma): not present");
+
+    if (info_ptr->valid & PNG_INFO_sBIT)
+        pm_message("sBIT chunk: present");
+    else
+        pm_message("sBIT chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_cHRM)
+        pm_message("cHRM chunk: present");
+    else
+        pm_message("cHRM chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_PLTE)
+        pm_message("PLTE chunk: present");
+    else
+        pm_message("PLTE chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_bKGD)
+        pm_message("bKGD chunk: present");
+    else
+        pm_message("bKGD chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_PLTE)
+        pm_message("hIST chunk: present");
+    else
+        pm_message("hIST chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_pHYs)
+        pm_message("pHYs chunk: present");
+    else
+        pm_message("pHYs chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_oFFs)
+        pm_message("oFFs chunk: present");
+    else
+        pm_message("oFFs chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_tIME)
+        pm_message("tIME chunk: present");
+    else
+        pm_message("tIME chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_pCAL)
+        pm_message("pCAL chunk: present");
+    else
+        pm_message("pCAL chunk: not present");
+
+    if (info_ptr->valid & PNG_INFO_sRGB)
+        pm_message("sRGB chunk: present");
+    else
+        pm_message("sRGB chunk: not present");
+}
+
+
+
+static void
+writePnm(FILE *      const ofP,
+         xelval      const maxval,
+         int         const pnm_type,
+         png_info *  const info_ptr,
+         png_byte ** const png_image) {
+
+    xel *xelrow;
+    int row;
+
+    if (verbose)
+        pm_message ("writing a %s file (maxval=%u)",
+                    pnm_type == PBM_TYPE ? "PBM" :
+                    pnm_type == PGM_TYPE ? "PGM" :
+                    pnm_type == PPM_TYPE ? "PPM" :
+                    "UNKNOWN!", 
+                    maxval);
+    
+    xelrow = pnm_allocrow(info_ptr->width);
+
+    pnm_writepnminit(stdout, info_ptr->width, info_ptr->height, maxval,
+                     pnm_type, FALSE);
+
+    for (row = 0; row < info_ptr->height; ++row) {
+        png_byte * png_pixelP;
+        int col;
+
+        png_pixelP = &png_image[row][0];  /* initial value */
+        for (col = 0; col < info_ptr->width; ++col) {
+            png_uint_16 const c = get_png_val(png_pixelP);
+            switch (info_ptr->color_type) {
+            case PNG_COLOR_TYPE_GRAY:
+                setXel(&xelrow[col], c, c, c,
+                       ((info_ptr->valid & PNG_INFO_tRNS) &&
+                        (c == gamma_correct(info_ptr->trans_values.gray,
+                                            totalgamma))) ?
+                       0 : maxval);
+                break;
+
+            case PNG_COLOR_TYPE_GRAY_ALPHA: {
+                png_uint_16 const a = get_png_val(png_pixelP);
+                setXel(&xelrow[col], c, c, c, a);
+            }
+            break;
+
+            case PNG_COLOR_TYPE_PALETTE:
+                setXel(&xelrow[col], info_ptr->palette[c].red,
+                       info_ptr->palette[c].green, 
+                       info_ptr->palette[c].blue,
+                       (info_ptr->valid & PNG_INFO_tRNS) &&
+                       c < info_ptr->num_trans ?
+                       info_ptr->trans[c] : maxval);
+                break;
+
+            case PNG_COLOR_TYPE_RGB: {
+                png_uint_16 const c2 = get_png_val(png_pixelP);
+                png_uint_16 const c3 = get_png_val(png_pixelP);
+                setXel(&xelrow[col], c, c2, c3,
+                       ((info_ptr->valid & PNG_INFO_tRNS) &&
+                        (c == gamma_correct(info_ptr->trans_values.red,
+                                            totalgamma)) &&
+                        (c2 == gamma_correct(info_ptr->trans_values.green,
+                                             totalgamma)) &&
+                        (c3 == gamma_correct(info_ptr->trans_values.blue,
+                                             totalgamma))) ?
+                       0 : maxval);
+            }
+                break;
+
+            case PNG_COLOR_TYPE_RGB_ALPHA: {
+                png_uint_16 const c2 = get_png_val(png_pixelP);
+                png_uint_16 const c3 = get_png_val(png_pixelP);
+                png_uint_16 const a = get_png_val(png_pixelP);
+                setXel(&xelrow[col], c, c2, c3, a);
+            }
+            break;
+
+            default:
+                pm_error ("unknown PNG color type: %d", info_ptr->color_type);
+            }
+        }
+        pnm_writepnmrow(ofP, xelrow, info_ptr->width, maxval, pnm_type, FALSE);
+    }
+    pnm_freerow (xelrow);
+}
+
+
+
 #define SIG_CHECK_SIZE 4
 
 static void 
@@ -254,28 +556,20 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
   unsigned char sig_buf [SIG_CHECK_SIZE];
   png_struct *png_ptr;
   png_info *info_ptr;
-  pixel *row;
   png_byte **png_image;
-  png_byte *png_pixel;
-  pixel *pnm_pixel;
   int x, y;
   int linesize;
-  png_uint_16 c, c2, c3, a;
   int pnm_type;
   int i;
   int trans_mix;
   pixel backcolor;
-  char gamma_string[80];
 
   /* these variables are declared static because gcc wasn't kidding
    * about "variable XXX might be clobbered by `longjmp' or `vfork'"
    * (stack corruption observed on Solaris 2.6 with gcc 2.8.1, even
    * in the absence of any other error condition) */
-  static const char *type_string;
-  static const char *alpha_string;
 
   *errorlevelP = 0;
-  type_string = alpha_string = "";
 
   if (fread (sig_buf, 1, SIG_CHECK_SIZE, ifp) != SIG_CHECK_SIZE) {
     pm_closer (ifp);
@@ -310,60 +604,7 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
   png_set_sig_bytes (png_ptr, SIG_CHECK_SIZE);
   png_read_info (png_ptr, info_ptr);
 
-  if (verbose) {
-    switch (info_ptr->color_type) {
-      case PNG_COLOR_TYPE_GRAY:
-        type_string = "gray";
-        alpha_string = "";
-        break;
-
-      case PNG_COLOR_TYPE_GRAY_ALPHA:
-        type_string = "gray";
-        alpha_string = "+alpha";
-        break;
-
-      case PNG_COLOR_TYPE_PALETTE:
-        type_string = "palette";
-        alpha_string = "";
-        break;
-
-      case PNG_COLOR_TYPE_RGB:
-        type_string = "truecolor";
-        alpha_string = "";
-        break;
-
-      case PNG_COLOR_TYPE_RGB_ALPHA:
-        type_string = "truecolor";
-        alpha_string = "+alpha";
-        break;
-    }
-    if (info_ptr->valid & PNG_INFO_tRNS) {
-      alpha_string = "+transparency";
-    }
-
-    if (info_ptr->valid & PNG_INFO_gAMA) {
-      sprintf (gamma_string, ", image gamma = %4.2f", info_ptr->gamma);
-    } else {
-      strcpy (gamma_string, "");
-    }
-
-    if (verbose) {
-      pm_message ("reading a %ldw x %ldh image, %d bit%s %s%s%s%s",
-		  info_ptr->width, info_ptr->height,
-		  info_ptr->bit_depth, info_ptr->bit_depth > 1 ? "s" : "",
-		  type_string, alpha_string, gamma_string,
-		  info_ptr->interlace_type ? ", Adam7 interlaced" : "");
-      pm_message ("background {index, gray, red, green, blue} = "
-                  "{%d, %d, %d, %d, %d}",
-                  info_ptr->background.index,
-                  info_ptr->background.gray,
-                  info_ptr->background.red,
-                  info_ptr->background.green,
-                  info_ptr->background.blue);
-    }
-  }
-
-  png_image = (png_byte **)malloc (info_ptr->height * sizeof (png_byte*));
+  MALLOCARRAY(png_image, info_ptr->height);
   if (png_image == NULL) {
     png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
     pm_closer (ifp);
@@ -477,7 +718,8 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
 	    }
 	    maxval = (1l << info_ptr->sig_bit.red) - 1;
 	    if (verbose)
-	      pm_message ("image has fewer significant bits, writing file with %d bits per channel", 
+	      pm_message ("image has fewer significant bits, "
+                      "writing file with %d bits per channel", 
 		info_ptr->sig_bit.red);
           } else
           if ((info_ptr->color_type == PNG_COLOR_TYPE_RGB ||
@@ -486,7 +728,8 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
 	    png_set_shift (png_ptr, &(info_ptr->sig_bit));
 	    maxval = (1l << info_ptr->sig_bit.red) - 1;
 	    if (verbose)
-	      pm_message ("image has fewer significant bits, writing file with %d bits per channel", 
+	      pm_message ("image has fewer significant bits, "
+                      "writing file with %d bits per channel", 
 		info_ptr->sig_bit.red);
           } else 
           if ((info_ptr->color_type == PNG_COLOR_TYPE_GRAY ||
@@ -495,7 +738,8 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
 	    png_set_shift (png_ptr, &(info_ptr->sig_bit));
 	    maxval = (1l << info_ptr->sig_bit.gray) - 1;
 	    if (verbose)
-	      pm_message ("image has fewer significant bits, writing file with %d bits",
+	      pm_message ("image has fewer significant bits, "
+                      "writing file with %d bits",
 		info_ptr->sig_bit.gray);
           }
         }
@@ -507,7 +751,8 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
 	    (info_ptr->sig_bit.gray < info_ptr->bit_depth)) {
 	  png_set_shift (png_ptr, &(info_ptr->sig_bit));
 	  if (verbose)
-	    pm_message ("image has fewer significant bits, writing file with %d bits", 
+	    pm_message ("image has fewer significant bits, "
+                    "writing file with %d bits", 
 		info_ptr->sig_bit.alpha);
 	  maxval = (1l << info_ptr->sig_bit.alpha) - 1;
         }
@@ -542,9 +787,9 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
 
   /* but if background was specified from the command-line, we always use that	*/
   /* I chose to do no gamma-correction in this case; which is a bit arbitrary	*/
-  if (background > -1)
+  if (background)
   {
-    backcolor = ppm_parsecolor (backstring, maxval);
+    backcolor = ppm_parsecolor (background, maxval);
     switch (info_ptr->color_type) {
       case PNG_COLOR_TYPE_GRAY:
       case PNG_COLOR_TYPE_GRAY_ALPHA:
@@ -563,9 +808,16 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
   png_read_image (png_ptr, png_image);
   png_read_end (png_ptr, info_ptr);
 
+  if (verbose)
+    /* Note that some of info_ptr is not defined until png_read_end() 
+       completes.  That's because it comes from chunks that are at the
+       end of the stream.
+    */
+    dump_png_info(info_ptr);
+
   if (mtime)
     show_time (info_ptr);
-  if (text)
+  if (tfp)
     save_text (info_ptr, tfp);
 
   if (info_ptr->valid & PNG_INFO_pHYs) {
@@ -580,15 +832,6 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
     }
   }
 
-  if ((row = pnm_allocrow (info_ptr->width)) == NULL) {
-    for (y = 0 ; y < info_ptr->height ; y++)
-      free (png_image[y]);
-    free (png_image);
-    png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-    pm_closer (ifp);
-    pm_error ("couldn't allocate space for image");
-  }
-
   if (alpha == alpha_only) {
     if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY ||
         info_ptr->color_type == PNG_COLOR_TYPE_RGB) {
@@ -638,77 +881,9 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
     maxmaxval = PPM_OVERALLMAXVAL;
   else maxmaxval = maxval;
 
-  if (verbose)
-    pm_message ("writing a %s file (maxval=%u)",
-                pnm_type == PBM_TYPE ? "PBM" :
-                pnm_type == PGM_TYPE ? "PGM" :
-                pnm_type == PPM_TYPE ? "PPM" :
-                                       "UNKNOWN!", 
-		maxmaxval);
-
-  pnm_writepnminit (stdout, info_ptr->width, info_ptr->height, maxval,
-                    pnm_type, FALSE);
-
-  for (y = 0 ; y < info_ptr->height ; y++) {
-    png_pixel = png_image[y];
-    pnm_pixel = row;
-    for (x = 0 ; x < info_ptr->width ; x++) {
-      c = get_png_val (png_pixel);
-      switch (info_ptr->color_type) {
-        case PNG_COLOR_TYPE_GRAY:
-          store_pixel (pnm_pixel, c, c, c,
-		((info_ptr->valid & PNG_INFO_tRNS) &&
-		 (c == gamma_correct (info_ptr->trans_values.gray, totalgamma))) ?
-			0 : maxval);
-          break;
-
-        case PNG_COLOR_TYPE_GRAY_ALPHA:
-          a = get_png_val (png_pixel);
-          store_pixel (pnm_pixel, c, c, c, a);
-          break;
-
-        case PNG_COLOR_TYPE_PALETTE:
-          store_pixel (pnm_pixel, info_ptr->palette[c].red,
-                       info_ptr->palette[c].green, info_ptr->palette[c].blue,
-                       (info_ptr->valid & PNG_INFO_tRNS) &&
-                        c<info_ptr->num_trans ?
-                        info_ptr->trans[c] : maxval);
-          break;
-
-        case PNG_COLOR_TYPE_RGB:
-          c2 = get_png_val (png_pixel);
-          c3 = get_png_val (png_pixel);
-          store_pixel (pnm_pixel, c, c2, c3,
-		((info_ptr->valid & PNG_INFO_tRNS) &&
-		 (c  == gamma_correct (info_ptr->trans_values.red, totalgamma)) &&
-		 (c2 == gamma_correct (info_ptr->trans_values.green, totalgamma)) &&
-		 (c3 == gamma_correct (info_ptr->trans_values.blue, totalgamma))) ?
-			0 : maxval);
-          break;
-
-        case PNG_COLOR_TYPE_RGB_ALPHA:
-          c2 = get_png_val (png_pixel);
-          c3 = get_png_val (png_pixel);
-          a = get_png_val (png_pixel);
-          store_pixel (pnm_pixel, c, c2, c3, a);
-          break;
-
-        default:
-          pnm_freerow (row);
-          for (i = 0 ; i < info_ptr->height ; i++)
-            free (png_image[i]);
-          free (png_image);
-          png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-          pm_closer (ifp);
-          pm_error ("unknown PNG color type");
-      }
-      pnm_pixel++;
-    }
-    pnm_writepnmrow (stdout, row, info_ptr->width, maxval, pnm_type, FALSE);
-  }
+  writePnm(stdout, maxval, pnm_type, info_ptr, png_image);
 
   fflush(stdout);
-  pnm_freerow (row);
   for (y = 0 ; y < info_ptr->height ; y++)
     free (png_image[y]);
   free (png_image);
@@ -717,78 +892,36 @@ convertpng (FILE * const ifp, FILE * const tfp, int * const errorlevelP)
 
 
 
-int main (int argc, char *argv[])
-{
-  FILE *ifp, *tfp;
-  int argn, errorlevel;
+int main(int argc, char *argv[]) {
+    struct cmdlineInfo cmdline;
+    FILE *ifp, *tfp;
+    int errorlevel;
 
-  const char * const usage = "[-verbose] [-alpha | -mix] [-background color] ...\n\
-             ... [-gamma value] [-text file] [-time] [pngfile]";
+    pnm_init (&argc, argv);
 
-  pnm_init (&argc, argv);
-  argn = 1;
+    parseCommandLine(argc, argv, &cmdline);
 
-  while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
-    if (pm_keymatch (argv[argn], "-verbose", 2)) {
-      verbose = TRUE;
-    } else
-    if (pm_keymatch (argv[argn], "-alpha", 2)) {
-      alpha = alpha_only;
-    } else
-    if (pm_keymatch (argv[argn], "-mix", 2)) {
-      alpha = mix;
-    } else
-    if (pm_keymatch (argv[argn], "-background", 2)) {
-      background = 1;
-      if (++argn < argc)
-        backstring = argv[argn];
-      else
-        pm_usage (usage);
-    } else
-    if (pm_keymatch (argv[argn], "-gamma", 2)) {
-      if (++argn < argc)
-        sscanf (argv[argn], "%f", &displaygamma);
-      else
-        pm_usage (usage);
-    } else
-    if (pm_keymatch (argv[argn], "-text", 3)) {
-      text = TRUE;
-      if (++argn < argc)
-        text_file = argv[argn];
-      else
-        pm_usage (usage);
-    } else
-    if (pm_keymatch (argv[argn], "-time", 3)) {
-      mtime = TRUE;
-    } else {
-      fprintf(stderr,"pngtopnm version %s, compiled with libpng version %s\n",
-        VERSION, PNG_LIBPNG_VER_STRING);
-      pm_usage (usage);
-    }
-    argn++;
-  }
+    verbose = cmdline.verbose;
+    alpha = cmdline.alpha;
+    background = cmdline.background;
+    displaygamma = cmdline.gamma;
+    mtime = cmdline.time;
 
-  if (argn != argc) {
-    ifp = pm_openr (argv[argn]);
-    ++argn;
-  } else {
-    ifp = stdin;
-  }
-  if (argn != argc)
-    pm_usage (usage);
+    ifp = pm_openr(cmdline.inputFilespec);
 
-  if (text)
-    tfp = pm_openw (text_file);
-  else
-    tfp = NULL;
+    if (cmdline.text)
+        tfp = pm_openw(cmdline.text);
+    else
+        tfp = NULL;
 
-  convertpng (ifp, tfp, &errorlevel);
+    convertpng (ifp, tfp, &errorlevel);
 
-  if (text)
-    pm_closew (tfp);
+    if (tfp)
+        pm_close(tfp);
 
-  pm_closer (ifp);
-  pm_closew (stdout);
+    pm_close(ifp);
+    pm_close(stdout);
 
-  return errorlevel;
+    return errorlevel;
 }
+
