@@ -16,14 +16,15 @@
 
 *****************************************************************************/
 
-#define _BSD_SOURCE 1
-    /* This makes sure strdup() is in string.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
 #include <ctype.h>		/* to declare isdigit(), etc. */
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <stdio.h>
+#include <assert.h>
 /* Note: jpeglib.h prerequires stdlib.h and ctype.h.  It should include them
    itself, but doesn't.
 */
@@ -537,18 +538,23 @@ write_exif_header(struct jpeg_compress_struct * const cinfoP,
    but don't recognize any error either.
 -----------------------------------------------------------------------------*/
     FILE * exif_file;
-    short length;
-
+    unsigned short length;
+    
     exif_file = pm_openr(exif_filespec);
 
-    pm_readbigshort(exif_file, &length);
+    pm_readbigshort(exif_file, (short*)&length);
 
     if (length == 0) {
         /* Special value meaning "no header" */
-    } else {
+    } else if (length < 3)
+        pm_error("Invalid length %u at start of exif file", length);
+    else {
         unsigned char * exif_data;
         int rc;
-        int const data_length = length - 2;  /* Subtract 2 byte length field*/
+        size_t const data_length = length - 2;  
+            /* Subtract 2 byte length field*/
+
+        assert(data_length > 0);
 
         exif_data = malloc(data_length);
         if (exif_data == NULL)
