@@ -61,6 +61,20 @@
 #include "pnm.h"
 #include "pngtxt.h"
 #include "mallocvar.h"
+#include "nstring.h"
+
+struct zlib_compression {
+    /* These are parameters that describe a form of zlib compression. 
+       In each of them, -1 means "don't care," and a nonnegative value is
+       as defined by zlib.
+    */
+    int level;
+    int mem_level;
+    int strategy;
+    int window_bits;
+    int method;
+    int buffer_size;
+};
 
 struct chroma {
     float wx;
@@ -86,14 +100,6 @@ typedef struct _jmpbuf_wrapper {
   jmp_buf jmpbuf;
 } jmpbuf_wrapper;
 
-/* GRR 19991205:  this is used as a test for pre-1999 versions of netpbm and
- *   pbmplus vs. 1999 or later (in which pm_close was split into two) 
- */
-#ifdef PBMPLUS_RAWBITS
-#  define pm_closer pm_close
-#  define pm_closew pm_close
-#endif
-
 #ifndef TRUE
 #  define TRUE 1
 #endif
@@ -124,6 +130,7 @@ static jmpbuf_wrapper pnmtopng_jmpbuf_struct;
 static int errorlevel;
 
 
+
 static png_color_16
 xelToPngColor_16(xel const input, 
                  xelval const maxval, 
@@ -157,15 +164,11 @@ closestColorInPalette(pixel          const targetColor,
 
     bestMatch = UINT_MAX;
     for (paletteIndex = 0; paletteIndex < paletteSize; ++paletteIndex) {
-        pixel const thisColor = palette_pnm[paletteIndex];
+        unsigned int const dist = 
+            PPM_DISTANCE(palette_pnm[paletteIndex], targetColor);
 
-        unsigned int match = 
-            SQR(PPM_GETR(thisColor) - PPM_GETR(targetColor)) +
-            SQR(PPM_GETG(thisColor) - PPM_GETG(targetColor)) +
-            SQR(PPM_GETB(thisColor) - PPM_GETG(targetColor));
-
-        if (match < bestMatch) {
-            bestMatch = match;
+        if (dist < bestMatch) {
+            bestMatch = dist;
             bestIndex = paletteIndex;
         }
     }
@@ -309,7 +312,7 @@ getChv(FILE *             const ifP,
        int                const format, 
        int                const maxColors, 
        colorhist_vector * const chvP,
-       int *              const colorsP) {
+       unsigned int *     const colorsP) {
 /*----------------------------------------------------------------------------
    Return a list of all the colors in a libpnm image and the number of
    times they occur.  The image is in the seekable file 'ifP', whose
@@ -591,22 +594,23 @@ alpha_trans(FILE *     const ifp,
 
 
 static void
-findRedundantBits(FILE * const ifp, 
-                  int const imagepos, 
-                  int const cols,
-                  int const rows,
-                  xelval const maxval,
-                  int const format,
-                  bool const alpha,
-                  bool const force,
+findRedundantBits(FILE *         const ifp, 
+                  int            const imagepos, 
+                  int            const cols,
+                  int            const rows,
+                  xelval         const maxval,
+                  int            const format,
+                  bool           const alpha,
+                  bool           const force,
                   unsigned int * const meaningfulBitsP) {
 /*----------------------------------------------------------------------------
-   Find out if we can use just a subset of the bits from each input sample.
-   Often, people create an image with e.g. 8 bit samples from one that has
-   e.g. only 4 bit samples by just repeating the bits.  1011 becomes
-   10111011.  We detect this case.  We return as *meaningfulBitsP the
-   minimum number of bits, starting from the least significant end, that
-   contain original information.
+   Find out if we can use just a subset of the bits from each input
+   sample.  Often, people create an image with e.g. 8 bit samples from
+   one that has e.g. only 4 bit samples by scaling by 256/16, which is
+   the same as repeating the bits.  E.g.  1011 becomes 10111011.  We
+   detect this case.  We return as *meaningfulBitsP the minimum number
+   of bits, starting from the least significant end, that contain
+   original information.
 -----------------------------------------------------------------------------*/
   if (!alpha && PNM_FORMAT_TYPE(format) == PGM_TYPE && !force) 
       meaningful_bits_pgm(ifp, imagepos, cols, rows, maxval, format,
@@ -619,7 +623,7 @@ findRedundantBits(FILE * const ifp,
 
   if (verbose && *meaningfulBitsP != pm_maxvaltobits(maxval))
       pm_message("Using only %d rightmost bits of input samples.  The "
-                 "rest are just copies of those.", *meaningfulBitsP);
+                 "rest are redundant.", *meaningfulBitsP);
 }
 
 
@@ -678,7 +682,7 @@ compute_nonalpha_palette(colorhist_vector const chv,
    (consisting of 'colors' distinct colors) assuming a pure-color (no
    transparency) palette.
 
-   If 'php' is non-null, assume it's a PPM file and read the palette
+   If 'pfp' is non-null, assume it's a PPM file and read the palette
    from that.  Make sure it contains the same colors as the palette
    we computed ourself would have.  Caller supplied the file because he
    wants the colors in a particular order in the palette.
@@ -906,7 +910,7 @@ compute_alpha_palette(FILE *         const ifP,
    don't return any palette information -- just return *tooBigP == TRUE.
 -----------------------------------------------------------------------------*/
     colorhist_vector chv;
-    int colors;
+    unsigned int colors;
 
     gray *alphas_of_color[MAXPALETTEENTRIES];
     unsigned int alphas_first_index[MAXPALETTEENTRIES];
@@ -975,10 +979,7 @@ makeOneColorTransparentInPalette(xel            const transColor,
                                  unsigned int * const transSizeP) {
 /*----------------------------------------------------------------------------
    Find the color 'transColor' in the color/alpha palette defined by
-   palette_pnm[], paletteSize, trans_pnm[] and *transSizeP.  If it's
-   not there and exact == TRUE, return without changing anything, but
-   issue a warning message.  If it's not there and exact == FALSE,
-   just find the closest color.
+   palette_pnm[], paletteSize, trans_pnm[] and *transSizeP.  
 
    Make that entry fully transparent.
 
@@ -986,6 +987,10 @@ makeOneColorTransparentInPalette(xel            const transColor,
    can do a better job when the opaque entries are all last in the 
    color/alpha palette).
 
+   If the specified color is not there and exact == TRUE, return
+   without changing anything, but issue a warning message.  If it's
+   not there and exact == FALSE, just find the closest color.
+
    We assume every entry in the palette is opaque upon entry.
 -----------------------------------------------------------------------------*/
     unsigned int transparentIndex;
@@ -1041,6 +1046,8 @@ findOrAddBackgroundInPalette(pixel          const backColor,
   Note that PNG specs say that transparency of the background is meaningless;
   i.e. a viewer must ignore the transparency of the palette entry when 
   using the background color.
+
+  Return the palette index of the background color as *backgroundIndexP.
 -----------------------------------------------------------------------------*/
     int backgroundIndex;  /* negative means not found */
     unsigned int paletteIndex;
@@ -1145,6 +1152,240 @@ buildColorAlphaLookup(pixel              palette_pnm[],
 
 
 
+static void
+tryAlphaPalette(FILE *         const ifP,
+                int            const cols,
+                int            const rows,
+                xelval         const maxval,
+                int            const format,
+                pm_filepos     const imagepos,
+                gray **        const alpha_mask,
+                FILE *         const pfP,
+                pixel *        const palette_pnm,
+                unsigned int * const paletteSizeP,
+                gray *         const trans_pnm,
+                unsigned int * const transSizeP,
+                const char **  const impossibleReasonP) {
+    
+    bool tooBig;
+    if (pfP)
+        pm_error("This program is not capable of generating "
+                 "a PNG with transparency when you specify "
+                 "the palette with -palette.");
+
+    compute_alpha_palette(ifP, cols, rows, maxval, format, 
+                          imagepos,  alpha_mask, palette_pnm, trans_pnm, 
+                          paletteSizeP, transSizeP, &tooBig);
+    if (tooBig) {
+        asprintfN(impossibleReasonP,
+                  "too many color/transparency pairs "
+                  "(more than the PNG maximum of %u", 
+                  MAXPALETTEENTRIES);
+    } else
+        *impossibleReasonP = NULL;
+} 
+
+
+
+static void
+computePixelWidth(int            const pnm_type,
+                  unsigned int   const pnm_meaningful_bits,
+                  bool           const alpha,
+                  unsigned int * const bitsPerSampleP,
+                  unsigned int * const bitsPerPixelP) {
+
+    unsigned int bitsPerSample, bitsPerPixel;
+
+    if (pnm_type == PPM_TYPE || alpha) {
+        /* PNG allows only depths of 8 and 16 for a truecolor image 
+           and for a grayscale image with an alpha channel.
+          */
+        if (pnm_meaningful_bits > 8)
+            bitsPerSample = 16;
+        else 
+            bitsPerSample = 8;
+    } else {
+        /* A grayscale, non-colormapped, no-alpha PNG may have any 
+             bit depth from 1 to 16
+          */
+        if (pnm_meaningful_bits > 8)
+            bitsPerSample = 16;
+        else if (pnm_meaningful_bits > 4)
+            bitsPerSample = 8;
+        else if (pnm_meaningful_bits > 2)
+            bitsPerSample = 4;
+        else if (pnm_meaningful_bits > 1)
+            bitsPerSample = 2;
+        else
+            bitsPerSample = 1;
+    }
+    if (alpha) {
+        if (pnm_type == PPM_TYPE)
+            bitsPerPixel = 4 * bitsPerSample;
+        else
+            bitsPerPixel = 2 * bitsPerSample;
+    } else {
+        if (pnm_type == PPM_TYPE)
+            bitsPerPixel = 3 * bitsPerSample;
+        else
+            bitsPerPixel = bitsPerSample;
+    }
+    if (bitsPerPixelP)
+        *bitsPerPixelP = bitsPerPixel;
+    if (bitsPerSampleP)
+        *bitsPerSampleP = bitsPerSample;
+}
+
+
+
+static unsigned int
+paletteIndexBits(unsigned int const nColors) {
+/*----------------------------------------------------------------------------
+  Return the number of bits that a palette index in the PNG will
+  occupy given that the palette has 'nColors' colors in it.  It is 1,
+  2, 4, or 8 bits.
+  
+  If 'nColors' is not a valid PNG palette size, return 0.
+-----------------------------------------------------------------------------*/
+    unsigned int retval;
+
+    if (nColors < 1)
+        retval = 0;
+    else if (nColors <= 2)
+        retval = 1;
+    else if (nColors <= 4)
+        retval = 2;
+    else if (nColors <= 16)
+        retval = 4;
+    else if (nColors <= 256)
+        retval = 8;
+    else
+        retval = 0;
+
+    return retval;
+}
+
+
+
+static void
+computeColorMap(FILE *         const ifP,
+                pm_filepos     const imagepos,
+                int            const cols,
+                int            const rows,
+                xelval         const maxval,
+                int            const format,
+                bool           const force,
+                FILE *         const pfP,
+                bool           const alpha,
+                int            const transparent,
+                pixel          const transcolor,
+                bool           const transexact,
+                int            const background,
+                pixel          const backcolor,
+                gray **        const alpha_mask,
+                unsigned int   const pnm_meaningful_bits,
+                /* Outputs */
+                pixel *        const palette_pnm,
+                unsigned int * const paletteSizeP,
+                gray *         const trans_pnm,
+                unsigned int * const transSizeP,
+                unsigned int * const backgroundIndexP,
+                const char **  const noColormapReasonP) {
+/*---------------------------------------------------------------------------
+  Determine whether to do a colormapped or truecolor PNG and if
+  colormapped, compute the full PNG palette -- both color and
+  transparency.
+
+  If we decide to do truecolor, we return as *noColormapReasonP a text
+  description of why, in newly malloced memory.  If we decide to go
+  with colormapped, we return *noColormapReasonP == NULL.
+
+  In the colormapped case, we return the palette as arrays
+  palette_pnm[] and trans_pnm[], allocated by Caller, with sizes
+  *paletteSizeP and *transSizeP.
+
+  If the image is to have a background color, we return the palette index
+  of that color as *backgroundIndexP.
+-------------------------------------------------------------------------- */
+    if (force)
+        asprintfN(noColormapReasonP, "You requested no color map");
+    else if (maxval > PALETTEMAXVAL)
+        asprintfN(noColormapReasonP, "The maxval of the input image (%u) "
+                  "exceeds the PNG palette maxval (%u)", 
+                  maxval, PALETTEMAXVAL);
+    else {
+        unsigned int bitsPerPixel;
+        computePixelWidth(PNM_FORMAT_TYPE(format), pnm_meaningful_bits, alpha,
+                          NULL, &bitsPerPixel);
+
+        if (!pfP && bitsPerPixel == 1)
+            /* No palette can beat 1 bit per pixel -- no need to waste time
+               counting the colors.
+            */
+            asprintfN(noColormapReasonP, "pixel is already only 1 bit");
+        else {
+            /* We'll have to count the colors ('colors') to know if a
+               palette is possible and desirable.  Along the way, we'll
+               compute the actual set of colors (chv) too, and then create
+               the palette itself if we decide we want one.
+            */
+            colorhist_vector chv;
+            unsigned int colors;
+            
+            getChv(ifP, imagepos, cols, rows, maxval, format, MAXCOLORS, 
+                   &chv, &colors);
+
+            if (chv == NULL) {
+                asprintfN(noColormapReasonP, 
+                          "More than %u colors found -- too many for a "
+                          "colormapped PNG", MAXCOLORS);
+            } else {
+                /* There are few enough colors that a palette is possible */
+                if (bitsPerPixel <= paletteIndexBits(colors) && !pfP)
+                    asprintfN(noColormapReasonP, 
+                              "palette index for %u colors would be "
+                              "no smaller than the indexed value (%u bits)", 
+                              colors, bitsPerPixel);
+                else {
+                    unsigned int paletteSize;
+                    unsigned int transSize;
+                    if (alpha)
+                        tryAlphaPalette(ifP, cols, rows, maxval, format,
+                                        imagepos, alpha_mask, pfP,
+                                        palette_pnm, &paletteSize, 
+                                        trans_pnm, &transSize,
+                                        noColormapReasonP);
+
+                    else {
+                        *noColormapReasonP = NULL;
+
+                        compute_nonalpha_palette(chv, colors, maxval, pfP,
+                                                 palette_pnm, &paletteSize, 
+                                                 trans_pnm, &transSize);
+    
+                        if (transparent != -1)
+                            makeOneColorTransparentInPalette(
+                                transcolor, transexact, 
+                                palette_pnm, paletteSize, trans_pnm, 
+                                &transSize);
+                    }
+                    if (!*noColormapReasonP) {
+                        if (background > -1)
+                            findOrAddBackgroundInPalette(
+                                backcolor, palette_pnm, &paletteSize,
+                                backgroundIndexP);
+                        *paletteSizeP = paletteSize;
+                        *transSizeP   = transSize;
+                    }
+                }
+            }
+            freeChv();
+        }
+    }
+}
+
+
+
 static void computeColorMapLookupTable(
     bool                   const colorMapped,
     pixel                        palette_pnm[],
@@ -1185,19 +1426,20 @@ static void computeColorMapLookupTable(
 
 
 static void
-computePixelWidth(bool           const colorMapped,
-                  unsigned int   const palette_size,
-                  int            const pnm_type,
-                  unsigned int   const pnm_meaningful_bits,
-                  bool           const alpha,
-                  unsigned int * const bitsPerSampleP,
-                  unsigned int * const bitsPerPixelP) {
+computeRasterWidth(bool           const colorMapped,
+                   unsigned int   const palette_size,
+                   int            const pnm_type,
+                   unsigned int   const pnm_meaningful_bits,
+                   bool           const alpha,
+                   unsigned int * const bitsPerSampleP,
+                   unsigned int * const bitsPerPixelP) {
 /*----------------------------------------------------------------------------
    Compute the number of bits per raster sample and per raster pixel:
    *bitsPerSampleP and *bitsPerPixelP.  Note that a raster element may be a
    palette index, or a gray value or color with or without alpha mask.
 -----------------------------------------------------------------------------*/
     if (colorMapped) {
+        /* The raster element is a palette index */
         if (palette_size <= 2)
             *bitsPerSampleP = 1;
         else if (palette_size <= 4)
@@ -1210,40 +1452,10 @@ computePixelWidth(bool           const colorMapped,
         if (verbose)
             pm_message("Writing %d-bit color indexes", *bitsPerSampleP);
     } else {
-        if (pnm_type == PPM_TYPE || alpha) {
-            /* PNG allows only depths of 8 and 16 for a truecolor image 
-               and for a grayscale image with an alpha channel.
-          */
-            if (pnm_meaningful_bits > 8)
-                *bitsPerSampleP = 16;
-            else 
-                *bitsPerSampleP = 8;
-        } else {
-            /* A grayscale, non-colormapped, no-alpha PNG may have any 
-             bit depth from 1 to 16
-          */
-            if (pnm_meaningful_bits > 8)
-                *bitsPerSampleP = 16;
-            else if (pnm_meaningful_bits > 4)
-                *bitsPerSampleP = 8;
-            else if (pnm_meaningful_bits > 2)
-                *bitsPerSampleP = 4;
-            else if (pnm_meaningful_bits > 1)
-                *bitsPerSampleP = 2;
-            else
-                *bitsPerSampleP = 1;
-        }
-        if (alpha) {
-            if (pnm_type == PPM_TYPE)
-                *bitsPerPixelP = 4 * *bitsPerSampleP;
-            else
-                *bitsPerPixelP = 2 * *bitsPerSampleP;
-        } else {
-            if (pnm_type == PPM_TYPE)
-                *bitsPerPixelP = 3 * *bitsPerSampleP;
-            else
-                *bitsPerPixelP = *bitsPerSampleP;
-        }
+        /* The raster element is an explicit pixel -- color and transparency */
+        computePixelWidth(pnm_type, pnm_meaningful_bits, alpha,
+                          bitsPerSampleP, bitsPerPixelP);
+
         if (verbose)
             pm_message("Writing %d bits per component per pixel", 
                        *bitsPerSampleP);
@@ -1317,9 +1529,9 @@ convertpnm(FILE * const ifp,
            int    const mtime,
            char * const date_string,
            char * const time_string,
-           int    const filter,
-           int    const compression,
-           int    const force
+           int    const filterSet,
+           int    const force,
+           struct zlib_compression zlib_compression
     ) {
 
   xel p;
@@ -1378,10 +1590,12 @@ convertpnm(FILE * const ifp,
   int alpha_rows;
   int alpha_cols;
   int alpha_can_be_transparency_index;
-
-  int colors;
+  const char * noColormapReason;
+      /* The reason that we shouldn't make a colormapped PNG, or NULL if
+         we should.  malloc'ed null-terminated string.
+      */
   unsigned int depth;
-      /* The number of bits per pixel in the (uncompressed) png 
+      /* The number of bits per sample in the (uncompressed) png 
          raster -- if the raster contains palette indices, this is the
          number of bits in the index.
       */
@@ -1539,92 +1753,32 @@ convertpnm(FILE * const ifp,
   findRedundantBits(ifp, imagepos, cols, rows, maxval, format, alpha,
                     force, &pnm_meaningful_bits);
   
-  /* -----------------------------------------------------------------
-     Determine whether to do a colormapped or truecolor PNG (variable
-     'colorMapped'), and if colormapped, compute the full PNG palette
-     -- both color and transparency -- as the arrays palette_pnm[]
-     and trans_pnm[], with sizes palette_size and trans_size.
-  -------------------------------------------------------------------- */
+  computeColorMap(ifp, imagepos, cols, rows, maxval, format,
+                  force, pfp, alpha, transparent, transcolor, transexact, 
+                  background, backcolor,
+                  alpha_mask, pnm_meaningful_bits,
+                  palette_pnm, &palette_size, trans_pnm, &trans_size,
+                  &background_index, &noColormapReason);
 
-  if (force)
+  if (noColormapReason) {
+      if (pfp)
+          pm_error("You specified a particular palette, but this image "
+                   "cannot be represented by any palette.  %s",
+                   noColormapReason);
+      if (verbose)
+          pm_message("Not using color map.  %s", noColormapReason);
+      strfree(noColormapReason);
       colorMapped = FALSE;
-  else if (pnm_type == PBM_TYPE)
-      colorMapped = FALSE;
-  else if (maxval > PALETTEMAXVAL)
-      colorMapped = FALSE;
-  else {
-      colorhist_vector chv;
-
-      getChv(ifp, imagepos, cols, rows, maxval, format, MAXCOLORS, 
-             &chv, &colors);
-
-      if (chv == NULL) {
-          if (pfp)
-              pm_error("You specified -palette, but there are more than %d "
-                       "colors in the "
-                       "image -- too many for a colormapped PNG", MAXCOLORS);
-          else {
-              colorMapped = FALSE;
-              if (verbose)
-                  pm_message("More than %d colors found -- too many for a "
-                             "colormapped PNG", MAXCOLORS);
-          }
-      } else {
-          if (!pfp && ((pnm_type == PGM_TYPE && pm_maxvaltobits(colors-1) > 
-               (pm_maxvaltobits(maxval) / 2)) ||
-              (pnm_type == PPM_TYPE && maxval > 255))) {
-              
-              colorMapped = FALSE;
-              if (verbose) 
-                  pm_message ("palette index would be larger than the "
-                              "indexed value, so not doing colormap");
-          } else {
-              if (alpha) {
-                  bool tooBig;
-                  if (pfp)
-                      pm_error("This program is not capable of generating "
-                               "a PNG with transparency when you specify "
-                               "the palette with -palette.");
-                  compute_alpha_palette(ifp, cols, rows, maxval, format, 
-                                        imagepos,  alpha_mask,
-                                        palette_pnm, trans_pnm, 
-                                        &palette_size, &trans_size,
-                                        &tooBig);
-                  if (tooBig) {
-                      colorMapped = FALSE;
-                      if (verbose)
-                          pm_message("too many color/transparency pairs, "
-                                     "writing a non-mapped file");
-                  } else
-                      colorMapped = TRUE;
-              } else {
-                  colorMapped = TRUE;
-                  compute_nonalpha_palette(chv, colors, maxval, pfp,
-                                           palette_pnm, &palette_size, 
-                                           trans_pnm, &trans_size);
-
-                  if (transparent != -1)
-                      makeOneColorTransparentInPalette(
-                          transcolor, transexact, 
-                          palette_pnm, palette_size, trans_pnm, &trans_size);
-              }
-              
-              if (background > -1) {
-                  findOrAddBackgroundInPalette(
-                      backcolor, palette_pnm, &palette_size, 
-                      &background_index);
-              }
-          }
-      }
-  }
+  } else
+      colorMapped = TRUE;
   
   computeColorMapLookupTable(colorMapped, palette_pnm, palette_size,
                              trans_pnm, trans_size, alpha, alpha_maxval,
                              &cht, &caht);
 
-  computePixelWidth(colorMapped, palette_size, pnm_type, 
-                    pnm_meaningful_bits, alpha,
-                    &depth, &fulldepth);
+  computeRasterWidth(colorMapped, palette_size, pnm_type, 
+                     pnm_meaningful_bits, alpha,
+                     &depth, &fulldepth);
   if (verbose)
     pm_message ("writing a%s %d-bit %s%s file%s",
                 fulldepth == 8 ? "n" : "", fulldepth,
@@ -1705,7 +1859,7 @@ convertpnm(FILE * const ifp,
     /* creating hIST chunk */
     if (hist) {
         colorhist_vector chv;
-        int colors;
+        unsigned int colors;
         colorhash_table cht;
         
         getChv(ifp, imagepos, cols, rows, maxval, format, MAXCOLORS, 
@@ -1757,8 +1911,6 @@ convertpnm(FILE * const ifp,
     }
   }
 
-  freeChv();
-
   /* bKGD chunk */
   if (background > -1) {
       info_ptr->valid |= PNG_INFO_bKGD;
@@ -1830,15 +1982,15 @@ convertpnm(FILE * const ifp,
   }
 
   /* explicit filter-type (or none) required */
-  if ((filter >= 0) && (filter <= 4))
+  if (filterSet != 0)
   {
-    png_set_filter (png_ptr, 0, filter);
+    png_set_filter (png_ptr, 0, filterSet);
   }
 
   /* zlib compression-level (or none) required */
-  if ((compression >= 0) && (compression <= 9))
+  if (zlib_compression.level >= 0)
   {
-    png_set_compression_level (png_ptr, compression);
+    png_set_compression_level (png_ptr, zlib_compression.level);
   }
 
   /* write the png-info struct */
@@ -1966,7 +2118,7 @@ displayVersion() {
        compiled for a particular version just to find out what
        version it is! It's not worth having a link failure, much
        less a compile failure, if we choose wrong.
-       png_get_header_ver is no in anything older than libpng 1.0.2a
+       png_get_header_ver is not in anything older than libpng 1.0.2a
        (Dec 1998).  png_libpng_ver is not there in libraries built
        without USE_GLOBAL_ARRAYS.  Cygwin versions are normally built
        without USE_GLOBAL_ARRAYS.  -bjh 2002.06.17.
@@ -2007,24 +2159,23 @@ main(int argc, char *argv[]) {
   int mtime = FALSE;
   char *date_string;
   char *time_string;
-  int filter = -1;
-  int compression = -1;
   int force = FALSE;
+  int filterSet = 0;
+  struct zlib_compression zlib_compression;
 
   FILE *ifp, *tfp, *afp, *pfp;
   char *palette_file;
       /* Name of user-supplied palette, file; NULL if none */
   int argn, errorlevel=0;
 
-  const char * const usage = "[-verbose] [-downscale] [-interlace] [-force] [-alpha file] ...\n\
-             ... [-transparent color] [-background color] [-gamma value] ...\n\
-             ... [-hist] [-chroma wx wy rx ry gx gy bx by] [-phys x y unit] ...\n\
-             ... [-text file] [-ztxt file] [-time [yy]yy-mm-dd hh:mm:ss] ...\n\
-             ... [-palette file] [-filter 0..4] [-compression 0..9] [pnmfile]";
-
   pnm_init (&argc, argv);
   argn = 1;
 
+  zlib_compression.level = -1;  /* initial value */
+  /* TODO: switch this to shhopt, and make options for the rest of
+     zlib_compression.
+  */
+  
   palette_file = NULL;   /* Initial value */
 
   while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
@@ -2044,7 +2195,7 @@ main(int argc, char *argv[]) {
       if (++argn < argc)
         alpha_file = argv[argn];
       else
-        pm_usage (usage);
+        pm_error("-alpha requires a value");
     } else
     if (pm_keymatch (argv[argn], "-transparent", 3)) {
       if (alpha)
@@ -2053,20 +2204,20 @@ main(int argc, char *argv[]) {
       if (++argn < argc)
         transstring = argv[argn];
       else
-        pm_usage (usage);
+        pm_error("-transparent requires a value");
     } else
     if (pm_keymatch (argv[argn], "-background", 2)) {
       background = 1;
       if (++argn < argc)
         backstring = argv[argn];
       else
-        pm_usage (usage);
+        pm_error("-background requires a value");
     } else
     if (pm_keymatch (argv[argn], "-gamma", 2)) {
       if (++argn < argc)
         sscanf (argv[argn], "%f", &gamma);
       else
-        pm_usage (usage);
+        pm_error("-gamma requires a value");
     } else
     if (pm_keymatch (argv[argn], "-hist", 2)) {
       hist = TRUE;
@@ -2075,63 +2226,63 @@ main(int argc, char *argv[]) {
       if (++argn < argc)
         sscanf (argv[argn], "%f", &chroma.wx);
       else
-        pm_usage (usage);
+        pm_error("-chroma requires 6 values");
       if (++argn < argc)
         sscanf (argv[argn], "%f", &chroma.wy);
       else
-        pm_usage (usage);
+        pm_error("-chroma requires 6 values");
       if (++argn < argc)
         sscanf (argv[argn], "%f", &chroma.rx);
       else
-        pm_usage (usage);
+        pm_error("-chroma requires 6 values");
       if (++argn < argc)
         sscanf (argv[argn], "%f", &chroma.ry);
       else
-        pm_usage (usage);
+        pm_error("-chroma requires 6 values");
       if (++argn < argc)
         sscanf (argv[argn], "%f", &chroma.gx);
       else
-        pm_usage (usage);
+        pm_error("-chroma requires 6 values");
       if (++argn < argc)
         sscanf (argv[argn], "%f", &chroma.gy);
       else
-        pm_usage (usage);
+        pm_error("-chroma requires 6 values");
       if (++argn < argc)
         sscanf (argv[argn], "%f", &chroma.bx);
       else
-        pm_usage (usage);
+        pm_error("-chroma requires 6 values");
       if (++argn < argc)
         sscanf (argv[argn], "%f", &chroma.by);
       else
-        pm_usage (usage);
+        pm_error("-chroma requires 6 values");
     } else
     if (pm_keymatch (argv[argn], "-phys", 3)) {
       if (++argn < argc)
         sscanf (argv[argn], "%d", &phys_x);
       else
-        pm_usage (usage);
+        pm_error("-phys requires 3 values");
       if (++argn < argc)
         sscanf (argv[argn], "%d", &phys_y);
       else
-        pm_usage (usage);
+        pm_error("-phys requires 3 values");
       if (++argn < argc)
         sscanf (argv[argn], "%d", &phys_unit);
       else
-        pm_usage (usage);
+        pm_error("-phys requires 3 values");
     } else
     if (pm_keymatch (argv[argn], "-text", 3)) {
       text = TRUE;
       if (++argn < argc)
         text_file = argv[argn];
       else
-        pm_usage (usage);
+        pm_error("-text requires a value");
     } else
     if (pm_keymatch (argv[argn], "-ztxt", 2)) {
       ztxt = TRUE;
       if (++argn < argc)
         text_file = argv[argn];
       else
-        pm_usage (usage);
+        pm_error("-ztxt requires a value");
     } else
     if (pm_keymatch (argv[argn], "-time", 3)) {
       mtime = TRUE;
@@ -2140,48 +2291,68 @@ main(int argc, char *argv[]) {
         if (++argn < argc)
           time_string = argv[argn];
         else
-          pm_usage (usage);
+          pm_error("-time requires 2 values");
       } else {
-        pm_usage (usage);
+        pm_error("-time requires 2 values");
       }
     } else 
     if (pm_keymatch (argv[argn], "-palette", 3)) {
       if (++argn < argc)
         palette_file = argv[argn];
       else
-        pm_usage (usage);
+        pm_error("-palette requires a value");
     } else
     if (pm_keymatch (argv[argn], "-filter", 3)) {
       if (++argn < argc)
       {
+        int filter;
         sscanf (argv[argn], "%d", &filter);
         if ((filter < 0) || (filter > 4))
-        {
-          pm_message
-            ("filter must be 0 (none), 1 (sub), 2 (up), 3 (avg) or 4 (paeth)");
-          pm_usage (usage);
-        }
+          pm_error("-filter is obsolete.  Use -nofilter, -sub, -up, -avg, "
+                   "and -paeth options instead.");
+        else
+          switch (filter) {
+          case 0: filterSet = PNG_FILTER_NONE;  break;
+          case 1: filterSet = PNG_FILTER_SUB;   break;
+          case 2: filterSet = PNG_FILTER_UP;    break;
+          case 3: filterSet = PNG_FILTER_AVG;   break;
+          case 4: filterSet = PNG_FILTER_PAETH; break;
+          }
       }
       else
-        pm_usage (usage);
+        pm_error("-filter is obsolete.  Use -nofilter, -sub, -up, -avg, "
+                 "and -paeth options instead.");
     } else
+    if (pm_keymatch (argv[argn], "-nofilter", 4))
+      filterSet |= PNG_FILTER_NONE;
+    else if (pm_keymatch (argv[argn], "-sub", 3))
+      filterSet |= PNG_FILTER_SUB;
+    else if (pm_keymatch (argv[argn], "-up", 3))
+      filterSet |= PNG_FILTER_UP;
+    else if (pm_keymatch (argv[argn], "-avg", 3))
+      filterSet |= PNG_FILTER_AVG;
+    else if (pm_keymatch (argv[argn], "-paeth", 3))
+      filterSet |= PNG_FILTER_PAETH;
+    else
     if (pm_keymatch (argv[argn], "-compression", 3)) {
       if (++argn < argc)
       {
-        sscanf (argv[argn], "%d", &compression);
-        if ((compression < 0) || (compression > 9))
+        sscanf (argv[argn], "%d", &zlib_compression.level);
+        if ((zlib_compression.level < 0) || (zlib_compression.level > 9))
         {
-          pm_message ("zlib compression must be between 0 (none) and 9 (max)");
-          pm_usage (usage);
+          pm_error("zlib compression must be between 0 (none) and 9 (max)");
         }
       }
       else
-        pm_usage (usage);
+        pm_error("-compression requires a value");
     } else
     if (pm_keymatch (argv[argn], "-force", 3)) {
       force = TRUE;
-    } else 
+    } else
+    if (pm_keymatch (argv[argn], "-libversion", 3))
       displayVersion();  /* exits program */
+    else
+      pm_error("Unrecognized option: '%s'", argv[argn]);
     argn++;
   }
 
@@ -2197,7 +2368,7 @@ main(int argc, char *argv[]) {
   }
 
   if (argn != argc)
-    pm_usage (usage);
+    pm_error("Too many arguments.");
 
   if (alpha)
     afp = pm_openr (alpha_file);
@@ -2219,8 +2390,8 @@ main(int argc, char *argv[]) {
                           transstring, alpha, alpha_file, background,
                           backstring, hist, chroma, phys_x, phys_y,
                           phys_unit, text, ztxt, text_file, mtime,
-                          date_string, time_string, filter,
-                          compression, force);
+                          date_string, time_string, filterSet,
+                          force, zlib_compression);
 
   if (afp)
     pm_closer (afp);
