@@ -16,6 +16,8 @@
 */
 
 #define _BSD_SOURCE  /* Make sure string.h contains strdup() */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #include <string.h>
 #include "pam.h"
 #include "shhopt.h"
@@ -40,9 +42,11 @@ struct cmdline_info {
     unsigned int imageheight;        /* in 1/72 inch; zero if unspec */
     unsigned int equalpixels;
     unsigned int setpage;
+    unsigned int verbose;
 };
 
 
+static bool verbose;
 
 static void putxelval ARGS(( xelval xv ));
 static void putrest ARGS(( void ));
@@ -118,6 +122,7 @@ parse_command_line(int argc, char ** argv,
     OPTENT3(0, "imageheight", OPT_FLOAT, &imageheight, &imageheight_spec,0);
     OPTENT3(0, "nosetpage",   OPT_FLAG,  NULL, &nosetpage,               0);
     OPTENT3(0, "setpage",     OPT_FLAG,  NULL, &cmdlineP->setpage,       0);
+    OPTENT3(0, "verbose",     OPT_FLAG,  NULL, &cmdlineP->verbose,       0);
     
     /* DEFAULTS */
     cmdlineP->scale = 1.0;
@@ -167,6 +172,12 @@ parse_command_line(int argc, char ** argv,
 
 
 
+/*----------------------------------------------------------------------------
+   The following global variables are the item putter state.
+
+   They are followed by the item putter methods, init_putitem() and
+   putitem().
+-----------------------------------------------------------------------------*/
 static unsigned int itemsinline;
     /* The number of items in the line we are currently building */
 static unsigned int bitsinitem;
@@ -186,7 +197,13 @@ static unsigned int repeat, itembuf[128], count, repeatitem, repeatcount;
 
 static void
 init_putitem(bool const rle, unsigned int const bitspersample) {
+/*----------------------------------------------------------------------------
+   Initialize the item putter.  Call this once per Postscript image that
+   you will write with putitem(), before for the first putitem().
 
+   We initialize the item putter state variables, which are the
+   global variable defined above.
+-----------------------------------------------------------------------------*/
     itemsinline = 0;
     items = 0;
 
@@ -204,6 +221,8 @@ init_putitem(bool const rle, unsigned int const bitspersample) {
 
 }
 
+
+
 static void
 putitem(void) {
     const char* const hexits = "0123456789abcdef";
@@ -609,6 +628,11 @@ compute_depth(xelval const input_maxval, unsigned int * const bitspersampleP,
                    "of %u", bitsRequiredByMaxval, *bitspersampleP);
 
     *ps_maxvalP = pm_bitstomaxval(*bitspersampleP);
+
+    if (verbose)
+        pm_message("Input maxval is %u.  Postscript raster will have "
+                   "%u bits per sample, so maxval = %u",
+                   input_maxval, *bitspersampleP, *ps_maxvalP);
 }    
 
 
@@ -666,7 +690,7 @@ convert_page(FILE *ifp, const int turnflag, const int turnokflag,
     float llx, lly;
     bool turned;
 
-    pnm_readpaminit(ifp, &inpam, sizeof(inpam));
+    pnm_readpaminit(ifp, &inpam, PAM_STRUCT_SIZE(tuple_type));
     tuplerow = pnm_allocpamrow(&inpam);
     
     if (!STRSEQ(inpam.tuple_type, PAM_PBM_TUPLETYPE) &&
@@ -725,6 +749,8 @@ main(int argc, char * argv[]) {
 
     parse_command_line(argc, argv, &cmdline);
 
+    verbose = cmdline.verbose;
+
     ifp = pm_openr(cmdline.input_filespec);
 
     if (strcmp(cmdline.input_filespec, "-") == 0 )
