@@ -657,7 +657,7 @@ typedef struct tagConv {
    removal of cmy under k */
 
 typedef struct {
-  int initialised ;
+  int initialized ;
   float theta ;
   float gamma ;
   float gammap ;
@@ -722,25 +722,25 @@ standardConvert( Conv *conv, Root *rt, float r, float g, float b,
                  int *c, int *m, int *y, int *k ) {
 
   float c0, m0, y0, k0 ; /* CMYK before colour removal */
-  float grey, white, colour ;
+  float gray, white, colour ;
   double phi ;
 
   Standard *s ; /* private conversion data */
 
   s = (Standard*)(conv->private) ;
 
-  if ( ! s->initialised ) {
+  if ( ! s->initialized ) {
     s->theta *= 2.0 * PI / 360.0 ; /* to radians */
     /* if gammap never specified in options, set to gamma now */
     if ( s->remove == UNSET ) {
       s->remove = 1 ;
       s->gammap = s->gamma ;
     }
-    s->initialised = 1 ;
+    s->initialized = 1 ;
   }
 
   /* rotate in rgb */
-  if ( abs( s->theta ) > TINY ) {
+  if ( fabs( s->theta ) > TINY ) {
     rgbToHueWhiteColour( r, g, b, &phi, &white, &colour ) ;
     hueWhiteColourToRgb( phi + s->theta, white, colour, &r, &g, &b ) ;
   }
@@ -755,13 +755,13 @@ standardConvert( Conv *conv, Root *rt, float r, float g, float b,
 
   /* remove colour under black and convert to integer range 0 - rt->maxOut */
   if ( s->remove ) {
-    grey = pow( k0, s->gammap ) ;
+    gray = pow( k0, s->gammap ) ;
   } else {
-    grey = 0.0 ;
+    gray = 0.0 ;
   }
-  *c = TOMAX( c0 - grey ) ;
-  *m = TOMAX( m0 - grey ) ;
-  *y = TOMAX( y0 - grey ) ;
+  *c = TOMAX( c0 - gray ) ;
+  *m = TOMAX( m0 - gray ) ;
+  *y = TOMAX( y0 - gray ) ;
 
   return ERR_OK ;
 }
@@ -819,7 +819,7 @@ newStandardMap( Conv **conv ) {
 
   if ( ! (*conv = (Conv*)calloc( 1, sizeof( Conv ) )) ) goto error ;
   if ( ! (s = (Standard*)calloc( 1, sizeof( Standard ) )) ) goto error ;
-  s->initialised = 0 ;
+  s->initialized = 0 ;
   s->remove = UNSET ; /* gammap unset */
   s->gamma = 1.0 ;
   (*conv)->private = s ;
