@@ -15,7 +15,8 @@
 -----------------------------------------------------------------------------*/
 
 #define _BSD_SOURCE 1   /* Make sure strdup() is in string.h */
-#define _XOPEN_SOURCE   /* Make sure fdopen() is in stdio.h */
+#define _XOPEN_SOURCE 500  
+    /* Make sure fdopen() is in stdio.h and strdup() is in string.h */
 
 #include <string.h>
 #include <unistd.h>
@@ -32,9 +33,9 @@
 enum orientation {PORTRAIT, LANDSCAPE, UNSPECIFIED};
 struct box {
     /* Description of a rectangle within an image; all coordinates 
-       measured in pixels with lower left corner of page being the 
+       measured in points (1/72") with lower left corner of page being the 
        origin.
-       */
+    */
     int llx;  /* lower left X coord */
         /* -1 for llx means whole box is undefined. */
     int lly;  /* lower left Y coord */
@@ -53,11 +54,12 @@ struct cmdline_info {
     unsigned int format_type;
     unsigned int verbose;
     float xborder;
-    int xmax;
-    int xsize;
+    unsigned int xmax;
+    unsigned int xsize;  /* zero means unspecified */
     float yborder;
-    int ymax;
-    int ysize;
+    unsigned int ymax;
+    unsigned int ysize;  /* zero means unspecified */
+    unsigned int dpi;    /* zero means unspecified */
     enum orientation orientation;
     unsigned int goto_stdout;
 } cmdline;
@@ -65,7 +67,7 @@ struct cmdline_info {
 
 static void
 parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
+                   struct cmdline_info *cmdlineP) {
 /*----------------------------------------------------------------------------
    Note that the file spec array we return is stored in the storage that
    was passed to us as the argv array.
@@ -81,84 +83,114 @@ parse_command_line(int argc, char ** argv,
     unsigned int portrait_opt, landscape_opt;
     float llx, lly, urx, ury;
     unsigned int llxSpec, llySpec, urxSpec, urySpec;
+    unsigned int xmaxSpec, ymaxSpec, xsizeSpec, ysizeSpec, dpiSpec;
     
     option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENT3(0, "forceplain", OPT_FLAG,  NULL, &cmdline_p->forceplain,     0);
-    OPTENT3(0, "llx",        OPT_FLOAT, &llx, &llxSpec,                   0);
-    OPTENT3(0, "lly",        OPT_FLOAT, &lly, &llySpec,                   0);
-    OPTENT3(0, "urx",        OPT_FLOAT, &urx, &urxSpec,                   0);
-    OPTENT3(0, "ury",        OPT_FLOAT, &ury, &urySpec,                   0);
-    OPTENT3(0, "nocrop",     OPT_FLAG,  NULL, &cmdline_p->nocrop,         0);
-    OPTENT3(0, "pbm",        OPT_FLAG,  NULL, &pbm_opt,                   0);
-    OPTENT3(0, "pgm",        OPT_FLAG,  NULL, &pgm_opt,                   0);
-    OPTENT3(0, "ppm",        OPT_FLAG,  NULL, &ppm_opt,                   0);
-    OPTENT3(0, "verbose",    OPT_FLAG,  NULL, &cmdline_p->verbose,        0);
-    OPTENT3(0, "xborder",    OPT_FLOAT, &cmdline_p->xborder, NULL,        0);
-    OPTENT3(0, "xmax",       OPT_UINT,  &cmdline_p->xmax, NULL,           0);
-    OPTENT3(0, "xsize",      OPT_UINT,  &cmdline_p->xsize, NULL,          0);
-    OPTENT3(0, "yborder",    OPT_FLOAT, &cmdline_p->yborder, NULL,        0);
-    OPTENT3(0, "ymax",       OPT_UINT,  &cmdline_p->ymax, NULL,           0);
-    OPTENT3(0, "ysize",      OPT_UINT,  &cmdline_p->ysize, NULL,          0);
-    OPTENT3(0, "portrait",   OPT_FLAG,  NULL, &portrait_opt,              0);
-    OPTENT3(0, "landscape",  OPT_FLAG,  NULL, &landscape_opt,             0);
-    OPTENT3(0, "stdout",     OPT_FLAG,  NULL, &cmdline_p->goto_stdout,    0);
+    OPTENT3(0, "forceplain", OPT_FLAG,  NULL, &cmdlineP->forceplain,     0);
+    OPTENT3(0, "llx",        OPT_FLOAT, &llx, &llxSpec,                  0);
+    OPTENT3(0, "lly",        OPT_FLOAT, &lly, &llySpec,                  0);
+    OPTENT3(0, "urx",        OPT_FLOAT, &urx, &urxSpec,                  0);
+    OPTENT3(0, "ury",        OPT_FLOAT, &ury, &urySpec,                  0);
+    OPTENT3(0, "nocrop",     OPT_FLAG,  NULL, &cmdlineP->nocrop,         0);
+    OPTENT3(0, "pbm",        OPT_FLAG,  NULL, &pbm_opt,                  0);
+    OPTENT3(0, "pgm",        OPT_FLAG,  NULL, &pgm_opt,                  0);
+    OPTENT3(0, "ppm",        OPT_FLAG,  NULL, &ppm_opt,                  0);
+    OPTENT3(0, "verbose",    OPT_FLAG,  NULL, &cmdlineP->verbose,        0);
+    OPTENT3(0, "xborder",    OPT_FLOAT, &cmdlineP->xborder, NULL,        0);
+    OPTENT3(0, "xmax",       OPT_UINT,  &cmdlineP->xmax, &xmaxSpec,      0);
+    OPTENT3(0, "xsize",      OPT_UINT,  &cmdlineP->xsize, &xsizeSpec,    0);
+    OPTENT3(0, "yborder",    OPT_FLOAT, &cmdlineP->yborder, NULL,        0);
+    OPTENT3(0, "ymax",       OPT_UINT,  &cmdlineP->ymax, &ymaxSpec,      0);
+    OPTENT3(0, "ysize",      OPT_UINT,  &cmdlineP->ysize, &ysizeSpec,    0);
+    OPTENT3(0, "dpi",        OPT_UINT,  &cmdlineP->dpi, &dpiSpec,        0);
+    OPTENT3(0, "portrait",   OPT_FLAG,  NULL, &portrait_opt,             0);
+    OPTENT3(0, "landscape",  OPT_FLAG,  NULL, &landscape_opt,            0);
+    OPTENT3(0, "stdout",     OPT_FLAG,  NULL, &cmdlineP->goto_stdout,    0);
 
     /* Set the defaults */
-    llx = lly = urx = ury = -1;
-    cmdline_p->xmax = 612;
-    cmdline_p->ymax = 792;
-    cmdline_p->xsize = cmdline_p->ysize = -1;
-    cmdline_p->xborder = cmdline_p->yborder = 0.1;
+    cmdlineP->xborder = cmdlineP->yborder = 0.1;
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
     opt.allowNegNum = FALSE;  /* We have no parms that are negative numbers */
 
     optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
-        /* Uses and sets argc, argv, and some of *cmdline_p and others. */
+        /* Uses and sets argc, argv, and some of *cmdlineP and others. */
 
-    if (argc-1 == 0)
-        cmdline_p->input_filespec = "-";  /* stdin */
-    else if (argc-1 == 1)
-        cmdline_p->input_filespec = argv[1];
-    else 
-        pm_error("Too many arguments (%d).  "
-                 "Only need one: the Postscript filespec", argc-1);
+    if (xmaxSpec) {
+        if (cmdlineP->xmax == 0)
+            pm_error("zero is not a valid value for -xmax");
+    } else
+        cmdlineP->xmax = 612;
+
+    if (ymaxSpec) {
+        if (cmdlineP->ymax == 0)
+            pm_error("zero is not a valid value for -ymax");
+    } else 
+        cmdlineP->ymax = 792;
+
+    if (xsizeSpec) {
+        if (cmdlineP->xsize == 0)
+            pm_error("zero is not a valid value for -xsize");
+    } else
+        cmdlineP->xsize = 0;
+
+    if (ysizeSpec) {
+        if (cmdlineP->ysize == 0)
+            pm_error("zero is not a valid value for -ysize");
+    } else 
+        cmdlineP->ysize = 0;
 
     if (portrait_opt & !landscape_opt)
-        cmdline_p->orientation = PORTRAIT;
+        cmdlineP->orientation = PORTRAIT;
     else if (!portrait_opt & landscape_opt)
-        cmdline_p->orientation = LANDSCAPE;
+        cmdlineP->orientation = LANDSCAPE;
     else if (!portrait_opt & !landscape_opt)
-        cmdline_p->orientation = UNSPECIFIED;
+        cmdlineP->orientation = UNSPECIFIED;
     else
         pm_error("Cannot specify both -portrait and -landscape options");
 
     if (pbm_opt)
-        cmdline_p->format_type = PBM_TYPE;
+        cmdlineP->format_type = PBM_TYPE;
     else if (pgm_opt)
-        cmdline_p->format_type = PGM_TYPE;
+        cmdlineP->format_type = PGM_TYPE;
     else if (ppm_opt)
-        cmdline_p->format_type = PPM_TYPE;
+        cmdlineP->format_type = PPM_TYPE;
     else
-        cmdline_p->format_type = PPM_TYPE;
+        cmdlineP->format_type = PPM_TYPE;
 
     /* If any one of the 4 bounding box coordinates is given on the
        command line, we default any of the 4 that aren't.  
     */
     if (llxSpec || llySpec || urxSpec || urySpec) {
-        if (!llxSpec) cmdline_p->extract_box.llx = 72;
-        else cmdline_p->extract_box.llx = llx * 72;
-        if (!llySpec) cmdline_p->extract_box.lly = 72;
-        else cmdline_p->extract_box.lly = lly * 72;
-        if (!urxSpec) cmdline_p->extract_box.urx = 540;
-        else cmdline_p->extract_box.urx = urx * 72;
-        if (!urySpec) cmdline_p->extract_box.ury = 720;
-        else cmdline_p->extract_box.ury = ury * 72;
+        if (!llxSpec) cmdlineP->extract_box.llx = 72;
+        else cmdlineP->extract_box.llx = llx * 72;
+        if (!llySpec) cmdlineP->extract_box.lly = 72;
+        else cmdlineP->extract_box.lly = lly * 72;
+        if (!urxSpec) cmdlineP->extract_box.urx = 540;
+        else cmdlineP->extract_box.urx = urx * 72;
+        if (!urySpec) cmdlineP->extract_box.ury = 720;
+        else cmdlineP->extract_box.ury = ury * 72;
     } else {
-        cmdline_p->extract_box.llx = -1;
+        cmdlineP->extract_box.llx = -1;
     }
 
+    if (dpiSpec) {
+        if (cmdlineP->dpi == 0)
+            pm_error("Zero is not a valid value for -dpi");
+    } else
+        cmdlineP->dpi = 0;
+
+    if (dpiSpec && xsizeSpec + ysizeSpec + xmaxSpec + ymaxSpec > 0)
+        pm_error("You may not specify both size options and -dpi");
+
+    if (argc-1 == 0)
+        cmdlineP->input_filespec = "-";  /* stdin */
+    else if (argc-1 == 1)
+        cmdlineP->input_filespec = argv[1];
+    else 
+        pm_error("Too many arguments (%d).  "
+                 "Only need one: the Postscript filespec", argc-1);
 }
 
 
@@ -200,13 +232,93 @@ add_ps_to_filespec(const char orig_filespec[], char ** const new_filespec_p,
 
 
 static void
-compute_size_res(const struct cmdline_info cmdline, 
-                 const enum orientation orientation, 
-                 const struct box bordered_box,
-                 int * const xsize_p, int * const ysize_p,
-                 int * const xres_p, int * const yres_p) {
+computeSizeResFromSizeSpec(unsigned int   const requestedXsize,
+                           unsigned int   const requestedYsize,
+                           unsigned int   const imageWidth,
+                           unsigned int   const imageHeight,
+                           unsigned int * const xsizeP,
+                           unsigned int * const ysizeP,
+                           unsigned int * const xresP,
+                           unsigned int * const yresP) {
 
-    int sx, sy;
+    if (requestedXsize) {
+        *xsizeP = requestedXsize;
+        *xresP = (unsigned int) (requestedXsize * 72 / imageWidth + 0.5);
+        if (!requestedYsize) {
+            *yresP = *xresP;
+            *ysizeP = (unsigned int) (imageHeight * (float)*yresP/72 + 0.5);
+            }
+        }
+
+    if (requestedYsize) {
+        *ysizeP = requestedYsize;
+        *yresP = (unsigned int) (requestedYsize * 72 / imageHeight + 0.5);
+        if (!requestedXsize) {
+            *xresP = *yresP;
+            *xsizeP = (unsigned int) (imageWidth * (float)*xresP/72 + 0.5);
+        }
+    } 
+}
+
+
+
+static void
+computeSizeResBlind(unsigned int   const xmax,
+                    unsigned int   const ymax,
+                    unsigned int   const imageWidth,
+                    unsigned int   const imageHeight,
+                    bool           const nocrop,
+                    unsigned int * const xsizeP,
+                    unsigned int * const ysizeP,
+                    unsigned int * const xresP,
+                    unsigned int * const yresP) {
+
+    *xresP = *yresP = MIN(xmax * 72 / imageWidth, 
+                          ymax * 72 / imageHeight);
+    
+    if (nocrop) {
+        *xsizeP = xmax;
+        *ysizeP = ymax;
+    } else {
+        *xsizeP = (unsigned int) (imageWidth * (float)*xresP / 72 + 0.5);
+        *ysizeP = (unsigned int) (imageHeight * (float)*yresP / 72 + 0.5);
+    }
+}
+
+
+
+static void
+compute_size_res(struct cmdline_info const cmdline, 
+                 enum orientation    const orientation, 
+                 struct box          const bordered_box,
+                 unsigned int *      const xsizeP, 
+                 unsigned int *      const ysizeP,
+                 unsigned int *      const xresP, 
+                 unsigned int *      const yresP) {
+/*----------------------------------------------------------------------------
+  Figure out how big the output image should be (return as
+  *xsizeP and *ysizeP) and what output device resolution Ghostscript
+  should assume (return as *xresP, *yresP).
+
+  A resolution number is the number of pixels per inch that the a
+  printer prints.  Since we're emulating a printed page with a PNM
+  image, and a PNM image has no spatial dimension (you can't say how
+  many inches wide a PNM image is), it's kind of confusing.  
+
+  If the user doesn't select a resolution, we choose the resolution
+  that causes the image to be a certain number of pixels, knowing how
+  big (in inches) Ghostscript wants the printed picture to be.  For
+  example, the part of the Postscript image we are going to print is 2
+  inches wide.  We want the PNM image to be 1000 pixels wide.  So we
+  tell Ghostscript that our horizontal output device resolution is 500
+  pixels per inch.
+  
+  *xresP and *yresP are in dots per inch.
+-----------------------------------------------------------------------------*/
+    unsigned int sx, sy;
+        /* The horizontal and vertical sizes of the input image, in points
+           (1/72 inch)
+        */
 
     if (orientation == LANDSCAPE) {
         sx = bordered_box.ury - bordered_box.lly;
@@ -215,44 +327,29 @@ compute_size_res(const struct cmdline_info cmdline,
         sx = bordered_box.urx - bordered_box.llx;
         sy = bordered_box.ury - bordered_box.lly;
     }
-    if (cmdline.ysize == -1 && cmdline.xsize == -1) {
-        /* User didn't specify either x or y size, so we compute them
-           from xmax and ymax
-           */
-        *yres_p = cmdline.ymax * 72 / sy;
-        *xres_p = *yres_p = 
-            MIN(cmdline.xmax * 72 / sx, cmdline.ymax * 72 / sy);
-        
-        if (cmdline.nocrop) {
-            *xsize_p = cmdline.xmax;
-            *ysize_p = cmdline.ymax;
-        } else {
-            *xsize_p = (int) (sx * *xres_p / 72 + 0.5);
-            *ysize_p = (int) (sy * *yres_p / 72 + 0.5);
-        }
-    } else {
-        /* Compute the x and y size from the specified x and/or y sizes */
-        if (cmdline.xsize != -1) {
-            *xsize_p = cmdline.xsize;
-            *xres_p = cmdline.xsize * 72 / sx;
-            if (cmdline.ysize == -1) {
-                *yres_p = *xres_p;
-                *ysize_p = (int) (sy * *yres_p/72 + 0.5);
-            }
-        }
 
-        if (cmdline.ysize != -1) {
-            *ysize_p = cmdline.ysize;
-            *yres_p = cmdline.ysize * 72 / sy;
-            if (cmdline.xsize == -1) {
-                *xres_p = *yres_p;
-                *xsize_p = (int) (sx * *xres_p/72 + 0.5);
-            }
-        }
+    if (cmdline.dpi) {
+        /* User gave resolution; we figure out output image size */
+        *xresP = *yresP = cmdline.dpi;
+        *xsizeP = (int) (cmdline.dpi * sx / 72 + 0.5);
+        *ysizeP = (int) (cmdline.dpi * sy / 72 + 0.5);
+    } else  if (cmdline.xsize || cmdline.ysize)
+        computeSizeResFromSizeSpec(cmdline.xsize, cmdline.ysize, sx, sy,
+                                   xsizeP, ysizeP, xresP, yresP);
+    else 
+        computeSizeResBlind(cmdline.xmax, cmdline.ymax, sx, sy, cmdline.nocrop,
+                            xsizeP, ysizeP, xresP, yresP);
+
+    if (cmdline.verbose) {
+        pm_message("output is %u pixels wide X %u pixels high",
+                   *xsizeP, *ysizeP);
+        pm_message("output device resolution is %u dpi horiz, %u dpi vert",
+                   *xresP, *yresP);
     }
 }
 
 
+
 enum postscript_language {COMMON_POSTSCRIPT, ENCAPSULATED_POSTSCRIPT};
 
 static enum postscript_language
@@ -278,9 +375,9 @@ language_declaration(const char input_filespec[], int const verbose) {
         if (fgets(line, sizeof(line), infile) == NULL)
             language = COMMON_POSTSCRIPT;
         else {
-            const char eps_header[] = "%!PS-Adobe EPSF-3.0";
-            
-            if (strncmp(line, eps_header, sizeof(eps_header)) == 0)
+            const char eps_header[] = " EPSF-";
+
+            if (strstr(line, eps_header))
                 language = ENCAPSULATED_POSTSCRIPT;
             else
                 language = COMMON_POSTSCRIPT;
@@ -369,27 +466,27 @@ compute_box_to_extract(const struct box cmdline_extract_box,
 
 
 static enum orientation
-compute_orientation(const struct cmdline_info cmdline, 
-                    const struct box extract_box) {
+compute_orientation(struct cmdline_info const cmdline, 
+                    struct box          const extract_box) {
 
     enum orientation retval;
-    const int input_width = extract_box.urx - extract_box.llx;
-    const int input_height = extract_box.ury - extract_box.lly;
+    unsigned int const input_width  = extract_box.urx - extract_box.llx;
+    unsigned int const input_height = extract_box.ury - extract_box.lly;
 
     if (cmdline.orientation != UNSPECIFIED)
         retval = cmdline.orientation;
     else {
-        if ((cmdline.xsize == -1 || cmdline.ysize == -1) &
-            (cmdline.xsize != -1 || cmdline.ysize != -1)) {
+        if ((!cmdline.xsize || !cmdline.ysize) &
+            (cmdline.xsize || cmdline.ysize)) {
             /* User specified one output dimension, but not the other,
                so we can't use output dimensions to make the decision.  So
                just use the input dimensions.
-               */
+            */
             if (input_height > input_width) retval = PORTRAIT;
             else retval = LANDSCAPE;
         } else {
             int output_width, output_height;
-            if (cmdline.xsize != -1) {
+            if (cmdline.xsize) {
                 /* He gave xsize and ysize, so that's the output size */
                 output_width = cmdline.xsize;
                 output_height = cmdline.ysize;
@@ -401,9 +498,9 @@ compute_orientation(const struct cmdline_info cmdline,
 
             if (input_height > input_width && output_height > output_width)
                 retval = PORTRAIT;
-            else 
-                if (input_height < input_width && output_height < output_width)
-                    retval = PORTRAIT;
+            else if (input_height < input_width && 
+                     output_height < output_width)
+                retval = PORTRAIT;
             else 
                 retval = LANDSCAPE;
         }
@@ -735,8 +832,8 @@ main(int argc, char **argv) {
 
     char *input_filespec;  /* malloc'ed */
         /* The file specification of our Postscript input file */
-    int xres, yres;   /* Resolution in pixels per inch */
-    int xsize, ysize;  /* output image size in pixels */
+    unsigned int xres, yres;    /* Resolution in pixels per inch */
+    unsigned int xsize, ysize;  /* output image size in pixels */
     struct box extract_box;
         /* coordinates of the box within the input we are to extract; i.e.
            that will become the output. 
