@@ -46,8 +46,8 @@
    give the user the -byrow option to order (2) only.
 */
 
-#define _BSD_SOURCE 1
-    /* Make sure strdup() is in string.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
 #include <string.h>
 #include "pnm.h"
@@ -333,21 +333,36 @@ static void
 pick_cmyk_pixel(const unsigned int samplebuf[], const int sample_cursor,
                 xelval * const r_p, xelval * const b_p, xelval * const g_p) {
 
-    unsigned char c, y, m, k;
-
-    c = samplebuf[sample_cursor+0];
-    y = samplebuf[sample_cursor+1];
-    m = samplebuf[sample_cursor+2];
-    k = samplebuf[sample_cursor+3];
-
-    /* I don't know why this works.  It seems to me that yellow pigment should
-       translate into both red and green light, not just green as you see here.
-       But this is the formula used by TIFFRGBAImageGet() in the Tiff library.
+    /* Note that the TIFF spec does not say which of the 4 samples is
+       which, but common sense says they're in order C,M,Y,K.  Before
+       Netpbm 10.21 (March 2004), we assumed C,Y,M,K for some reason.
+       But combined with a compensating error in the CMYK->RGB
+       calculation, it had the same effect as C,M,Y,K.
     */
-    
-    *r_p = ((255-k)*(255-c))/255;
-    *g_p = ((255-k)*(255-y))/255;
-    *b_p = ((255-k)*(255-m))/255;
+    unsigned int const c = samplebuf[sample_cursor+0];
+    unsigned int const m = samplebuf[sample_cursor+1];
+    unsigned int const y = samplebuf[sample_cursor+2];
+    unsigned int const k = samplebuf[sample_cursor+3];
+
+    /* The CMYK->RGB formula used by TIFFRGBAImageGet() in the TIFF 
+       library is the following, (with some apparent confusion with
+       the names of the yellow and magenta pigments being reversed).
+
+       R = (1-K)*(1-C)     (with C,Y,M,K normalized to 0..1)
+       G = (1-K)*(1-M)
+       B = (1-K)*(1-Y)
+
+       We used that too before Netpbm 10.21 (March 2004).
+
+       Now we use the inverse of what Pnmtotiffcmyk has always used, which
+       makes sense as follows:  A microliter of black ink is simply a 
+       substitute for a microliter each of cyan, magenta, and yellow ink.
+       Yellow ink removes blue light from what the white paper reflects.  
+    */
+
+    *r_p = 255 - MIN(255, c + k);
+    *g_p = 255 - MIN(255, m + k);
+    *b_p = 255 - MIN(255, y + k);
 }
 
 
@@ -378,23 +393,27 @@ computeFillorder(unsigned short   const fillorderTag,
 
 
 static void
-analyze_image_type(TIFF * const tif, unsigned short const bps, 
-                   unsigned short const spp, unsigned short const photomet,
-                   xelval * const maxvalP, int * const formatP, 
-                   xel colormap[], bool const headerdump) {
+analyze_image_type(TIFF *         const tif, 
+                   unsigned short const bps, 
+                   unsigned short const spp, 
+                   unsigned short const photomet,
+                   xelval *       const maxvalP, 
+                   int *          const formatP, 
+                   xel                  colormap[],
+                   bool           const headerdump) {
 
     bool grayscale; 
 
-    if ( bps == 1 && spp == 1 ) {
-        if ( cmdline.headerdump )
-            pm_message("bilevel" );
+    if (bps == 1 && spp == 1) {
+        if (cmdline.headerdump)
+            pm_message("bilevel");
         grayscale = TRUE;
         *maxvalP = 1;
     } else {
         /* How come we don't deal with the photometric for the monochrome 
            case (make sure it's one we know)?  -Bryan 00.03.04
         */
-        switch ( photomet ) {
+        switch (photomet) {
         case PHOTOMETRIC_MINISBLACK:
         case PHOTOMETRIC_MINISWHITE:
             if (spp != 1)
@@ -856,6 +875,9 @@ convertRasterInMemory(FILE *         const imageoutFile,
         } else {
             uint32* raster ;
 
+            /* Note that TIFFRGBAImageGet() converts any bits per sample
+               to 8.  Maxval of the raster it returns is always 255.
+            */
             MALLOCARRAY(raster, cols * rows);
             if (raster == NULL) {
                 pm_message("Unable to allocate space for a raster of %u "
@@ -933,7 +955,13 @@ convertIt(TIFF*              const tif,
                 tif, photomet, planarconfig, bps, spp, fillorder,
                 colormap, &status);
         }
-        if (status != CONV_DONE) {
+        if (status == CONV_DONE) {
+            if (bps > 8)
+                pm_message("actual resolution has been reduced to 24 bits "
+                           "per pixel in the conversion.  You can get the "
+                           "full %u bits that are in the TIFF with the "
+                           "-byrow option.", bps);
+        } else {
             if (status != CONV_NOTATTEMPTED)
                 pm_message("In-memory conversion failed; "
                            "using more primitive row-by-row conversion.");
@@ -941,7 +969,7 @@ convertIt(TIFF*              const tif,
             convertRasterByRows(
                 imageoutFile, alphaFile, cols, rows, maxval, format, 
                 tif, photomet, planarconfig, bps, spp, fillorder, colormap);
-        }            
+        }
     }
 }
 
