@@ -17,8 +17,8 @@
 */
 
 
-#define _BSD_SOURCE 1
-    /* Make sure strdup() is in string.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
 #include <stdio.h>
 #include <string.h>
@@ -35,6 +35,7 @@ static struct cmdline_info {
     char *input_filename;
     unsigned int verbose;
     unsigned int debug;
+    unsigned int headerdump;
 } cmdline;
 
 
@@ -137,6 +138,7 @@ parseCommandLine(int argc, char ** argv,
 
     OPTENT3(0,   "verbose",    OPT_FLAG,   NULL, &cmdlineP->verbose,       0);
     OPTENT3(0,   "debug",      OPT_FLAG,   NULL, &cmdlineP->debug,         0);
+    OPTENT3(0,   "headerdump", OPT_FLAG,   NULL, &cmdlineP->headerdump,    0);
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -285,32 +287,43 @@ processX10Header(X10WDFileHeader *  const h10P,
 
 
 static void
-fixH11ByteOrder(X11WDFileHeader *  const h11P) {
-    h11P->header_size = bs_long( h11P->header_size );
-    h11P->file_version = bs_long( h11P->file_version );
-    h11P->pixmap_format = bs_long( h11P->pixmap_format );
-    h11P->pixmap_depth = bs_long( h11P->pixmap_depth );
-    h11P->pixmap_width = bs_long( h11P->pixmap_width );
-    h11P->pixmap_height = bs_long( h11P->pixmap_height );
-    h11P->xoffset = bs_long( h11P->xoffset );
-    h11P->byte_order = bs_long( h11P->byte_order );
-    h11P->bitmap_unit = bs_long( h11P->bitmap_unit );
-    h11P->bitmap_bit_order = bs_long( h11P->bitmap_bit_order );
-    h11P->bitmap_pad = bs_long( h11P->bitmap_pad );
-    h11P->bits_per_pixel = bs_long( h11P->bits_per_pixel );
-    h11P->bytes_per_line = bs_long( h11P->bytes_per_line );
-    h11P->visual_class = bs_long( h11P->visual_class );
-    h11P->red_mask = bs_long( h11P->red_mask );
-    h11P->green_mask = bs_long( h11P->green_mask );
-    h11P->blue_mask = bs_long( h11P->blue_mask );
-    h11P->bits_per_rgb = bs_long( h11P->bits_per_rgb );
-    h11P->colormap_entries = bs_long( h11P->colormap_entries );
-    h11P->ncolors = bs_long( h11P->ncolors );
-    h11P->window_width = bs_long( h11P->window_width );
-    h11P->window_height = bs_long( h11P->window_height );
-    h11P->window_x = bs_long( h11P->window_x );
-    h11P->window_y = bs_long( h11P->window_y );
-    h11P->window_bdrwidth = bs_long( h11P->window_bdrwidth );
+fixH11ByteOrder(X11WDFileHeader *  const h11P,
+                X11WDFileHeader ** const h11FixedPP) {
+
+    X11WDFileHeader * h11FixedP;
+
+    MALLOCVAR_NOFAIL(h11FixedP);
+
+    if (h11P->file_version == X11WD_FILE_VERSION) {
+        memcpy(h11FixedP, h11P, sizeof(*h11FixedP));
+    } else {
+        h11FixedP->header_size = bs_long( h11P->header_size );
+        h11FixedP->file_version = bs_long( h11P->file_version );
+        h11FixedP->pixmap_format = bs_long( h11P->pixmap_format );
+        h11FixedP->pixmap_depth = bs_long( h11P->pixmap_depth );
+        h11FixedP->pixmap_width = bs_long( h11P->pixmap_width );
+        h11FixedP->pixmap_height = bs_long( h11P->pixmap_height );
+        h11FixedP->xoffset = bs_long( h11P->xoffset );
+        h11FixedP->byte_order = bs_long( h11P->byte_order );
+        h11FixedP->bitmap_unit = bs_long( h11P->bitmap_unit );
+        h11FixedP->bitmap_bit_order = bs_long( h11P->bitmap_bit_order );
+        h11FixedP->bitmap_pad = bs_long( h11P->bitmap_pad );
+        h11FixedP->bits_per_pixel = bs_long( h11P->bits_per_pixel );
+        h11FixedP->bytes_per_line = bs_long( h11P->bytes_per_line );
+        h11FixedP->visual_class = bs_long( h11P->visual_class );
+        h11FixedP->red_mask = bs_long( h11P->red_mask );
+        h11FixedP->green_mask = bs_long( h11P->green_mask );
+        h11FixedP->blue_mask = bs_long( h11P->blue_mask );
+        h11FixedP->bits_per_rgb = bs_long( h11P->bits_per_rgb );
+        h11FixedP->colormap_entries = bs_long( h11P->colormap_entries );
+        h11FixedP->ncolors = bs_long( h11P->ncolors );
+        h11FixedP->window_width = bs_long( h11P->window_width );
+        h11FixedP->window_height = bs_long( h11P->window_height );
+        h11FixedP->window_x = bs_long( h11P->window_x );
+        h11FixedP->window_y = bs_long( h11P->window_y );
+        h11FixedP->window_bdrwidth = bs_long( h11P->window_bdrwidth );
+    }
+    *h11FixedPP = h11FixedP;
 }
 
 
@@ -325,12 +338,12 @@ readX11Colormap(FILE *      const file,
     int rc;
 
     /* Read X11 colormap. */
-    MALLOCARRAY( x11colors, ncolors );
+    MALLOCARRAY(x11colors, ncolors);
     if (x11colors == NULL)
         pm_error("out of memory");
     rc = fread(x11colors, sizeof(X11XColor), ncolors, file);
     if (rc != ncolors)
-        pm_error( "couldn't read X11 XWD colormap" );
+        pm_error("couldn't read X11 XWD colormap");
     if (byteSwap) {
         unsigned int i;
         for (i = 0; i < ncolors; ++i) {
@@ -369,6 +382,78 @@ colormapAllGray(const X11XColor* const x11colors,
 
 
 
+static void
+dumpX11Header(X11WDFileHeader * const h11P) {
+
+    const char * formatDesc;
+    const char * byteOrderDesc;
+    const char * bitOrderDesc;
+    const char * visualClassDesc;
+
+    X11WDFileHeader * h11FixedP;
+
+    fixH11ByteOrder(h11P, &h11FixedP);
+ 
+    switch(h11FixedP->pixmap_format) {
+    case XYBitmap: formatDesc = "XY bit map"; break;
+    case XYPixmap: formatDesc = "XY pix map"; break;
+    case ZPixmap:  formatDesc = "Z pix map";  break;
+    default:       formatDesc = "???";
+    }
+
+    switch(h11FixedP->byte_order) {
+    case LSBFirst: byteOrderDesc = "LSB first"; break;
+    case MSBFirst: byteOrderDesc = "MSB first"; break;
+    default:       byteOrderDesc = "???";
+    }
+
+    switch (h11FixedP->bitmap_bit_order) {
+    case LSBFirst: bitOrderDesc = "LSB first"; break;
+    case MSBFirst: bitOrderDesc = "MSB first"; break;
+    default:       bitOrderDesc = "???";
+    }
+
+    switch (h11FixedP->visual_class) {
+    case StaticGray:  visualClassDesc = "StaticGray";  break;
+    case GrayScale:   visualClassDesc = "GrayScale";   break;
+    case StaticColor: visualClassDesc = "StaticColor"; break;
+    case PseudoColor: visualClassDesc = "PseudoColor"; break;
+    case DirectColor: visualClassDesc = "DirectColor"; break;
+    default:          visualClassDesc = "???";
+    }
+
+    pm_message("File version: %u", h11FixedP->file_version);
+    pm_message("Format: %s (%u)", formatDesc, h11FixedP->pixmap_format);
+    pm_message("Width:  %u", h11FixedP->pixmap_width);
+    pm_message("Height: %u", h11FixedP->pixmap_height);
+    pm_message("Depth: %u bits", h11FixedP->pixmap_depth);
+    pm_message("X offset: %u", h11FixedP->xoffset);
+    pm_message("byte order: %s (%u)", byteOrderDesc, h11FixedP->byte_order);
+    pm_message("bitmap unit: %u", h11FixedP->bitmap_unit);
+    pm_message("bit order: %s (%u)", 
+               bitOrderDesc, h11FixedP->bitmap_bit_order);
+    pm_message("bitmap pad: %u", h11FixedP->bitmap_pad);
+    pm_message("bits per pixel: %u", h11FixedP->bits_per_pixel);
+    pm_message("bytes per line: %u", h11FixedP->bytes_per_line);
+    pm_message("visual class: %s (%u)", 
+               visualClassDesc, h11FixedP->visual_class);
+    pm_message("red mask:   %08x", h11FixedP->red_mask);
+    pm_message("green mask: %08x", h11FixedP->green_mask);
+    pm_message("blue mask:  %08x", h11FixedP->blue_mask);
+    pm_message("bits per rgb: %u", h11FixedP->bits_per_rgb);
+    pm_message("number of colormap entries: %u", h11FixedP->colormap_entries);
+    pm_message("number of colors in colormap: %u", h11FixedP->ncolors);
+    pm_message("window width:  %u", h11FixedP->window_width);
+    pm_message("window height: %u", h11FixedP->window_height);
+    pm_message("window upper left X coordinate: %u", h11FixedP->window_x);
+    pm_message("window upper left Y coordinate: %u", h11FixedP->window_y);
+    pm_message("window border width: %u", h11FixedP->window_bdrwidth);
+
+    free(h11FixedP);
+}
+
+
+
 static void
 processX11Header(X11WDFileHeader *  const h11P, 
                  FILE *             const file,
@@ -390,105 +475,106 @@ processX11Header(X11WDFileHeader *  const h11P,
     int i;
     X11XColor* x11colors;
     bool grayscale;
-    bool byte_swap;
+    bool const byte_swap = (h11P->file_version != X11WD_FILE_VERSION);
+    X11WDFileHeader * h11FixedP;
+ 
+    fixH11ByteOrder(h11P, &h11FixedP);
 
-    *maxvalP = 65535;   /* Initial assumption */
-
-    if ( h11P->file_version != X11WD_FILE_VERSION ) {
-        if (verbose)
-            pm_message("Input is byte-swapped");
-        byte_swap = TRUE;
-        fixH11ByteOrder(h11P);
-    } else
-        byte_swap = FALSE;
-
-    for ( i = 0; i < h11P->header_size - sizeof(*h11P); ++i )
-        if ( getc( file ) == EOF )
-            pm_error( "couldn't read rest of X11 XWD file header" );
+    if (byte_swap && verbose)
+        pm_message("Input is different endianness from this machine.");
+    
+    for (i = 0; i < h11FixedP->header_size - sizeof(*h11FixedP); ++i)
+        if (getc(file) == EOF)
+            pm_error("couldn't read rest of X11 XWD file header");
 
     /* Check whether we can handle this dump. */
-    if ( h11P->pixmap_depth > 24 )
+    if ( h11FixedP->pixmap_depth > 24 )
         pm_error( "can't handle X11 pixmap_depth > 24" );
-    if ( h11P->bits_per_rgb > 24 )
+    if ( h11FixedP->bits_per_rgb > 24 )
         pm_error( "can't handle X11 bits_per_rgb > 24" );
-    if ( h11P->pixmap_format != ZPixmap && h11P->pixmap_depth != 1 )
+    if ( h11FixedP->pixmap_format != ZPixmap && h11FixedP->pixmap_depth != 1 )
         pm_error(
             "can't handle X11 pixmap_format %d with depth != 1",
-            h11P->pixmap_format );
-    if ( h11P->bitmap_unit != 8 && h11P->bitmap_unit != 16 &&
-         h11P->bitmap_unit != 32 )
+            h11FixedP->pixmap_format );
+    if ( h11FixedP->bitmap_unit != 8 && h11FixedP->bitmap_unit != 16 &&
+         h11FixedP->bitmap_unit != 32 )
         pm_error(
             "X11 bitmap_unit (%d) is non-standard - can't handle",
-            h11P->bitmap_unit );
+            h11FixedP->bitmap_unit );
     /* The following check was added in 10.19 (November 2003) */
-    if ( h11P->bitmap_pad != 8 && h11P->bitmap_pad != 16 &&
-         h11P->bitmap_pad != 32 )
+    if ( h11FixedP->bitmap_pad != 8 && h11FixedP->bitmap_pad != 16 &&
+         h11FixedP->bitmap_pad != 32 )
         pm_error(
             "X11 bitmap_pad (%d) is non-standard - can't handle",
-            h11P->bitmap_unit );
+            h11FixedP->bitmap_unit );
 
-    if ( h11P->ncolors > 0 ) {
-        readX11Colormap( file, h11P->ncolors, byte_swap, &x11colors );
-        grayscale = colormapAllGray( x11colors, h11P->ncolors );
+    if ( h11FixedP->ncolors > 0 ) {
+        readX11Colormap( file, h11FixedP->ncolors, byte_swap, &x11colors );
+        grayscale = colormapAllGray( x11colors, h11FixedP->ncolors );
     } else
         grayscale = TRUE;
 
-    *visualclassP = (enum visualclass) h11P->visual_class;
+    *visualclassP = (enum visualclass) h11FixedP->visual_class;
     if ( *visualclassP == DirectColor ) {
+        unsigned int i;
         *formatP = PPM_TYPE;
         *maxvalP = 65535;
         /*
           DirectColor is like PseudoColor except that there are essentially
-          3 colormaps (shade maps) -- one for each primary color.  Each pixel
-          is composed of 3 separate indices
+          3 colormaps (shade maps) -- one for each color component.  Each pixel
+          is composed of 3 separate indices.
         */
 
-        *colorsP = pnm_allocrow( h11P->ncolors );
-        for ( i = 0; i < h11P->ncolors; ++i )
+        *colorsP = pnm_allocrow( h11FixedP->ncolors );
+        for ( i = 0; i < h11FixedP->ncolors; ++i )
             PPM_ASSIGN(
                 (*colorsP)[i], x11colors[i].red, x11colors[i].green,
                 x11colors[i].blue);
     } else if ( *visualclassP == TrueColor ) {
         *formatP = PPM_TYPE;
 
-        *maxvalP = pm_lcm(pm_bitstomaxval(one_bits(h11P->red_mask)),
-                          pm_bitstomaxval(one_bits(h11P->green_mask)),
-                          pm_bitstomaxval(one_bits(h11P->blue_mask)),
+        *maxvalP = pm_lcm(pm_bitstomaxval(one_bits(h11FixedP->red_mask)),
+                          pm_bitstomaxval(one_bits(h11FixedP->green_mask)),
+                          pm_bitstomaxval(one_bits(h11FixedP->blue_mask)),
                           PPM_OVERALLMAXVAL
             );
     }
-    else if ( *visualclassP == StaticGray && h11P->bits_per_pixel == 1 ) {
+    else if ( *visualclassP == StaticGray && h11FixedP->bits_per_pixel == 1 ) {
         *formatP = PBM_TYPE;
         *maxvalP = 1;
         *colorsP = pnm_allocrow( 2 );
         PNM_ASSIGN1( (*colorsP)[0], *maxvalP );
         PNM_ASSIGN1( (*colorsP)[1], 0 );
     } else if ( *visualclassP == StaticGray ) {
+        unsigned int i;
         *formatP = PGM_TYPE;
-        *maxvalP = ( 1 << h11P->bits_per_pixel ) - 1;
+        *maxvalP = ( 1 << h11FixedP->bits_per_pixel ) - 1;
         *colorsP = pnm_allocrow( *maxvalP + 1 );
         for ( i = 0; i <= *maxvalP; ++i )
             PNM_ASSIGN1( (*colorsP)[i], i );
     } else {
-        *colorsP = pnm_allocrow( h11P->ncolors );
+        *colorsP = pnm_allocrow( h11FixedP->ncolors );
         if ( grayscale ) {
+            unsigned int i;
             *formatP = PGM_TYPE;
-            for ( i = 0; i < h11P->ncolors; ++i )
+            for ( i = 0; i < h11FixedP->ncolors; ++i )
                 PNM_ASSIGN1( (*colorsP)[i], x11colors[i].red );
         } else {
+            unsigned int i;
             *formatP = PPM_TYPE;
-            for ( i = 0; i < h11P->ncolors; ++i )
+            for ( i = 0; i < h11FixedP->ncolors; ++i )
                 PPM_ASSIGN(
                     (*colorsP)[i], x11colors[i].red, x11colors[i].green,
                     x11colors[i].blue);
         }
+        *maxvalP = 65535;
     }
 
-    *colsP = h11P->pixmap_width;
-    *rowsP = h11P->pixmap_height;
+    *colsP = h11FixedP->pixmap_width;
+    *rowsP = h11FixedP->pixmap_height;
     *padrightP =
-        h11P->bytes_per_line * 8 / h11P->bits_per_pixel -
-        h11P->pixmap_width;
+        h11FixedP->bytes_per_line * 8 / h11FixedP->bits_per_pixel -
+        h11FixedP->pixmap_width;
     /* According to X11/XWDFile.h, the item size is 'bitmap_pad' for some
        images and 'bitmap_unit' for others.  This is strange, so there may
        be some subtlety of their definitions that we're missing.
@@ -510,32 +596,42 @@ processX11Header(X11WDFileHeader *  const h11P,
        figure whichever one is greater (assuming both are meaningful)
        has to be the item size.  
     */
-    *bits_per_itemP = MAX(h11P->bitmap_pad, h11P->bitmap_unit);
-    if (h11P->bits_per_pixel > *bits_per_itemP)
+    *bits_per_itemP = MAX(h11FixedP->bitmap_pad, h11FixedP->bitmap_unit);
+    if (h11FixedP->bits_per_pixel > *bits_per_itemP)
         pm_error("Invalid xwd.  bits per pixel (%u) is greater than the "
                  "bits per item (%u)", 
-                 h11P->bits_per_pixel, *bits_per_itemP);
+                 h11FixedP->bits_per_pixel, *bits_per_itemP);
 
-    *bits_per_pixelP = h11P->bits_per_pixel;
+    *bits_per_pixelP = h11FixedP->bits_per_pixel;
 
-    *byte_orderP = (enum byteorder) h11P->byte_order;
-    *bit_orderP = (enum byteorder) h11P->bitmap_bit_order;
-    *red_maskP = h11P->red_mask;
-    *green_maskP = h11P->green_mask;
-    *blue_maskP = h11P->blue_mask;
+    *byte_orderP = (enum byteorder) h11FixedP->byte_order;
+    *bit_orderP = (enum byteorder) h11FixedP->bitmap_bit_order;
+    *red_maskP = h11FixedP->red_mask;
+    *green_maskP = h11FixedP->green_mask;
+    *blue_maskP = h11FixedP->blue_mask;
+
+    free(h11FixedP);
 } 
 
 
 
 static void
-getinit( FILE *file, int * const colsP, int * const rowsP, 
-         int * const padrightP, xelval * const maxvalP, 
-         enum visualclass * const visualclassP, int * const formatP, 
-         xel ** const colorsP, int * const bits_per_pixelP, 
-         int * const bits_per_itemP, unsigned long * const red_maskP, 
-         unsigned long * const green_maskP, unsigned long * const blue_maskP,
-         enum byteorder * const byte_orderP,
-         enum byteorder * const bit_orderP) {
+getinit(FILE *             const file, 
+        int *              const colsP, 
+        int *              const rowsP, 
+        int *              const padrightP, 
+        xelval *           const maxvalP, 
+        enum visualclass * const visualclassP, 
+        int *              const formatP, 
+        xel **             const colorsP,
+        int *              const bits_per_pixelP, 
+        int *              const bits_per_itemP, 
+        unsigned long *    const red_maskP, 
+        unsigned long *    const green_maskP,
+        unsigned long *    const blue_maskP,
+        enum byteorder *   const byte_orderP,
+        enum byteorder *   const bit_orderP,
+        bool               const headerDump) {
 
     /* Assume X11 headers are larger than X10 ones. */
     unsigned char header[sizeof(X11WDFileHeader)];
@@ -572,7 +668,7 @@ getinit( FILE *file, int * const colsP, int * const rowsP,
                          red_maskP, green_maskP, blue_maskP, 
                          byte_orderP, bit_orderP);
     } else if (h11P->file_version == X11WD_FILE_VERSION ||
-             bs_long(h11P->file_version) == X11WD_FILE_VERSION) {
+               bs_long(h11P->file_version) == X11WD_FILE_VERSION) {
         
         int rc;
 
@@ -585,13 +681,16 @@ getinit( FILE *file, int * const colsP, int * const rowsP,
         if (rc != 1)
             pm_error("couldn't read end of X11 XWD file header");
 
+        if (headerDump)
+            dumpX11Header(h11P);
+
         processX11Header(h11P, file, colsP, rowsP, padrightP, maxvalP, 
                          visualclassP, formatP, 
                          colorsP, bits_per_pixelP, bits_per_itemP, 
                          red_maskP, green_maskP, blue_maskP, 
                          byte_orderP, bit_orderP);
     } else
-        pm_error( "unknown XWD file version: %d", h11P->file_version );
+        pm_error("unknown XWD file version: %u", h11P->file_version);
 }
 
 
@@ -604,13 +703,13 @@ getpix(FILE *file, struct row_control * const row_controlP,
    Get a pixel from the input image.
 
    A pixel is a bit string.  It may be either an rgb triplet or an index
-   into the colormap (or even an rgb triplet of indices into the colormap!).
+   into the colormap (or even an rgb triplet of indices into the colormaps!).
    We don't care -- it's just a bit string.
    
    The basic unit of storage in the input file is an "item."  An item
    can be 1, 2, or 4 bytes, and 'bits_per_item' tells us which.  Each
-   item can have its bytes stored in little-endian or big-endian
-   format, and 'byte_order' tells us which.
+   item can have its bytes stored in forward or reverse order, and
+   'byte_order' tells us which.
 
    Each item can contain one or more pixels, and may contain
    fractional pixels.  'bits_per_pixel' tells us how many bits each
@@ -1067,10 +1166,11 @@ main( int argc, char *argv[] ) {
     else
         ifp = stdin;
 
-    getinit( ifp, &cols, &rows, &padright, &maxval, &visualclass, &format, 
-             &colors, &bits_per_pixel, &bits_per_item, 
-             &red_mask, &green_mask, &blue_mask, &byte_order, &bit_order );
-
+    getinit(ifp, &cols, &rows, &padright, &maxval, &visualclass, &format, 
+            &colors, &bits_per_pixel, &bits_per_item, 
+            &red_mask, &green_mask, &blue_mask, &byte_order, &bit_order,
+            cmdline.headerdump);
+    
     if (verbose) 
         reportInfo(cols, rows, padright, maxval, visualclass,
                    format, bits_per_pixel, bits_per_item,
