@@ -1,21 +1,21 @@
 /*===========================================================================*
- * iframe.c								     *
- *									     *
- *	Procedures concerned with the I-frame encoding			     *
- *									     *
- * EXPORTED PROCEDURES:							     *
- *	GenIFrame							     *
- *	SetSlicesPerFrame						     *
- *	SetBlocksPerSlice						     *
- *	SetIQScale							     *
- *	GetIQScale							     *
- *	ResetIFrameStats						     *
- *	ShowIFrameSummary						     *
- *	EstimateSecondsPerIFrame					     *
- *	EncodeYDC							     *
- *	EncodeCDC							     *
+ * iframe.c                                  *
+ *                                       *
+ *  Procedures concerned with the I-frame encoding               *
+ *                                       *
+ * EXPORTED PROCEDURES:                              *
+ *  GenIFrame                                *
+ *  SetSlicesPerFrame                            *
+ *  SetBlocksPerSlice                            *
+ *  SetIQScale                               *
+ *  GetIQScale                               *
+ *  ResetIFrameStats                             *
+ *  ShowIFrameSummary                            *
+ *  EstimateSecondsPerIFrame                         *
+ *  EncodeYDC                                *
+ *  EncodeCDC                                *
  *      time_elapsed                                                         *
- *									     *
+ *                                       *
  *===========================================================================*/
 
 /*
@@ -143,6 +143,7 @@
 #include <sys/times.h>
 #endif
 
+#include <sys/types.h>
 #include <sys/param.h>
 #include "all.h"
 #include "mtypes.h"
@@ -155,32 +156,33 @@
 #include "parallel.h"
 #include "postdct.h"
 #include "rate.h"
+#include "specifics.h"
 #include "opts.h"
 
 /*==================*
  * STATIC VARIABLES *
  *==================*/
 
-static  int	lastNumBits = 0;
-static  int	lastIFrame = 0;
+static  int lastNumBits = 0;
+static  int lastIFrame = 0;
 static int numBlocks = 0;
 static int numBits;
 static int numFrames = 0;
 static int numFrameBits = 0;
 static int32 totalTime = 0;
-static float	totalSNR = 0.0;
-static float	totalPSNR = 0.0;
+static float    totalSNR = 0.0;
+static float    totalPSNR = 0.0;
 
 static int lengths[256] = {
-    0, 1, 2, 2, 3, 3, 3, 3,	    /* 0 - 7 */
-    4, 4, 4, 4, 4, 4, 4, 4,	    /* 8 - 15 */
-    5, 5, 5, 5, 5, 5, 5, 5,	    /* 16 - 31 */
+    0, 1, 2, 2, 3, 3, 3, 3,     /* 0 - 7 */
+    4, 4, 4, 4, 4, 4, 4, 4,     /* 8 - 15 */
+    5, 5, 5, 5, 5, 5, 5, 5,     /* 16 - 31 */
     5, 5, 5, 5, 5, 5, 5, 5,
-    6, 6, 6, 6, 6, 6, 6, 6,	    /* 32 - 63 */
+    6, 6, 6, 6, 6, 6, 6, 6,     /* 32 - 63 */
     6, 6, 6, 6, 6, 6, 6, 6,
     6, 6, 6, 6, 6, 6, 6, 6,
     6, 6, 6, 6, 6, 6, 6, 6,
-    7, 7, 7, 7, 7, 7, 7, 7,	    /* 64 - 127 */
+    7, 7, 7, 7, 7, 7, 7, 7,     /* 64 - 127 */
     7, 7, 7, 7, 7, 7, 7, 7,
     7, 7, 7, 7, 7, 7, 7, 7,
     7, 7, 7, 7, 7, 7, 7, 7,
@@ -211,13 +213,13 @@ static int lengths[256] = {
  * GLOBAL VARIABLES *
  *==================*/
 
-int	qscaleI;
-int	slicesPerFrame;
-int	blocksPerSlice;
-int	fCodeI, fCodeP, fCodeB;
-boolean	printSNR = FALSE;
-boolean	printMSE = FALSE;
-boolean	decodeRefFrames = FALSE;
+int qscaleI;
+int slicesPerFrame;
+int blocksPerSlice;
+int fCodeI, fCodeP, fCodeB;
+boolean printSNR = FALSE;
+boolean printMSE = FALSE;
+boolean decodeRefFrames = FALSE;
 Block **dct=NULL, **dctr=NULL, **dctb=NULL;
 dct_data_type   **dct_data; /* used in p/bframe.c */
 int  TIME_RATE;
@@ -226,7 +228,7 @@ int  TIME_RATE;
 /*=====================*
  * EXPORTED PROCEDURES *
  *=====================*/
-extern void	PrintItoIBitRate _ANSI_ARGS_((int numBits, int frameNum));
+extern void PrintItoIBitRate _ANSI_ARGS_((int numBits, int frameNum));
 
 /*===============================*
  * INTERNAL PROCEDURE prototypes *
@@ -239,43 +241,43 @@ int
   SetFCodeHelper(SR)
 int SR;
 {
-    int	    range,fCode;
+    int     range,fCode;
 
     if ( pixelFullSearch ) {
-	range = SR;
+    range = SR;
     } else {
-	range = SR*2;
+    range = SR*2;
     }
 
     if ( range < 256 ) {
-	if ( range < 64 ) {
-	    if ( range < 32 ) {
-		fCode = 1;
-	    } else {
-		fCode = 2;
-	    }
-	} else {
-	    if ( range < 128 ) {
-		fCode = 3;
-	    } else {
-		fCode = 4;
-	    }
-	}
+    if ( range < 64 ) {
+        if ( range < 32 ) {
+        fCode = 1;
+        } else {
+        fCode = 2;
+        }
     } else {
-	if ( range < 1024 ) {
-	    if ( range < 512 ) {
-		fCode = 5;
-	    } else {
-		fCode = 6;
-	    }
-	} else {
-	    if ( range < 2048 ) {
-		fCode = 7;
-	    } else {
-		fprintf(stderr, "ERROR:  INVALID SEARCH RANGE!!!\n");
-		exit(1);
-	    }
-	}
+        if ( range < 128 ) {
+        fCode = 3;
+        } else {
+        fCode = 4;
+        }
+    }
+    } else {
+    if ( range < 1024 ) {
+        if ( range < 512 ) {
+        fCode = 5;
+        } else {
+        fCode = 6;
+        }
+    } else {
+        if ( range < 2048 ) {
+        fCode = 7;
+        } else {
+        fprintf(stderr, "ERROR:  INVALID SEARCH RANGE!!!\n");
+        exit(1);
+        }
+    }
       }
     return fCode;
 }
@@ -284,12 +286,12 @@ int SR;
  *
  * SetFCode
  *
- *	set the forward_f_code and backward_f_code according to the search
- *	range.  Must be called AFTER pixelFullSearch and searchRange have
- *	been initialized.  Irrelevant for I-frames, but computation is
- *	negligible (done only once, as well)
+ *  set the forward_f_code and backward_f_code according to the search
+ *  range.  Must be called AFTER pixelFullSearch and searchRange have
+ *  been initialized.  Irrelevant for I-frames, but computation is
+ *  negligible (done only once, as well)
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    fCodeI,fCodeP,fCodeB
  *
@@ -306,9 +308,9 @@ SetFCode()
  *
  * SetSlicesPerFrame
  *
- *	set the number of slices per frame
+ *  set the number of slices per frame
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    slicesPerFrame
  *
@@ -325,9 +327,9 @@ SetSlicesPerFrame(number)
  *
  * SetBlocksPerSlice
  *
- *	set the number of blocks per slice, based on slicesPerFrame
+ *  set the number of blocks per slice, based on slicesPerFrame
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    blocksPerSlice
  *
@@ -335,14 +337,14 @@ SetSlicesPerFrame(number)
 void
 SetBlocksPerSlice()
 {
-    int	    totalBlocks;
+    int     totalBlocks;
 
     totalBlocks = (Fsize_y>>4)*(Fsize_x>>4);
 
     if ( slicesPerFrame > totalBlocks ) {
-	blocksPerSlice = 1;
+    blocksPerSlice = 1;
     } else {
-	blocksPerSlice = totalBlocks/slicesPerFrame;
+    blocksPerSlice = totalBlocks/slicesPerFrame;
     }
 }
 
@@ -351,9 +353,9 @@ SetBlocksPerSlice()
  *
  * SetIQScale
  *
- *	set the I-frame Q-scale
+ *  set the I-frame Q-scale
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    qscaleI
  *
@@ -369,9 +371,9 @@ int qI;
  *
  * GetIQScale
  *
- *	Get the I-frame Q-scale
+ *  Get the I-frame Q-scale
  *
- * RETURNS:	the Iframe Qscale
+ * RETURNS: the Iframe Qscale
  *
  * SIDE EFFECTS:    none
  *
@@ -386,28 +388,27 @@ GetIQScale()
  *
  * GenIFrame
  *
- *	generate an I-frame; appends result to bb
+ *  generate an I-frame; appends result to bb
  *
- * RETURNS:	I-frame appended to bb
+ * RETURNS: I-frame appended to bb
  *
  * SIDE EFFECTS:    none
  *
  *===========================================================================*/
 void
-GenIFrame(bb, current)
-     BitBucket *bb;
-     MpegFrame *current;
-{
-    register int x, y;
-    register int index;
-    FlatBlock	 fb[6];
-    Block 	 dec[6];
+GenIFrame(BitBucket * const bb, 
+          MpegFrame * const current) {
+
+    int x, y;
+    int index;
+    FlatBlock    fb[6];
+    Block    dec[6];
     int32 y_dc_pred, cr_dc_pred, cb_dc_pred;
     int          totalBits;
-    int	         totalFrameBits;
+    int          totalFrameBits;
     int32        startTime, endTime;
     float        snr[3], psnr[3];
-    int	         mbAddress;
+    int          mbAddress;
     int          QScale;
     BlockMV      *info; /* Not used in Iframes, but nice to pass in anyway */
     int          bitstreamMode, newQScale;
@@ -419,27 +420,27 @@ GenIFrame(bb, current)
     /* set-up for statistics */
     numFrames++;
     totalFrameBits = bb->cumulativeBits;
-    if ( ( ! childProcess) && showBitRatePerFrame ) {
-      if ( lastNumBits == 0 ) {
-	lastNumBits = bb->cumulativeBits;
-	lastIFrame = current->id;
-      } else {
-	/* ASSUMES 30 FRAMES PER SECOND */
-	
-	if (! realQuiet) {
-	  fprintf(stdout, "I-to-I (frames %5d to %5d) bitrate:  %8d\n",
-		  lastIFrame, current->id-1,
-		  ((bb->cumulativeBits-lastNumBits)*30)/
-		  (current->id-lastIFrame));
-	}
-	
-	fprintf(bitRateFile, "I-to-I (frames %5d to %5d) bitrate:  %8d\n",
-		lastIFrame, current->id-1,
-		((bb->cumulativeBits-lastNumBits)*30)/
-		(current->id-lastIFrame));
-	lastNumBits = bb->cumulativeBits;	    
-	lastIFrame = current->id;
-      }
+    if ( showBitRatePerFrame ) {
+        if ( lastNumBits == 0 ) {
+            lastNumBits = bb->cumulativeBits;
+            lastIFrame = current->id;
+        } else {
+            /* ASSUMES 30 FRAMES PER SECOND */
+    
+            if (! realQuiet) {
+                fprintf(stdout, "I-to-I (frames %5d to %5d) bitrate:  %8d\n",
+                        lastIFrame, current->id-1,
+                        ((bb->cumulativeBits-lastNumBits)*30)/
+                        (current->id-lastIFrame));
+            }
+    
+            fprintf(bitRateFile, "I-to-I (frames %5d to %5d) bitrate:  %8d\n",
+                    lastIFrame, current->id-1,
+                    ((bb->cumulativeBits-lastNumBits)*30)/
+                    (current->id-lastIFrame));
+            lastNumBits = bb->cumulativeBits;       
+            lastIFrame = current->id;
+        }
     }
     
     startTime = time_elapsed();
@@ -452,28 +453,28 @@ GenIFrame(bb, current)
     /*   Allocate bits for this frame for rate control purposes */
     bitstreamMode = getRateMode();
     if (bitstreamMode == FIXED_RATE) {
-      targetRateControl(current);
+        targetRateControl(current);
     }
 
     Mhead_GenPictureHeader(bb, I_FRAME, current->id, fCodeI);
     /* Check for Qscale change */
     if (specificsOn) {
-      newQScale = SpecLookup(current->id, 0, 0 /* junk */, &info, QScale);
-      if (newQScale != -1) {
-	QScale = newQScale;
-      }
-      /* check for slice */
-      newQScale = SpecLookup(current->id, 1, 1, &info, QScale);
-      if (newQScale != -1) {
-	QScale = newQScale;
-      }
+        newQScale = SpecLookup(current->id, 0, 0 /* junk */, &info, QScale);
+        if (newQScale != -1) {
+            QScale = newQScale;
+        }
+        /* check for slice */
+        newQScale = SpecLookup(current->id, 1, 1, &info, QScale);
+        if (newQScale != -1) {
+            QScale = newQScale;
+        }
     }
     Mhead_GenSliceHeader(bb, 1, QScale, NULL, 0);
     
     if ( referenceFrame == DECODED_FRAME ) {
-      Frame_AllocDecoded(current, TRUE);
+        Frame_AllocDecoded(current, TRUE);
     } else if ( printSNR ) {
-      Frame_AllocDecoded(current, FALSE);
+        Frame_AllocDecoded(current, FALSE);
     }
     
     y_dc_pred = cr_dc_pred = cb_dc_pred = 128;
@@ -482,116 +483,114 @@ GenIFrame(bb, current)
 
     /* DCT the macroblocks */
     for (y = 0;  y < (Fsize_y >> 3);  y += 2) {
-      for (x = 0;  x < (Fsize_x >> 3);  x += 2) {
-	if (collect_quant && (collect_quant_detailed & 1)) fprintf(collect_quant_fp, "l\n");
-	if (DoLaplace) {LaplaceCnum = 0;}
-	mp_fwd_dct_block2(current->y_blocks[y][x], dct[y][x]);
-	mp_fwd_dct_block2(current->y_blocks[y][x+1], dct[y][x+1]);
-	mp_fwd_dct_block2(current->y_blocks[y+1][x], dct[y+1][x]);
-	mp_fwd_dct_block2(current->y_blocks[y+1][x+1], dct[y+1][x+1]);
-	if (collect_quant && (collect_quant_detailed & 1)) fprintf(collect_quant_fp, "c\n");
-	if (DoLaplace) {LaplaceCnum = 1;}
-	mp_fwd_dct_block2(current->cb_blocks[y>>1][x>>1], dctb[y>>1][x>>1]);
-	if (DoLaplace) {LaplaceCnum = 2;}
-	mp_fwd_dct_block2(current->cr_blocks[y>>1][x>>1], dctr[y>>1][x>>1]);
-      }}
-	
-    if (DoLaplace) {
-      CalcLambdas();
+        for (x = 0;  x < (Fsize_x >> 3);  x += 2) {
+            if (collect_quant && (collect_quant_detailed & 1)) 
+                fprintf(collect_quant_fp, "l\n");
+            if (DoLaplace) {LaplaceCnum = 0;}
+            mp_fwd_dct_block2(current->y_blocks[y][x], dct[y][x]);
+            mp_fwd_dct_block2(current->y_blocks[y][x+1], dct[y][x+1]);
+            mp_fwd_dct_block2(current->y_blocks[y+1][x], dct[y+1][x]);
+            mp_fwd_dct_block2(current->y_blocks[y+1][x+1], dct[y+1][x+1]);
+            if (collect_quant && (collect_quant_detailed & 1)) 
+                fprintf(collect_quant_fp, "c\n");
+            if (DoLaplace) {LaplaceCnum = 1;}
+            mp_fwd_dct_block2(current->cb_blocks[y>>1][x>>1], 
+                              dctb[y>>1][x>>1]);
+            if (DoLaplace) {LaplaceCnum = 2;}
+            mp_fwd_dct_block2(current->cr_blocks[y>>1][x>>1], 
+                              dctr[y>>1][x>>1]);
+        }
     }
+    
+    if (DoLaplace)
+        CalcLambdas();
 
     for (y = 0;  y < (Fsize_y >> 3);  y += 2) {
-      for (x = 0;  x < (Fsize_x >> 3);  x += 2) {
-	/* Check for Qscale change */
-	if (specificsOn) {
-	  newQScale = SpecLookup(current->id, 2, mbAddress, &info, QScale);
-	  if (newQScale != -1) {
-	    QScale = newQScale;
-	  }
-	}
-	
-	/*  Determine if new Qscale needed for Rate Control purposes  */
-	if (bitstreamMode == FIXED_RATE) {
-	  rc_blockStart = bb->cumulativeBits;
-	  newQScale = needQScaleChange(qscaleI,
-				       current->y_blocks[y][x],
-				       current->y_blocks[y][x+1],
-				       current->y_blocks[y+1][x],
-				       current->y_blocks[y+1][x+1]);
-	  if (newQScale > 0) {
-	    QScale = newQScale;
-	  }
-	}
-	
-	if ( (mbAddress % blocksPerSlice == 0) && (mbAddress != 0) ) {
-	  /* create a new slice */
-	  if (specificsOn) {
-	    /* Make sure no slice Qscale change */
-	    newQScale = SpecLookup(current->id,1,mbAddress/blocksPerSlice, &info, QScale);
-	    if (newQScale != -1) QScale = newQScale;
-	  }
-	  Mhead_GenSliceEnder(bb);
-	  Mhead_GenSliceHeader(bb, 1+(y>>1), QScale, NULL, 0);
-	  y_dc_pred = cr_dc_pred = cb_dc_pred = 128;
-	  
-	  GEN_I_BLOCK(I_FRAME, current, bb, 1+(x>>1), QScale);
-	} else {
-	  GEN_I_BLOCK(I_FRAME, current, bb, 1, QScale);
-	}
+        for (x = 0;  x < (Fsize_x >> 3);  x += 2) {
+            /* Check for Qscale change */
+            if (specificsOn) {
+                newQScale = 
+                    SpecLookup(current->id, 2, mbAddress, &info, QScale);
+                if (newQScale != -1) {
+                    QScale = newQScale;
+                }
+            }
+    
+            /*  Determine if new Qscale needed for Rate Control purposes  */
+            if (bitstreamMode == FIXED_RATE) {
+                rc_blockStart = bb->cumulativeBits;
+                newQScale = needQScaleChange(qscaleI,
+                                             current->y_blocks[y][x],
+                                             current->y_blocks[y][x+1],
+                                             current->y_blocks[y+1][x],
+                                             current->y_blocks[y+1][x+1]);
+                if (newQScale > 0) {
+                    QScale = newQScale;
+                }
+            }
+    
+            if ( (mbAddress % blocksPerSlice == 0) && (mbAddress != 0) ) {
+                /* create a new slice */
+                if (specificsOn) {
+                    /* Make sure no slice Qscale change */
+                    newQScale = SpecLookup(current->id, 1,
+                                           mbAddress/blocksPerSlice, &info, 
+                                           QScale);
+                    if (newQScale != -1) QScale = newQScale;
+                }
+                Mhead_GenSliceEnder(bb);
+                Mhead_GenSliceHeader(bb, 1+(y>>1), QScale, NULL, 0);
+                y_dc_pred = cr_dc_pred = cb_dc_pred = 128;
+      
+                GEN_I_BLOCK(I_FRAME, current, bb, 1+(x>>1), QScale);
+            } else {
+                GEN_I_BLOCK(I_FRAME, current, bb, 1, QScale);
+            }
 
-	if (WriteDistortionNumbers) {
-	  CalcDistortion(current, y, x);
-	}
-	
-	if ( decodeRefFrames ) {
-	  /* now, reverse the DCT transform */
-	  LaplaceCnum = 0;
-	  for ( index = 0; index < 6; index++ ) {
-	    if (!DoLaplace) {
-	      Mpost_UnQuantZigBlock(fb[index], dec[index], QScale, TRUE);
-	    } else {
-	      if (index == 4) {LaplaceCnum = 1;}
-	      if (index == 5) {LaplaceCnum = 2;}
-	      Mpost_UnQuantZigBlockLaplace(fb[index], dec[index], QScale, TRUE);
-	    }
-	    mpeg_jrevdct((int16 *)dec[index]);		
-	    }
-	  
-	  /* now, unblockify */
-	  BlockToData(current->decoded_y, dec[0], y, x);
-	  BlockToData(current->decoded_y, dec[1], y, x+1);
-	  BlockToData(current->decoded_y, dec[2], y+1, x);
-	  BlockToData(current->decoded_y, dec[3], y+1, x+1);
-	  BlockToData(current->decoded_cb, dec[4], y>>1, x>>1);
-	  BlockToData(current->decoded_cr, dec[5], y>>1, x>>1);
-	}
-	
-	numBlocks++;
-	mbAddress++;
-	/*   Rate Control */
-	if (bitstreamMode == FIXED_RATE) {
-	  incMacroBlockBits(bb->cumulativeBits - rc_blockStart);
-	  rc_blockStart = bb->cumulativeBits;
-	  MB_RateOut(TYPE_IFRAME);
-	}
-      }
+            if (WriteDistortionNumbers) {
+                CalcDistortion(current, y, x);
+            }
+    
+            if ( decodeRefFrames ) {
+                /* now, reverse the DCT transform */
+                LaplaceCnum = 0;
+                for ( index = 0; index < 6; index++ ) {
+                    if (!DoLaplace) {
+                        Mpost_UnQuantZigBlock(fb[index], dec[index], QScale, 
+                                              TRUE);
+                    } else {
+                        if (index == 4) {LaplaceCnum = 1;}
+                        if (index == 5) {LaplaceCnum = 2;}
+                        Mpost_UnQuantZigBlockLaplace(fb[index], dec[index], 
+                                                     QScale, TRUE);
+                    }
+                    mpeg_jrevdct((int16 *)dec[index]);      
+                }
+      
+                /* now, unblockify */
+                BlockToData(current->decoded_y, dec[0], y, x);
+                BlockToData(current->decoded_y, dec[1], y, x+1);
+                BlockToData(current->decoded_y, dec[2], y+1, x);
+                BlockToData(current->decoded_y, dec[3], y+1, x+1);
+                BlockToData(current->decoded_cb, dec[4], y>>1, x>>1);
+                BlockToData(current->decoded_cr, dec[5], y>>1, x>>1);
+            }
+    
+            numBlocks++;
+            mbAddress++;
+            /*   Rate Control */
+            if (bitstreamMode == FIXED_RATE) {
+                incMacroBlockBits(bb->cumulativeBits - rc_blockStart);
+                rc_blockStart = bb->cumulativeBits;
+                MB_RateOut(TYPE_IFRAME);
+            }
+        }
     }
     
     if ( printSNR ) {
-      BlockComputeSNR(current,snr,psnr);
-      totalSNR += snr[0];
-      totalPSNR += psnr[0];
-    }
-    
-    if ( (referenceFrame == DECODED_FRAME) && NonLocalRefFrame(current->id) ) {
-      if ( remoteIO ) {
-	SendDecodedFrame(current);
-      } else {
-	WriteDecodedFrame(current);
-      }
-      
-      /* now, tell decode server it is ready */
-      NotifyDecodeServerReady(current->id);
+        BlockComputeSNR(current,snr,psnr);
+        totalSNR += snr[0];
+        totalPSNR += psnr[0];
     }
     
     numBits += (bb->cumulativeBits-totalBits);
@@ -601,7 +600,7 @@ GenIFrame(bb, current)
     Mhead_GenSliceEnder(bb);
     /*   Rate Control  */
     if (bitstreamMode == FIXED_RATE) {
-      updateRateControl(TYPE_IFRAME);
+        updateRateControl(TYPE_IFRAME);
     }
     
     endTime = time_elapsed();
@@ -609,34 +608,38 @@ GenIFrame(bb, current)
     
     numFrameBits += (bb->cumulativeBits-totalFrameBits);
     
-    if ( ( ! childProcess) && showBitRatePerFrame ) {
-      /* ASSUMES 30 FRAMES PER SECOND */
-      fprintf(bitRateFile, "%5d\t%8d\n", current->id,
-	      30*(bb->cumulativeBits-totalFrameBits));
+    if ( showBitRatePerFrame ) {
+        /* ASSUMES 30 FRAMES PER SECOND */
+        fprintf(bitRateFile, "%5d\t%8d\n", current->id,
+                30*(bb->cumulativeBits-totalFrameBits));
     }
     
-    if ( (! childProcess) && frameSummary && (! realQuiet) ) {
-      
-      /* ASSUMES 30 FRAMES PER SECOND */
-      fprintf(stdout, "FRAME %d (I):  %ld seconds  (%d bits/s output)\n", 
-	      current->id, (long)((endTime-startTime)/TIME_RATE),
-	      30*(bb->cumulativeBits-totalFrameBits));
-      if ( printSNR ) {
-	fprintf(stdout, "FRAME %d:  SNR:  %.1f\t%.1f\t%.1f\tPSNR:  %.1f\t%.1f\t%.1f\n",
-		current->id, snr[0], snr[1], snr[2],
-		psnr[0], psnr[1], psnr[2]);
-      }
+    if ( frameSummary && !realQuiet ) {
+        
+        /* ASSUMES 30 FRAMES PER SECOND */
+        fprintf(stdout, 
+                "FRAME %d (I):  %ld seconds  (%d bits/s output)\n", 
+                current->id, (long)((endTime-startTime)/TIME_RATE),
+                30*(bb->cumulativeBits-totalFrameBits));
+        if ( printSNR ) {
+            fprintf(stdout, 
+                    "FRAME %d:  SNR:  %.1f\t%.1f\t%.1f\t"
+                    "PSNR:  %.1f\t%.1f\t%.1f\n",
+                        current->id, snr[0], snr[1], snr[2],
+                    psnr[0], psnr[1], psnr[2]);
+        }
     }
 }
 
 
+
 /*===========================================================================*
  *
  * ResetIFrameStats
  *
- *	reset the I-frame statistics
+ *  reset the I-frame statistics
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    none
  *
@@ -652,63 +655,63 @@ ResetIFrameStats()
 }
 
 
-/*===========================================================================*
- *
- * ShowIFrameSummary
- *
- *	prints out statistics on all I-frames
- *
- * RETURNS:	time taken for I-frames (in seconds)
- *
- * SIDE EFFECTS:    none
- *
- *===========================================================================*/
+
 float
-ShowIFrameSummary(inputFrameBits, totalBits, fpointer)
-    int inputFrameBits;
-    int32 totalBits;
-    FILE *fpointer;
-{
-    if ( numFrames == 0 ) {
-	return 0.0;
-    }
-
-    fprintf(fpointer, "-------------------------\n");
-    fprintf(fpointer, "*****I FRAME SUMMARY*****\n");
-    fprintf(fpointer, "-------------------------\n");
-
-    fprintf(fpointer, "  Blocks:    %5d     (%6d bits)     (%5d bpb)\n",
-	    numBlocks, numBits, numBits/numBlocks);
-    fprintf(fpointer, "  Frames:    %5d     (%6d bits)     (%5d bpf)     (%2.1f%% of total)\n",
-	    numFrames, numFrameBits, numFrameBits/numFrames,
-	    100.0*(float)numFrameBits/(float)totalBits);
-    fprintf(fpointer, "  Compression:  %3d:1     (%9.4f bpp)\n",
-	    numFrames*inputFrameBits/numFrameBits,
-	    24.0*(float)numFrameBits/(float)(numFrames*inputFrameBits));
-    if ( printSNR )
-	fprintf(fpointer, "  Avg Y SNR/PSNR:  %.1f     %.1f\n",
-		totalSNR/(float)numFrames, totalPSNR/(float)numFrames);
-    if ( totalTime == 0 ) {
-	fprintf(fpointer, "  Seconds:  NONE\n");
-    } else {
-	fprintf(fpointer, "  Seconds:  %9ld     (%9.4f fps)  (%9ld pps)  (%9ld mps)\n",
-		(long)(totalTime/TIME_RATE),
-		(float)((float)(TIME_RATE*numFrames)/(float)totalTime),
-		(long)((float)TIME_RATE*(float)numFrames*(float)inputFrameBits/(24.0*(float)totalTime)),
-		(long)((float)TIME_RATE*(float)numFrames*(float)inputFrameBits/(256.0*24.0*(float)totalTime)));
-    }
-
+IFrameTotalTime(void) {
     return (float)totalTime/(float)TIME_RATE;
 }
 
 
+
+void
+ShowIFrameSummary(unsigned int const inputFrameBits, 
+                  unsigned int const totalBits, 
+                  FILE *       const fpointer) {
+/*----------------------------------------------------------------------------
+   Print out statistics on all I frames.
+-----------------------------------------------------------------------------*/
+    if (numFrames > 0) {
+        fprintf(fpointer, "-------------------------\n");
+        fprintf(fpointer, "*****I FRAME SUMMARY*****\n");
+        fprintf(fpointer, "-------------------------\n");
+
+        fprintf(fpointer, "  Blocks:    %5d     (%6d bits)     (%5d bpb)\n",
+                numBlocks, numBits, numBits/numBlocks);
+        fprintf(fpointer, "  Frames:    %5d     (%6d bits)     (%5d bpf)"
+                "(%2.1f%% of total)\n",
+                numFrames, numFrameBits, numFrameBits/numFrames,
+                100.0*(float)numFrameBits/(float)totalBits);
+        fprintf(fpointer, "  Compression:  %3d:1     (%9.4f bpp)\n",
+                numFrames*inputFrameBits/numFrameBits,
+                24.0*(float)numFrameBits/(float)(numFrames*inputFrameBits));
+        if ( printSNR )
+            fprintf(fpointer, "  Avg Y SNR/PSNR:  %.1f     %.1f\n",
+                    totalSNR/(float)numFrames, totalPSNR/(float)numFrames);
+        if ( totalTime == 0 ) {
+            fprintf(fpointer, "  Seconds:  NONE\n");
+        } else {
+            fprintf(fpointer, "  Seconds:  %9ld     (%9.4f fps)  (%9ld pps)  "
+                    "(%9ld mps)\n",
+                    (long)(totalTime/TIME_RATE),
+                    (float)((float)(TIME_RATE*numFrames)/(float)totalTime),
+                    (long)((float)TIME_RATE * (float)numFrames *
+                           (float)inputFrameBits/(24.0*(float)totalTime)),
+                    (long)((float)TIME_RATE*(float)numFrames *
+                           (float)inputFrameBits/(256.0*24.0 *
+                                                  (float)totalTime)));
+        }
+    }
+}
+
+
+
 /*===========================================================================*
  *
  * EstimateSecondsPerIFrame
  *
- *	estimates the number of seconds required per I-frame
+ *  estimates the number of seconds required per I-frame
  *
- * RETURNS:	seconds (floating point value)
+ * RETURNS: seconds (floating point value)
  *
  * SIDE EFFECTS:    none
  *
@@ -724,9 +727,9 @@ EstimateSecondsPerIFrame()
  *
  * EncodeYDC
  *
- *	Encode the DC portion of a DCT of a luminance block
+ *  Encode the DC portion of a DCT of a luminance block
  *
- * RETURNS:	result appended to bb
+ * RETURNS: result appended to bb
  *
  * SIDE EFFECTS:    updates pred_term
  *
@@ -739,36 +742,36 @@ EncodeYDC(dc_term, pred_term, bb)
 {
     /* see Table B.5a -- MPEG-I doc */
     static int codes[9] = {
-	0x4, 0x0, 0x1, 0x5, 0x6, 0xe, 0x1e, 0x3e, 0x7e
+    0x4, 0x0, 0x1, 0x5, 0x6, 0xe, 0x1e, 0x3e, 0x7e
     };
     static int codeLengths[9] = {
-	3,   2,   2,   3,   3,   4,   5,    6,    7
+    3,   2,   2,   3,   3,   4,   5,    6,    7
     };
     int ydiff, ydiff_abs;
-    int	length;
+    int length;
 
     ydiff = (dc_term - (*pred_term));
     if (ydiff > 255) {
 #ifdef BLEAH 
       fprintf(stdout, "TRUNCATED\n");
 #endif
-	ydiff = 255;
+    ydiff = 255;
     } else if (ydiff < -255) {
 #ifdef BLEAH 
       fprintf(stdout, "TRUNCATED\n");
 #endif
-	ydiff = -255;
+    ydiff = -255;
     }
 
     ydiff_abs = ABS(ydiff);
     length = lengths[ydiff_abs];
     Bitio_Write(bb, codes[length], codeLengths[length]);
     if ( length != 0 ) {
-	if ( ydiff > 0 ) {
-	    Bitio_Write(bb, ydiff_abs, length);
-	} else {
-	    Bitio_Write(bb, ~ydiff_abs, length);
-	}
+    if ( ydiff > 0 ) {
+        Bitio_Write(bb, ydiff_abs, length);
+    } else {
+        Bitio_Write(bb, ~ydiff_abs, length);
+    }
     }
 
     (*pred_term) += ydiff;
@@ -779,9 +782,9 @@ EncodeYDC(dc_term, pred_term, bb)
  *
  * EncodeCDC
  *
- *	Encode the DC portion of a DCT of a chrominance block
+ *  Encode the DC portion of a DCT of a chrominance block
  *
- * RETURNS:	result appended to bb
+ * RETURNS: result appended to bb
  *
  * SIDE EFFECTS:    updates pred_term
  *
@@ -794,36 +797,36 @@ EncodeCDC(dc_term, pred_term, bb)
 {
     /* see Table B.5b -- MPEG-I doc */
     static int codes[9] = {
-	0x0, 0x1, 0x2, 0x6, 0xe, 0x1e, 0x3e, 0x7e, 0xfe
+    0x0, 0x1, 0x2, 0x6, 0xe, 0x1e, 0x3e, 0x7e, 0xfe
     };
     static int codeLengths[9] = {
-	2,   2,   2,   3,   4,   5,    6,    7,    8
+    2,   2,   2,   3,   4,   5,    6,    7,    8
     };
     int cdiff, cdiff_abs;
-    int	length;
+    int length;
 
     cdiff = (dc_term - (*pred_term));
     if (cdiff > 255) {
 #ifdef BLEAH
-fprintf(stdout, "TRUNCATED\n");	
+fprintf(stdout, "TRUNCATED\n"); 
 #endif
-	cdiff = 255;
+    cdiff = 255;
     } else if (cdiff < -255) {
 #ifdef BLEAH
-fprintf(stdout, "TRUNCATED\n");	
+fprintf(stdout, "TRUNCATED\n"); 
 #endif
-	cdiff = -255;
+    cdiff = -255;
     }
 
     cdiff_abs = ABS(cdiff);
     length = lengths[cdiff_abs];
     Bitio_Write(bb, codes[length], codeLengths[length]);
     if ( length != 0 ) {
-	if ( cdiff > 0 ) {
-	    Bitio_Write(bb, cdiff_abs, length);
-	} else {
-	    Bitio_Write(bb, ~cdiff_abs, length);
-	}
+    if ( cdiff > 0 ) {
+        Bitio_Write(bb, cdiff_abs, length);
+    } else {
+        Bitio_Write(bb, ~cdiff_abs, length);
+    }
     }
 
     (*pred_term) += cdiff;
@@ -836,11 +839,11 @@ BlockComputeSNR(current, snr, psnr)
      float snr[];
      float psnr[];
 {
-  register int32	tempInt;
+  register int32    tempInt;
   register int y, x;
-  int32	varDiff[3];
-  double	ratio[3];
-  double	total[3];
+  int32 varDiff[3];
+  double    ratio[3];
+  double    total[3];
   register uint8 **origY=current->orig_y, **origCr=current->orig_cr, 
   **origCb=current->orig_cb;
   register uint8 **newY=current->decoded_y, **newCr=current->decoded_cr, 
@@ -870,7 +873,7 @@ BlockComputeSNR(current, snr, psnr)
     SignalCr = (int32 **) malloc(ysz);
     NoiseCr  = (int32 **) malloc(ysz);
     if (SignalY == NULL || NoiseY == NULL || SignalCr == NULL || 
-	NoiseCb == NULL || SignalCb == NULL || NoiseCr == NULL) {
+    NoiseCb == NULL || SignalCb == NULL || NoiseCr == NULL) {
       fprintf(stderr, "Out of memory in BlockComputeSNR\n");
       exit(-1);
     }
@@ -952,7 +955,7 @@ WriteDecodedFrame(frame)
 {
     FILE    *fpointer;
     char    fileName[256];
-    int	width, height;
+    int width, height;
     register int y;
 
     /* need to save decoded frame to disk because it might be accessed
@@ -964,23 +967,23 @@ WriteDecodedFrame(frame)
     sprintf(fileName, "%s.decoded.%d", outputFileName, frame->id);
 
     if (!realQuiet) {
-	fprintf(stdout, "Outputting to %s\n", fileName);
-	fflush(stdout);
+    fprintf(stdout, "Outputting to %s\n", fileName);
+    fflush(stdout);
     }
 
     fpointer = fopen(fileName, "wb");
 
-	for ( y = 0; y < height; y++ ) {
-	    fwrite(frame->decoded_y[y], 1, width, fpointer);
-	}
+    for ( y = 0; y < height; y++ ) {
+        fwrite(frame->decoded_y[y], 1, width, fpointer);
+    }
 
-	for (y = 0; y < (height >> 1); y++) {			/* U */
-	    fwrite(frame->decoded_cb[y], 1, width >> 1, fpointer);
-	}
+    for (y = 0; y < (height >> 1); y++) {           /* U */
+        fwrite(frame->decoded_cb[y], 1, width >> 1, fpointer);
+    }
 
-	for (y = 0; y < (height >> 1); y++) {			/* V */
-	    fwrite(frame->decoded_cr[y], 1, width >> 1, fpointer);
-	}
+    for (y = 0; y < (height >> 1); y++) {           /* V */
+        fwrite(frame->decoded_cr[y], 1, width >> 1, fpointer);
+    }
     fflush(fpointer);
     fclose(fpointer);
 }
@@ -988,23 +991,23 @@ WriteDecodedFrame(frame)
 
 void
 PrintItoIBitRate(numBits, frameNum)
-    int	    numBits;
-    int	    frameNum;
+    int     numBits;
+    int     frameNum;
 {
-    if ( ( ! childProcess) && showBitRatePerFrame ) {
-	/* ASSUMES 30 FRAMES PER SECOND */
+    if ( showBitRatePerFrame ) {
+    /* ASSUMES 30 FRAMES PER SECOND */
 
-	if (! realQuiet) {
-	fprintf(stdout, "I-to-I (frames %5d to %5d) bitrate:  %8d\n",
-		lastIFrame, frameNum-1,
-		((numBits-lastNumBits)*30)/
-		(frameNum-lastIFrame));
+    if (! realQuiet) {
+    fprintf(stdout, "I-to-I (frames %5d to %5d) bitrate:  %8d\n",
+        lastIFrame, frameNum-1,
+        ((numBits-lastNumBits)*30)/
+        (frameNum-lastIFrame));
         }
 
-	fprintf(bitRateFile, "I-to-I (frames %5d to %5d) bitrate:  %8d\n",
-		lastIFrame, frameNum-1,
-		((numBits-lastNumBits)*30)/
-		(frameNum-lastIFrame));
+    fprintf(bitRateFile, "I-to-I (frames %5d to %5d) bitrate:  %8d\n",
+        lastIFrame, frameNum-1,
+        ((numBits-lastNumBits)*30)/
+        (frameNum-lastIFrame));
     }
 }
 
@@ -1013,9 +1016,9 @@ PrintItoIBitRate(numBits, frameNum)
  *
  * AllocDctBlocks
  *
- *	allocate memory for dct blocks
+ *  allocate memory for dct blocks
  *
- * RETURNS:	nothing
+ * RETURNS: nothing
  *
  * SIDE EFFECTS:    creates dct, dctr, dctb
  *
@@ -1032,15 +1035,15 @@ AllocDctBlocks()
     dct = (Block **) malloc(sizeof(Block *) * dcty);
     ERRCHK(dct, "malloc");
     for (i = 0; i < dcty; i++) {
-	dct[i] = (Block *) malloc(sizeof(Block) * dctx);
-	ERRCHK(dct[i], "malloc");
+    dct[i] = (Block *) malloc(sizeof(Block) * dctx);
+    ERRCHK(dct[i], "malloc");
     }
 
     dct_data = (dct_data_type **) malloc(sizeof(dct_data_type *) * dcty);
     ERRCHK(dct_data, "malloc");
     for (i = 0; i < dcty; i++) {
-	dct_data[i] = (dct_data_type *) malloc(sizeof(dct_data_type) * dctx);
-	ERRCHK(dct[i], "malloc");
+    dct_data[i] = (dct_data_type *) malloc(sizeof(dct_data_type) * dctx);
+    ERRCHK(dct[i], "malloc");
     }
 
     dctr = (Block **) malloc(sizeof(Block *) * (dcty >> 1));
@@ -1048,10 +1051,10 @@ AllocDctBlocks()
     ERRCHK(dctr, "malloc");
     ERRCHK(dctb, "malloc");
     for (i = 0; i < (dcty >> 1); i++) {
-	dctr[i] = (Block *) malloc(sizeof(Block) * (dctx >> 1));
-	dctb[i] = (Block *) malloc(sizeof(Block) * (dctx >> 1));
-	ERRCHK(dctr[i], "malloc");
-	ERRCHK(dctb[i], "malloc");
+    dctr[i] = (Block *) malloc(sizeof(Block) * (dctx >> 1));
+    dctb[i] = (Block *) malloc(sizeof(Block) * (dctx >> 1));
+    ERRCHK(dctr[i], "malloc");
+    ERRCHK(dctb[i], "malloc");
     }
 }
 
