@@ -324,7 +324,7 @@ getHostName(void) {
 
     rc = uname(&utsname);
 
-    if (rc != 0)
+    if (rc < 0)
         pm_error("Unable to find out host name.  "
                  "uname() failed with errno %d (%s)", errno, strerror(errno));
 
