@@ -34,12 +34,15 @@
 */
 
 #define _BSD_SOURCE   /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include "ppm.h"
 #include "shhopt.h"
 #include "nstring.h"
+#include "mallocvar.h"
 
 /* Max number of entries we will put in the XPM color map 
    Don't forget the one entry for transparency.
@@ -232,8 +235,8 @@ genCmap(colorhist_vector const chv,
     int cmapIndex;
     int mval;
     cixel_map * cmap;  /* malloc'ed */
-
-    cmap = malloc(sizeof(cixel_map) * (ncolors+1));
+    
+    MALLOCARRAY(cmap, ncolors + 1);
     if (cmapP == NULL)
         pm_error("Out of memory allocating %d bytes for a color map.",
                  sizeof(cixel_map) * (ncolors+1));
