@@ -10,14 +10,31 @@
 ** implied warranty.
 */
 
-/* 
-   There is some fancy virtual memory management in this program to avoid
+/*
+   transformGen() is the general transformation function.
+   
+   The following are enhancements for specific cases:
+   
+     transformRowByRowPbm()
+     transformRowsBottomTopPbm()
+     transformRowByRowNonPbm()
+     transformRowsBottomTopNonPbm()
+     transformPbm()
+   
+   Although transformGen() is used only when none of the enhancement
+   functions apply, it is capable of handling all cases.  (Only that it
+   is slow, and uses more memory.)  In the same manner, transformPbm() is
+   capable of handling all pbm transformations and transformRowByRowNonPbm()
+   transformRowsBottomTomNonPbm() are capable of handling pbm.
+
+
+   There is some fancy virtual memory management in transformGen() to avoid
    page thrashing when you flip a large image in a columns-for-rows
    way (e.g. -transpose).
    
    The page thrashing we're trying to avoid could happen because the
-   output of the transformation is stored in an array of xels in
-   virtual memory.  An xel array is stored in column-first order,
+   output of the transformation is stored in an array of tuples in
+   virtual memory.  A tuple array is stored in column-first order,
    meaning that all the columns of particular row are contiguous, the
    next row is next to that, etc.  If you fill up that array by
    filling in Column 0 sequentially in every row from top to bottom,
@@ -26,7 +43,7 @@
 
    If the number of virtual memory pages you touch exceeds the amount
    of real memory the process can get, then by the time you hit the bottom
-   of the xel array, the pages that hold the top are already paged out.
+   of the tuple array, the pages that hold the top are already paged out.
    So if you go back and do Column 1 from top to bottom, you will again
    touch lots of pages and have to page in every one of them.  Do this 
    for 100 columns, and you might page in every page in the array 100 times
@@ -38,7 +55,7 @@
    that by doing Column 0 from top to say Row 10, then Column 1 from top
    to Row 10, etc. all the way across the image.  Assuming 10 rows fits
    in real memory, you will keep the virtual memory for the first 10 rows
-   of the xel array in real memory until you've filled them in completely.
+   of the tuple array in real memory until you've filled them in completely.
    Now you go back and do Column 0 from Row 11 to Row 20, then Column 1
    from Row 11 to Row 20, and so on.
 
@@ -59,14 +76,30 @@
    only in one burst, input pages do not compete with output pages for
    real memory -- the working set is the output pages, which get referenced
    cyclically.
+
+   This all worked when we used the pnm xel format, but now that we
+   use the pam tuple format, there's an extra memory reference that
+   could be causing trouble.  Because tuples have varying depth, a pam
+   row is an array of pointers to the tuples.  To access a tuple, we
+   access the tuple pointer, then the tuple.  We could probably do better,
+   because the samples are normally in the memory immediately following
+   the tuple pointers, so we could compute where a tuple's samples live
+   without actually loading the tuple address from memory.  I.e. the 
+   address of the tuple for Column 5 of Row 9 of a 3-deep 100-wide
+   image is (void*)tuples[9] + 100 * sizeof(tuple*) + 5*(3*sizeof(sample)).
 */
 
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
+
 #include <limits.h>
 #include <string.h>
 
 #include "pam.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 #include "nstring.h"
+#include "bitreverse.h"
 
 enum xformType {LEFTRIGHT, TOPBOTTOM, TRANSPOSE};
 
@@ -110,17 +143,17 @@ parseXformOpt(const char *     const xformOpt,
         const char * token;
         token = strsepN(&cursor, ",");
         if (token) {
-                if (strcmp(token, "leftright") == 0)
-                    xformList[xformCount++] = LEFTRIGHT;
-                else if (strcmp(token, "topbottom") == 0)
-                    xformList[xformCount++] = TOPBOTTOM;
-                else if (strcmp(token, "transpose") == 0)
-                    xformList[xformCount++] = TRANSPOSE;
-                else if (strcmp(token, "") == 0)
-                    { /* ignore it */}
-                else
-                    pm_error("Invalid transform type in -xform option: '%s'",
-                             token );
+            if (strcmp(token, "leftright") == 0)
+                xformList[xformCount++] = LEFTRIGHT;
+            else if (strcmp(token, "topbottom") == 0)
+                xformList[xformCount++] = TOPBOTTOM;
+            else if (strcmp(token, "transpose") == 0)
+                xformList[xformCount++] = TRANSPOSE;
+            else if (strcmp(token, "") == 0)
+            { /* ignore it */}
+            else
+                pm_error("Invalid transform type in -xform option: '%s'",
+                         token );
         } else
             eol = TRUE;
     }
@@ -130,6 +163,7 @@ parseXformOpt(const char *     const xformOpt,
 }
 
 
+
 static void
 parseCommandLine(int argc, char ** const argv,
                  struct cmdlineInfo * const cmdlineP) {
@@ -314,62 +348,344 @@ computeXformMatrix(struct xformMatrix * const xformP,
 
 
 static void
-transformRowByRow(struct pam *       const inpamP, 
-                  struct pam *       const outpamP,
-                  struct xformMatrix const xform) {
-    tuple* tuplerow;
-    tuple* newtuplerow;
-    int row;
+bitOrderReverse(unsigned char * const bitrow, 
+                unsigned int    const cols) {
+/*----------------------------------------------------------------------------
+  Reverse the bits in a packed pbm row (1 bit per pixel).  I.e. the leftmost
+  bit becomes the rightmost, etc.
+-----------------------------------------------------------------------------*/
+    unsigned int const lastfullByteIdx = cols/8 -1;
+
+    if (cols == 0 || bitrow == NULL )
+        pm_error("Invalid arguments passed to bitOrderReverse");
+
+    if (cols <= 8)
+        bitrow[0] = bitreverse[bitrow[0]] << (8-cols);
+    else if (cols%8 == 0) {
+        unsigned int i, j;
+        for (i = 0, j = lastfullByteIdx; i <= j; ++i, --j) {
+            unsigned char const t = bitreverse[bitrow[j]]; 
+            bitrow[j]=bitreverse[bitrow[i]];
+            bitrow[i]=t;
+        }
+    } else {
+        unsigned int const m = cols%8; 
+
+        unsigned int i,j;
+            /* Cursors into bitrow[].  i moves from left to center;
+               j moves from right to center as bits of bitrow[] are exchanged.
+            */
+        unsigned char th,tl;  /* 16 bit temp ( th << 8 | tl ) */
+        tl = 0;
+        for (i = 0, j = lastfullByteIdx; i <= lastfullByteIdx/2; ++i, --j) {
+            th = bitreverse[bitrow[i]];
+            bitrow[i] =
+                bitreverse[0xff & ((bitrow[j]<<8 | bitrow[j+1]) >> (8-m))];
+            bitrow[j+1] = 0xff & ((th<<8 | tl) >> m);
+            tl=th;
+        }
+        if (i > j) 
+            /* bitrow[] has an even number of full bytes (lastfullByteIdx is
+               odd), so we overlapped above.  Now we correct.
+            */
+            bitrow[j+1] = 0xff & ((bitreverse[bitrow[i]]<<8|tl ) >> m );
+    }
+}
+
+
+
+static void
+transformRowByRowPbm(struct pam * const inpamP, 
+                     struct pam * const outpamP,
+                     bool         const reverse) {
+/*----------------------------------------------------------------------------
+  Transform a PBM image either by flipping it left for right, or just leaving
+  it alone, as indicated by 'reverse'.
+
+  Process the image one row at a time and use fast packed PBM bit
+  reverse algorithm (where required).
+-----------------------------------------------------------------------------*/
+    unsigned char * bitrow;
+    unsigned int row;
+
+    bitrow = pbm_allocrow_packed(outpamP->width); 
+
+    pbm_writepbminit(outpamP->file, outpamP->width, outpamP->height, 0);
+
+    for (row = 0; row < inpamP->height; ++row) {
+        pbm_readpbmrow_packed(inpamP->file,  bitrow, inpamP->width,
+                              inpamP->format);
+
+        if (reverse)
+            bitOrderReverse(bitrow, inpamP->width);
+
+        pbm_writepbmrow_packed(outpamP->file, bitrow, outpamP->width, 0);
+    }
+    pbm_freerow_packed(bitrow);
+}
+
+
+
+static void
+transformRowByRowNonPbm(struct pam * const inpamP, 
+                        struct pam * const outpamP,
+                        bool         const reverse) {
+/*----------------------------------------------------------------------------
+  Flip an image left for right or leave it alone.
+
+  Process one row at a time.
+
+  This works on any image, but is slower and uses more memory than the
+  PBM-only transformRowByRowPbm().
+-----------------------------------------------------------------------------*/
+    tuple * tuplerow;
+    tuple * newtuplerow;
+        /* This is not a full tuple row.  It is either an array of pointers
+           to the tuples in 'tuplerow' (in reverse order) or just 'tuplerow'
+           itself.
+        */
+    tuple * scratchTuplerow;
+    
+    unsigned int row;
     
     tuplerow = pnm_allocpamrow(inpamP);
-    newtuplerow = pnm_allocpamrow(outpamP);
-    pnm_writepaminit(outpamP);
     
-    for (row = 0; row < inpamP->height; ++row) {
-        int col;
+    if (reverse) {
+        /* Set up newtuplerow[] to point to the tuples of tuplerow[] in
+           reverse order.
+        */
+        unsigned int col;
+        
+        MALLOCARRAY_NOFAIL(scratchTuplerow, inpamP->width);
+
+        for (col = 0; col < inpamP->width; ++col) 
+            scratchTuplerow[col] = tuplerow[inpamP->width - col - 1];
+        newtuplerow = scratchTuplerow;
+    } else {
+        scratchTuplerow = NULL;
+        newtuplerow = tuplerow;
+    }
+    pnm_writepaminit(outpamP);
+
+    for (row = 0; row < inpamP->height ; ++row) {
         pnm_readpamrow(inpamP, tuplerow);
-        for (col = 0; col < inpamP->width; ++col) {
-            /* Transform a point:
-            **
-            **            [ a b 0 ]
-            **  [ x y 1 ] [ c d 0 ] = [ x2 y2 1 ]
-            **            [ e f 1 ]
-            */
-            int const newcol = xform.a * col + xform.c * row + 
-                xform.e * (outpamP->width - 1);
-            pnm_assigntuple(inpamP, newtuplerow[newcol], tuplerow[col]);
-        }
         pnm_writepamrow(outpamP, newtuplerow);
     }
+    
+    if (scratchTuplerow)
+        free(scratchTuplerow);
     pnm_freepamrow(tuplerow);
-    pnm_freepamrow(newtuplerow);
+}
+
+
+
+static void
+transformRowByRow(struct pam * const inpamP,
+                  struct pam * const outpamP,
+                  bool         const reverse,
+                  bool         const verbose) {
+
+    if (verbose)
+        pm_message("Transforming row by row, top to bottom");
+
+    switch (PNM_FORMAT_TYPE(inpamP->format)) {
+    case PBM_TYPE:
+        transformRowByRowPbm(inpamP, outpamP, reverse);
+        break;
+    default:
+        transformRowByRowNonPbm(inpamP, outpamP, reverse);
+        break;
+    }
+} 
+
+
+
+static void
+transformRowsBottomTopPbm(struct pam * const inpamP,
+                          struct pam * const outpamP,
+                          bool         const reverse) { 
+/*----------------------------------------------------------------------------
+  Flip a PBM image top for bottom.  Iff 'reverse', also flip it left for right.
+
+  Read complete image into memory in packed PBM format; Use fast
+  packed PBM bit reverse algorithm (where required).
+-----------------------------------------------------------------------------*/
+    unsigned int const rows=inpamP->height;
+
+    unsigned char ** bitrow;
+    int row;
+        
+    bitrow = pbm_allocarray_packed(outpamP->width, outpamP->height);
+        
+    for (row = 0; row < rows; ++row)
+        pbm_readpbmrow_packed(inpamP->file, bitrow[row], 
+                              inpamP->width, inpamP->format);
+
+    pbm_writepbminit(outpamP->file, outpamP->width, outpamP->height, 0);
+
+    for (row = 0; row < rows; ++row) {
+        if (reverse) 
+            bitOrderReverse(bitrow[rows-row-1], inpamP->width);
+
+        pbm_writepbmrow_packed(outpamP->file, bitrow[rows - row - 1],
+                               outpamP->width, 0);
+    }
+    pbm_freearray_packed(bitrow, outpamP->height);
+}
+
+
+
+static void
+transformRowsBottomTopNonPbm(struct pam * const inpamP, 
+                             struct pam * const outpamP,
+                             bool         const reverse) {
+/*----------------------------------------------------------------------------
+  Read complete image into memory as a tuple array.
+
+  This can do any transformation except a column-for-row transformation,
+  on any type of image, but is slower and uses more memory than the 
+  PBM-only transformRowsBottomTopPbm().
+-----------------------------------------------------------------------------*/
+    tuple** tuplerows;
+    tuple * scratchTuplerow;
+        /* This is not a full tuple row -- just an array of pointers to
+           the tuples in 'tuplerows'.
+        */
+    unsigned int row;
+
+    if (reverse)
+        MALLOCARRAY_NOFAIL(scratchTuplerow, inpamP->width);
+    else
+        scratchTuplerow = NULL;
+  
+    tuplerows = pnm_allocpamarray(outpamP);
+
+    for (row = 0; row < inpamP->height ; ++row)
+        pnm_readpamrow(inpamP, tuplerows[row]);
+
+    pnm_writepaminit(outpamP);
+
+    for (row = 0; row < inpamP->height ; ++row) {
+        tuple * newtuplerow;
+        tuple * const tuplerow = tuplerows[inpamP->height - row - 1];
+        if (reverse) {
+            unsigned int col;
+            newtuplerow = scratchTuplerow;
+            for (col = 0; col < inpamP->width; ++col) 
+                newtuplerow[col] = tuplerow[inpamP->width - col - 1];
+        } else
+            newtuplerow = tuplerow;
+        pnm_writepamrow(outpamP, newtuplerow);
+    }
+
+    if (scratchTuplerow)
+        free(scratchTuplerow);
+    
+    pnm_freepamarray(tuplerows, outpamP);
+}
+
+
+
+static void
+transformRowsBottomTop(struct pam * const inpamP,
+                       struct pam * const outpamP,
+                       bool         const reverse,
+                       bool         const verbose) {
+
+    if (PNM_FORMAT_TYPE(inpamP->format) == PBM_TYPE) {
+        if (verbose)
+            pm_message("Transforming PBM row by row, bottom to top");
+        transformRowsBottomTopPbm(inpamP, outpamP, reverse);
+    } else {
+        if (verbose)
+            pm_message("Transforming non-PBM row by row, bottom to top");
+        transformRowsBottomTopNonPbm(inpamP, outpamP, reverse);
+    }
 }
 
 
 
 static void __inline__
-transformPoint( int const col, int const newcols,
-                int const row, int const newrows, 
-                struct xformMatrix const xform, 
-                int * const newcolP, int * const newrowP ) {
-    /* Transform a point:
-    **
-    **            [ a b 0 ]
-    **  [ x y 1 ] [ c d 0 ] = [ x2 y2 1 ]
-    **            [ e f 1 ]
+transformPoint(int                const col, 
+               int                const newcols,
+               int                const row, 
+               int                const newrows, 
+               struct xformMatrix const xform, 
+               int *              const newcolP, 
+               int *              const newrowP ) {
+/*----------------------------------------------------------------------------
+   Compute the location in the output of a pixel that is at row 'row',
+   column 'col' in the input.  Assume the output image is 'newcols' by
+   'newrows' and the transformation is as described by 'xform'.
+
+   Return the output image location of the pixel as *newcolP and *newrowP.
+-----------------------------------------------------------------------------*/
+    /* The transformation is:
+     
+                 [ a b 0 ]
+       [ x y 1 ] [ c d 0 ] = [ x2 y2 1 ]
+                 [ e f 1 ]
     */
-    *newcolP = xform.a * col + xform.c * row + 
-        xform.e * ( newcols - 1 );
-    *newrowP = xform.b * col + xform.d * row + 
-        xform.f * ( newrows - 1 );
+    *newcolP = xform.a * col + xform.c * row + xform.e * (newcols - 1);
+    *newrowP = xform.b * col + xform.d * row + xform.f * (newrows - 1);
+}
+
+
+
+static void
+transformPbm(struct pam *       const inpamP,
+             struct pam *       const outpamP,
+             struct xformMatrix const xform) { 
+/*----------------------------------------------------------------------------
+   This is the same as transformGen, except that it uses less 
+   memory, since the PBM buffer format uses one bit per pixel instead
+   of twelve bytes + pointer space
+
+   This can do any PBM transformation, but is slower and uses more
+   memory than the more restricted transformRowByRowPbm() and
+   transformRowsBottomTopPbm().
+-----------------------------------------------------------------------------*/
+    bit* bitrow;
+    bit** newbits;
+    int row;
+            
+    bitrow = pbm_allocrow(inpamP->width);
+    newbits = pbm_allocarray(pbm_packed_bytes(outpamP->width), 
+                             outpamP->height);
+            
+    /* Initialize entire array to zeroes.  One bits will be or'ed in later */
+    for (row = 0; row < outpamP->height; ++row) {
+        int col;
+        for (col = 0; col < pbm_packed_bytes(outpamP->width); ++col) 
+             newbits[row][col] = 0; 
+    }
+    
+    for (row = 0; row < inpamP->height; ++row) {
+        int col;
+        pbm_readpbmrow(inpamP->file, bitrow, inpamP->width, inpamP->format);
+        for (col = 0; col < inpamP->width; ++col) {
+            int newcol, newrow;
+            transformPoint(col, outpamP->width, row, outpamP->height, xform,
+                           &newcol, &newrow);
+            newbits[newrow][newcol/8] |= bitrow[col] << (7 - newcol % 8);
+                /* Use of "|=" patterned after pbm_readpbmrow_packed. */
+         }
+    }
+    
+    pbm_writepbminit(outpamP->file, outpamP->width, outpamP->height, 0);
+    for (row = 0; row < outpamP->height; ++row)
+        pbm_writepbmrow_packed(outpamP->file, newbits[row], outpamP->width, 
+                               0);
+    
+    pbm_freearray(newbits, outpamP->height);
+    pbm_freerow(bitrow);
 }
 
 
 
 static unsigned int 
 optimalSegmentSize(struct xformMatrix  const xform, 
-                   int                 const outputCols,
-                   int                 const format,
+                   struct pam *        const pamP,
                    unsigned int        const availableMemory,
                    unsigned int        const pageSize) {
 /*----------------------------------------------------------------------------
@@ -385,31 +701,28 @@ optimalSegmentSize(struct xformMatrix  const xform,
    'pageSize' is the size of a page in bytes.  A page means the unit that
    is paged in or out.
    
-   'outputCols' is the number of columns in each row of the output array.
-
-   'format' is the format of the output array (PPM, etc.).
+   'pamP' describes the storage required to represent a row of the
+   output array.
 -----------------------------------------------------------------------------*/
     unsigned int segmentSize;
 
     if (xform.b == 0)
-        segmentSize = UINT_MAX;
+        segmentSize = pamP->width;
     else {
         unsigned int const otherNeeds = 200*1024;
             /* A wild guess at how much real memory is needed by the program
-               for purposes other than the output xel array.
+               for purposes other than the output tuple array.
             */
         if (otherNeeds > availableMemory)
-            segmentSize = UINT_MAX;  /* Can't prevent thrashing */
+            segmentSize = pamP->width;  /* Can't prevent thrashing */
         else {
             unsigned int const availablePages = 
                 (availableMemory - otherNeeds) / pageSize;
             if (availablePages <= 1)
-                segmentSize = UINT_MAX; /* Can't prevent thrashing */
+                segmentSize = pamP->width; /* Can't prevent thrashing */
             else {
-                unsigned int bytesPerXel = 
-                    PNM_FORMAT_TYPE(format) == PBM_TYPE ? 1 : sizeof(xelval)*3;
                 unsigned int const bytesPerRow = 
-                    outputCols * bytesPerXel;
+                    pamP->width * pamP->depth * pamP->bytes_per_sample;
                 unsigned int rowsPerPage = 
                     MAX(1, (pageSize + (pageSize/2)) / bytesPerRow);
                     /* This is how many consecutive rows we can touch
@@ -446,6 +759,9 @@ transformNonPbm(struct pam *       const inpamP,
 
   if 'segmentSize' is less than the whole image, ifP must be a seekable
   file.
+
+  This can do any transformation, but is slower and uses more memory
+  than the PBM-only transformPbm().
 -----------------------------------------------------------------------------*/
     pm_filepos imagepos;
         /* The input file position of the raster.  But defined only if
@@ -500,49 +816,40 @@ transformNonPbm(struct pam *       const inpamP,
 
 
 static void
-transformPbm(struct pam *       const inpamP,
+transformGen(struct pam *       const inpamP,
              struct pam *       const outpamP,
-             struct xformMatrix const xform) { 
+             struct xformMatrix const xform,
+             unsigned int       const availableMemory,
+             unsigned int       const pageSize,
+             bool               const verbose) {
 /*----------------------------------------------------------------------------
-   This is the same as transformNonPbm, except that it uses less 
-   memory, since the PBM buffer format uses one bit per pixel instead
-   of twelve bytes
+  Produce the transformed output on Standard Output.
+
+  Assume input file is positioned to the raster (just after the
+  header).
+
+  This can transform any image in any way, but is slower and uses more
+  memory than the more restricted transformRowByRow() and
+  transformRowsBottomTop().
 -----------------------------------------------------------------------------*/
-    bit* bitrow;
-    bit** newbits;
-    int row;
-            
-    bitrow = pbm_allocrow(inpamP->width);
-    newbits = pbm_allocarray(pbm_packed_bytes(outpamP->width), 
-                             outpamP->height);
-            
-    /* Initialize entire array to zeroes.  One bits will be or'ed in later */
-    for (row = 0; row < outpamP->height; ++row) {
-        int col;
-        for (col = 0; col < pbm_packed_bytes(outpamP->width); ++col) 
-             newbits[row][col] = 0; 
+    unsigned int const segmentSize = 
+        optimalSegmentSize(xform, outpamP, availableMemory, pageSize);
+    
+    switch (PNM_FORMAT_TYPE(inpamP->format)) {
+    case PBM_TYPE: 
+        transformPbm(inpamP, outpamP, xform);
+        break;
+    default:
+        if (segmentSize < outpamP->width) {
+            if (verbose && xform.b !=0)
+                pm_message("Transforming %u columns of %u total at a time", 
+                           segmentSize, outpamP->width);
+            else
+                pm_message("Transforming entire image at once");
+        }
+        transformNonPbm(inpamP, outpamP, xform, segmentSize, verbose);
+        break;
     }
-    
-    for (row = 0; row < inpamP->height; ++row) {
-        int col;
-        pbm_readpbmrow(inpamP->file, bitrow, inpamP->width, inpamP->format);
-        for (col = 0; col < inpamP->width; ++col) {
-            int newcol, newrow;
-            transformPoint(col, outpamP->width, row, outpamP->height, xform,
-                           &newcol, &newrow);
-            newbits[newrow][newcol/8] |= bitrow[col] << (7 - newcol % 8);
-                /* Use of "|=" patterned after pbm_readpbmrow_packed. */
-         }
-    }
-    
-    pbm_writepbminit(outpamP->file, outpamP->width, outpamP->height, 
-                     outpamP->plainformat);
-    for (row = 0; row < outpamP->height; ++row)
-        pbm_writepbmrow_packed(outpamP->file, newbits[row], outpamP->width, 
-                               outpamP->plainformat);
-    
-    pbm_freearray(newbits, outpamP->height);
-    pbm_freerow(bitrow);
 }
 
 
@@ -564,7 +871,7 @@ main(int argc, char * argv[]) {
     else
         ifP = pm_openr(cmdline.inputFilespec);
     
-    pnm_readpaminit(ifP, &inpam, sizeof(inpam));
+    pnm_readpaminit(ifP, &inpam, PAM_STRUCT_SIZE(tuple_type));
 
     computeXformMatrix(&xform, cmdline.xformCount, cmdline.xformList);
 
@@ -574,43 +881,29 @@ main(int argc, char * argv[]) {
     outpam.width  = abs(xform.a) * inpam.width + abs(xform.c) * inpam.height;
     outpam.height = abs(xform.b) * inpam.width + abs(xform.d) * inpam.height;
     
-    if (xform.b == 0 && xform.d == 1 && xform.f == 0) {
-        /* In this case newrow is always equal to row, so we can do the
-           transform line by line and avoid in-memory buffering altogether.
+    if (xform.b == 0 && xform.d == 1 && xform.f == 0)
+        /* In this case Row N of the output is based only on Row N of
+           the input, so we can transform row by row and avoid
+           in-memory buffering altogether.  
         */
-        if (cmdline.verbose)
-            pm_message("Transforming row by row");
-        transformRowByRow(&inpam, &outpam, xform);
-	} else {
-        unsigned int const segmentSize = 
-            optimalSegmentSize(xform, outpam.width, outpam.format, 
-                               cmdline.availableMemory, cmdline.pageSize);
-    
-        if (cmdline.verbose) {
-            if (segmentSize < outpam.width)
-                pm_message("Transforming %u columns of %u total at a time", 
-                           segmentSize, outpam.width);
-            else
-                pm_message("Transforming entire image at once");
-        }
+        transformRowByRow(&inpam, &outpam, xform.a == -1, cmdline.verbose);
+    else if (xform.b == 0 && xform.c == 0) 
+        /* In this case, Row N of the output is based only on Row ~M of the
+           input.  We need all the rows in memory, but have to pass
+           through them only twice, so there is no page thrashing concern.
+        */
+        transformRowsBottomTop(&inpam, &outpam, xform.a == -1, 
+                               cmdline.verbose);
+    else
+        /* This is a colum-for-row type of transformation, which requires
+           complex traversal of an in-memory image.
+        */
+        transformGen(&inpam, &outpam, xform,
+                     cmdline.availableMemory, cmdline.pageSize, 
+                     cmdline.verbose);
 
-        /* Generic case.  Read in the anymap a line at a time and transform
-           it into an in-memory array.
-        */
-        switch (PNM_FORMAT_TYPE(inpam.format)) {
-        case PBM_TYPE: 
-            transformPbm(&inpam, &outpam, xform);
-            break;
-        default:
-            transformNonPbm(&inpam, &outpam, xform, segmentSize, 
-                            cmdline.verbose);
-            break;
-        }
-	}
-    
     pm_close(inpam.file);
     pm_close(outpam.file);
     
-    exit(0);
+    return 0;
 }
-
