@@ -153,10 +153,10 @@ applyFunction(struct cmdlineInfo const cmdline,
                 outSample = ROUNDU(inSample / cmdline.u.divisor);
                 break;
             case FN_ADD:
-                outSample = MAX(0, inSample + cmdline.u.adder);
+                outSample = MAX(0, (long)inSample + cmdline.u.adder);
                 break;
             case FN_SUBTRACT:
-                outSample = MAX(0, inSample - cmdline.u.subtractor);
+                outSample = MAX(0, (long)inSample - cmdline.u.subtractor);
                 break;
             case FN_MAX:
                 outSample = MAX(inSample, cmdline.u.min);
