@@ -29,6 +29,7 @@
 
 #include "pnm.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 
 enum brightMethod {BRIGHT_MAX, BRIGHT_LUMIN};
 
@@ -39,9 +40,11 @@ struct cmdlineInfo {
     const char *inputFilespec;  /* Filespec of input file */
     unsigned int bvalueSpec;
     xelval bvalue;
+    unsigned int bpercentSpec;
     float bpercent;
     unsigned int wvalueSpec;
     xelval wvalue;
+    unsigned int wpercentSpec;
     float wpercent;
     enum brightMethod brightMethod;
     unsigned int keephues;
@@ -67,16 +70,15 @@ parseCommandLine (int argc, char ** argv,
          */
     optStruct3 opt;
 
-    unsigned int bpercentSpec, wpercentSpec;
     unsigned int brightmax;
 
     unsigned int option_def_index;
 
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0,   "bpercent",      OPT_FLOAT,   
-            &cmdlineP->bpercent,   &bpercentSpec, 0);
+            &cmdlineP->bpercent,   &cmdlineP->bpercentSpec, 0);
     OPTENT3(0,   "wpercent",      OPT_FLOAT,   
-            &cmdlineP->wpercent,   &wpercentSpec, 0);
+            &cmdlineP->wpercent,   &cmdlineP->wpercentSpec, 0);
     OPTENT3(0,   "bvalue",        OPT_UINT,   
             &cmdlineP->bvalue,     &cmdlineP->bvalueSpec, 0);
     OPTENT3(0,   "wvalue",        OPT_UINT,   
@@ -93,15 +95,9 @@ parseCommandLine (int argc, char ** argv,
     optParseOptions3( &argc, argv, opt, sizeof(opt), 0 );
         /* Uses and sets argc, argv, and some of *cmdline_p and others. */
 
-    if(wpercentSpec && cmdlineP->wvalueSpec)
-        pm_error("You may specify only one of -wpercent and -wvalue");
-
-    if(bpercentSpec && cmdlineP->bvalueSpec)
-        pm_error("You may specify only one of -bpercent and -bvalue");
-
-    if (!wpercentSpec)
+    if (!cmdlineP->wpercentSpec)
         cmdlineP->wpercent = 1.0;
-    if (!bpercentSpec)
+    if (!cmdlineP->bpercentSpec)
         cmdlineP->bpercent = 2.0;
 
     if (cmdlineP->wpercent < 0.0)
@@ -263,23 +259,33 @@ computeEndValues(FILE *   const ifp,
 -----------------------------------------------------------------------------*/
     unsigned int * hist;  /* malloc'ed */
 
-    hist = malloc((PNM_OVERALLMAXVAL+1)*sizeof(hist[0]));
+    MALLOCARRAY(hist, PNM_OVERALLMAXVAL+1);
 
     if (hist == NULL)
         pm_error("Unable to allocate storage for intensity histogram.");
     else {
         buildHistogram(ifp, cols, rows, maxval, format, hist, BRIGHT_LUMIN);
-        
-        if (cmdline.bvalueSpec)
+
+        if (cmdline.bvalueSpec && !cmdline.bpercentSpec) {
             *bvalueP = cmdline.bvalue;
-        else 
+        } else {
             computeBottomPercentile(hist, maxval, cols*rows, cmdline.bpercent, 
                                 bvalueP);
-        if (cmdline.wvalueSpec) 
+            if (cmdline.bvalueSpec && cmdline.bvalue < *bvalueP) {
+                *bvalueP = cmdline.bvalue;
+            }
+        }
+
+        if (cmdline.wvalueSpec && !cmdline.wpercentSpec) {
             *wvalueP = cmdline.wvalue;
-        else
+        } else {
             computeTopPercentile(hist, maxval, cols*rows, cmdline.wpercent, 
                                  wvalueP);
+            if (cmdline.wvalueSpec && cmdline.wvalue > *wvalueP) {
+                *wvalueP = cmdline.wvalue;
+            }
+        }
+
         free(hist);
     }
 }
