@@ -26,6 +26,7 @@
 #include "pam.h"
 #include "pammap.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 
 #define MAXCOLORS 32767u
 
@@ -227,18 +228,18 @@ initFserr(struct pam * const pamP, struct fserr * const fserrP) {
 
     unsigned int const fserrSize = pamP->width + 2;
 
-    fserrP->thiserr = malloc(pamP->depth * sizeof(long*));
+    MALLOCARRAY(fserrP->thiserr, pamP->depth);
     if (fserrP->thiserr == NULL)
         pm_error("Out of memory allocating Floyd-Steinberg structures");
-    fserrP->nexterr = malloc(pamP->depth * sizeof(long*));
+    MALLOCARRAY(fserrP->nexterr, pamP->depth);
     if (fserrP->nexterr == NULL)
         pm_error("Out of memory allocating Floyd-Steinberg structures");
     
     for (plane = 0; plane < pamP->depth; ++plane) {
-        fserrP->thiserr[plane] = (long*) malloc(fserrSize * sizeof(long));
+        MALLOCARRAY(fserrP->thiserr[plane], fserrSize);
         if (fserrP->thiserr[plane] == NULL)
             pm_error("Out of memory allocating Floyd-Steinberg structures");
-        fserrP->nexterr[plane] = (long*) malloc(fserrSize * sizeof(long));
+        MALLOCARRAY(fserrP->nexterr[plane], fserrSize);
         if (fserrP->nexterr[plane] == NULL)
             pm_error("Out of memory allocating Floyd-Steinberg structures");
     }
@@ -414,7 +415,7 @@ searchColormapCloseComplex(struct pam * const pamP,
 
         for (plane=0; plane < pamP->depth; ++plane) {
             sample const normalizedSample = (sample)
-                (tuple[plane] * maxvalNormalizer) + 0.5;
+                (tuple[plane] * maxvalNormalizer + 0.5);
             
             newdist += sqr(normalizedSample - colormap[i]->tuple[plane])
                 / divider;
@@ -721,7 +722,7 @@ main(int argc, char * argv[] ) {
 
     ifp = pm_openr(cmdline.inputFilespec);
 
-    pnm_readpaminit(ifp, &inpam, sizeof(inpam));
+    pnm_readpaminit(ifp, &inpam, PAM_STRUCT_SIZE(tuple_type));
     
     {
         FILE * mapfile;
@@ -729,7 +730,7 @@ main(int argc, char * argv[] ) {
         tuple ** maptuples;
 
         mapfile = pm_openr(cmdline.mapFilespec);
-        maptuples = pnm_readpam(mapfile, &mappam, sizeof(mappam));
+        maptuples = pnm_readpam(mapfile, &mappam, PAM_STRUCT_SIZE(tuple_type));
         pm_close(mapfile);
 
         computeColorMapFromMap(&mappam, maptuples, &colormap, &colormapSize);
