@@ -342,8 +342,6 @@ shearYColAntialias(xel ** const inxels,
     for (outrow = 0; outrow < outrows; ++outrow)
         outxels[outrow][col] = bgxel;
 
-pm_message("outrows=%d bgxel red=%d", outrows, PPM_GETR(bgxel));
-pm_message("top left pixel red = %d", PPM_GETR(outxels[0][0]));        
     prevxel = bgxel;
     for (inrow = 0; inrow < inrows; ++inrow) {
         int const outrow = inrow + intnew0;
@@ -539,6 +537,7 @@ shearImageY(xel ** const inxels,
 
 int
 main(int argc, char *argv[]) { 
+
     struct cmdlineInfo cmdline;
     FILE* ifP;
     xel* xelrow;
@@ -561,32 +560,32 @@ main(int argc, char *argv[]) {
 
     ifP = pm_openr(cmdline.inputFilespec);
 
-    pnm_readpnminit( ifP, &cols, &rows, &maxval, &format );
-    xelrow = pnm_allocrow( cols );
-
+    pnm_readpnminit(ifP, &cols, &rows, &maxval, &format);
+    xelrow = pnm_allocrow(cols);
+    
     computeNewFormat(!cmdline.noantialias, format, maxval, 
                      &newformat, &newmaxval);
 
-    xshearfac = tan( cmdline.angle / 2.0 );
-    if ( xshearfac < 0.0 )
+    xshearfac = tan(cmdline.angle / 2.0);
+    if (xshearfac < 0.0)
         xshearfac = -xshearfac;
-    yshearfac = sin( cmdline.angle );
-    if ( yshearfac < 0.0 )
+    yshearfac = sin(cmdline.angle);
+    if (yshearfac < 0.0)
         yshearfac = -yshearfac;
     tempcols = rows * xshearfac + cols + 0.999999;
-    yshearjunk = ( tempcols - cols ) * yshearfac;
+    yshearjunk = (tempcols - cols) * yshearfac;
     newrows = tempcols * yshearfac + rows + 0.999999;
-    x2shearjunk = ( newrows - rows - yshearjunk ) * xshearfac;
+    x2shearjunk = (newrows - rows - yshearjunk) * xshearfac;
     newrows -= 2 * yshearjunk;
     newcols = newrows * xshearfac + tempcols + 0.999999 - 2 * x2shearjunk;
     direction = cmdline.angle > 0 ? COUNTERCLOCKWISE : CLOCKWISE;
 
     /* First shear X from input file into temp1xels. */
-    temp1xels = pnm_allocarray( tempcols, rows );
-    for ( row = 0; row < rows; ++row ) {
+    temp1xels = pnm_allocarray(tempcols, rows);
+    for (row = 0; row < rows; ++row) {
         float const new0 = xshearfac * (cmdline.angle > 0 ? row : (rows-row));
 
-        pnm_readpnmrow( ifP, xelrow, cols, maxval, format );
+        pnm_readpnmrow(ifP, xelrow, cols, maxval, format);
 
         pnm_promoteformatrow(xelrow, cols, maxval, format, 
                              newmaxval, newformat);
@@ -598,8 +597,8 @@ main(int argc, char *argv[]) {
         shearX(xelrow, temp1xels[row], cols, newformat, bgxel, 
                !cmdline.noantialias, new0, tempcols);
     }
-    pm_close( ifP );
-    pnm_freerow( xelrow );
+    pm_close(ifP);
+    pnm_freerow(xelrow);
 
     if (cmdline.debugStage1) {
         pnm_writepnm(stdout, temp1xels, tempcols, rows, 
@@ -608,11 +607,11 @@ main(int argc, char *argv[]) {
     }
   
     /* Now inverse shear Y from temp1xels into temp2xels. */
-    temp2xels = pnm_allocarray( tempcols, newrows );
+    temp2xels = pnm_allocarray(tempcols, newrows);
     shearImageY(temp1xels, temp2xels, tempcols, rows, newrows, newformat,
                 bgxel, !cmdline.noantialias, direction, yshearfac, yshearjunk);
 
-    pnm_freearray( temp1xels, rows );
+    pnm_freearray(temp1xels, rows);
 
     if (cmdline.debugStage2) {
         pnm_writepnm(stdout, temp2xels, tempcols, newrows, 
@@ -621,20 +620,20 @@ main(int argc, char *argv[]) {
     }
   
     /* Finally, shear X from temp2xels to output file. */
-    pnm_writepnminit( stdout, newcols, newrows, newmaxval, newformat, 0 );
-    newxelrow = pnm_allocrow( newcols );
-    for ( row = 0; row < newrows; ++row ) {
+    pnm_writepnminit(stdout, newcols, newrows, newmaxval, newformat, 0);
+    newxelrow = pnm_allocrow(newcols);
+    for (row = 0; row < newrows; ++row) {
         float const new0 = xshearfac * (cmdline.angle > 0 ? row : newrows-row);
 
         shearFinal(temp2xels[row], newxelrow, tempcols, newcols, newformat, 
                    bgxel, !cmdline.noantialias, new0, x2shearjunk);
 
-        pnm_writepnmrow( stdout, newxelrow, newcols, newmaxval, newformat, 0 );
+        pnm_writepnmrow(stdout, newxelrow, newcols, newmaxval, newformat, 0);
     }
-
-    pnm_freerow( newxelrow );
-    pnm_freearray( temp2xels, rows );
-    pm_close( stdout );
+    
+    pnm_freerow(newxelrow);
+    pnm_freearray(temp2xels, newrows);
+    pm_close(stdout);
     
     exit( 0 );
 }
