@@ -22,37 +22,17 @@
 
     Author:
         John Walker
-        Autodesk SA
-        Avenue des Champs-Montants 14b
-        CH-2074 MARIN
-        Switzerland
-        Usenet: kelvin@Autodesk.com
-        Fax:    038/33 88 15
-        Voice:  038/33 76 33
+        http://www.fourmilab.ch/
 
     Permission  to  use, copy, modify, and distribute this software and
     its documentation  for  any  purpose  and  without  fee  is  hereby
     granted,  without any conditions or restrictions.  This software is
     provided "as is" without express or implied warranty.
 
-                PLUGWARE!
-
-    If you like this kind of stuff, you may also enjoy "James  Gleick's
-    Chaos--The  Software"  for  MS-DOS,  available for $59.95 from your
-    local software store or directly from Autodesk, Inc., Attn: Science
-    Series,  2320  Marinship Way, Sausalito, CA 94965, USA.  Telephone:
-    (800) 688-2344 toll-free or, outside the  U.S. (415)  332-2344  Ext
-    4886.   Fax: (415) 289-4718.  "Chaos--The Software" includes a more
-    comprehensive   fractal    forgery    generator    which    creates
-    three-dimensional  landscapes  as  well as clouds and planets, plus
-    five more modules which explore other aspects of Chaos.   The  user
-    guide  of  more  than  200  pages includes an introduction by James
-    Gleick and detailed explanations by Rudy Rucker of the  mathematics
-    and algorithms used by each program.
-
 */
 
 #include <math.h>
+#include <assert.h>
 #include "ppm.h"
 #include "mallocvar.h"
 
@@ -63,6 +43,12 @@
 #define M_E 2.7182818284590452354
 #endif
 
+#ifdef VMS
+static double const hugeVal = HUGE_VAL;
+#else
+static double const hugeVal = 1e50;
+#endif
+
 /* Definitions used to address real and imaginary parts in a two-dimensional
    array of complex numbers as stored by fourn(). */
 
@@ -71,9 +57,11 @@
 
 /* Co-ordinate indices within arrays. */
 
-#define X     0
-#define Y     1
-#define Z     2
+typedef struct {
+    double x;
+    double y;
+    double z; 
+} vector;
 
 /* Definition for obtaining random numbers. */
 
@@ -92,43 +80,28 @@
    
 #define ELEMENTS(array) (sizeof(array)/sizeof((array)[0]))
 
-/*  Data types  */
-
-typedef int Boolean;
-#define FALSE 0
-#define TRUE 1
-
-#define V     (void)
-
-/*  Display parameters  */
-
-#define SCRSAT   255              /* Screen maximum intensity */
-
 /* prototypes */
 static void fourn ARGS((float data[], int nn[], int ndim, int isign));
 static void initgauss ARGS((unsigned int seed));
 static double gauss ARGS((void));
 static void spectralsynth ARGS((float **x, unsigned int n, double h));
-static void initseed ARGS((void));
 static void temprgb ARGS((double temp, double *r, double *g, double *b));
 static void etoile ARGS((pixel *pix));
-static void genplanet ARGS((float *a, unsigned int n));
-static Boolean planet ARGS((void));
 /*  Local variables  */
 
 static double arand, gaussadd, gaussfac; /* Gaussian random parameters */
 static double fracdim;            /* Fractal dimension */
 static double powscale;           /* Power law scaling exponent */
 static int meshsize = 256;        /* FFT mesh size */
-static unsigned int rseed;        /* Current random seed */
-static Boolean seedspec = FALSE;      /* Did the user specify a seed ? */
-static Boolean clouds = FALSE;        /* Just generate clouds */
-static Boolean stars = FALSE;         /* Just generate stars */
+static unsigned int seedarg;        /* Seed specified by user */
+static bool seedspec = FALSE;      /* Did the user specify a seed ? */
+static bool clouds = FALSE;        /* Just generate clouds */
+static bool stars = FALSE;         /* Just generate stars */
 static int screenxsize = 256;         /* Screen X size */
 static int screenysize = 256;         /* Screen Y size */
 static double inclangle, hourangle;   /* Star position relative to planet */
-static Boolean inclspec = FALSE;      /* No inclination specified yet */
-static Boolean hourspec = FALSE;      /* No hour specified yet */
+static bool inclspec = FALSE;      /* No inclination specified yet */
+static bool hourspec = FALSE;      /* No hour specified yet */
 static double icelevel;           /* Ice cap theshold */
 static double glaciers;           /* Glacier level */
 static int starfraction;          /* Star fraction */
@@ -257,7 +230,7 @@ static double gauss()
     return gaussfac * sum - gaussadd;
 }
 
-/*  SPECTRALSYNTH  --  Spectrally  synthesised  fractal  motion in two
+/*  SPECTRALSYNTH  --  Spectrally  synthesized  fractal  motion in two
                dimensions.  This algorithm is given under  the
                name   SpectralSynthesisFM2D  on  page  108  of
                Peitgen & Saupe. */
@@ -319,19 +292,22 @@ static void spectralsynth(x, n, h)
     fourn(a, nsize, 2, -1);       /* Take inverse 2D Fourier transform */
 }
 
-/*  INITSEED  --  Generate initial random seed, if needed.  */
 
-static void initseed()
-{
+static unsigned int
+initseed(void) {
+    /*  Generate initial random seed.  */
+
     int i;
 
     i = time(NULL) ^ 0xF37C;
-    V srand(i);
-    for (i = 0; i < 7; i++)
-        V rand();
-    rseed = rand();
+    srand(i);
+    for (i = 0; i < 7; ++i)
+        rand();
+    return rand();
 }
 
+
+
 /*  TEMPRGB  --  Calculate the relative R, G, and B components  for  a
          black  body  emitting  light  at a given temperature.
          The Planck radiation equation is solved directly  for
@@ -409,395 +385,642 @@ static void etoile(pix)
     }
 }
 
-/*  GENPLANET  --  Generate planet from elevation array.  */
 
-static void genplanet(a, n)
-    float *a;
-    unsigned int n;
-{
-    int i, j;
-    unsigned char *cp, *ap;
+
+static double
+uprj(unsigned int const a,
+     unsigned int const size) {
+
+    return (double)a/(size-1);
+}
+
+
+
+static double
+atSat(double const x,
+      double const y,
+      double const dsat) {
+
+    return x*(1.0-dsat) + y*dsat;
+}
+
+
+
+static unsigned char *
+makeCp(float *      const a,
+       unsigned int const n,
+       pixval       const maxval) {
+
+    /* Prescale the grid points into intensities. */
+
+    unsigned char * cp;
+    unsigned char * ap;
+
+    if (UINT_MAX / n < n)
+        pm_error("arithmetic overflow squaring %u", n);
+    cp = malloc(n * n);
+    if (cp == NULL)
+        pm_error("Unable to allocate %u bytes for cp array", n);
+
+    ap = cp;
+    {
+        unsigned int i;
+        for (i = 0; i < n; i++) {
+            unsigned int j;
+            for (j = 0; j < n; j++)
+                *ap++ = ((double)maxval * (Real(a, i, j) + 1.0)) / 2.0;
+        }
+    }
+    return cp;
+}
+
+
+
+static void
+createPlanetStuff(float *          const a,
+                  unsigned int     const n,
+                  double **        const uP,
+                  double **        const u1P,
+                  unsigned int **  const bxfP,
+                  unsigned int **  const bxcP,
+                  unsigned char ** const cpP,
+                  vector *         const sunvecP,
+                  unsigned int     const cols,
+                  pixval           const maxval) {
+
     double *u, *u1;
     unsigned int *bxf, *bxc;
-
-#define RGBQuant    255
-    pixel *pixels;            /* Pixel vector */
-    pixel *rpix;              /* Current pixel pointer */
-
-#define Atthick 1.03             
-   /* Atmosphere thickness as a percentage of planet's diameter */
-    double athfac = sqrt(Atthick * Atthick - 1.0);
-    double sunvec[3];
-
-    Boolean flipped = FALSE;
+    unsigned char * cp;
     double shang, siang;
+    bool flipped;
 
-    ppm_writeppminit(stdout, screenxsize, screenysize,
-                     (pixval) RGBQuant, FALSE);
+    /* Compute incident light direction vector. */
 
-    if (!stars) {
-        MALLOCARRAY(u,   screenxsize);
-        MALLOCARRAY(u1,  screenxsize);
-        MALLOCARRAY(bxf, screenxsize);
-        MALLOCARRAY(bxc, screenxsize);
+    shang = hourspec ? hourangle : Cast(0, 2 * M_PI);
+    siang = inclspec ? inclangle : Cast(-M_PI * 0.12, M_PI * 0.12);
 
-        if (u == NULL || u1 == NULL || bxf == NULL || bxc == NULL) 
-            pm_error("Cannot allocate %d element interpolation tables.",
-                     screenxsize);
+    sunvecP->x = sin(shang) * cos(siang);
+    sunvecP->y = sin(siang);
+    sunvecP->z = cos(shang) * cos(siang);  /* initial value */
 
-        /* Compute incident light direction vector. */
+    /* Allow only 25% of random pictures to be crescents */
 
-        shang = hourspec ? hourangle : Cast(0, 2 * M_PI);
-        siang = inclspec ? inclangle : Cast(-M_PI * 0.12, M_PI * 0.12);
+    if (!hourspec && ((rand() % 100) < 75)) {
+        flipped = (sunvecP->z < 0);
+        sunvecP->z = fabs(sunvecP->z);
+    } else
+        flipped = FALSE;
 
-        sunvec[X] = sin(shang) * cos(siang);
-        sunvec[Y] = sin(siang);
-        sunvec[Z] = cos(shang) * cos(siang);
+    if (!clouds) {
+        pm_message(
+            "        -inclination %.0f -hour %d -ice %.2f -glaciers %.2f",
+            (siang * (180.0 / M_PI)),
+            (int) (((shang * (12.0 / M_PI)) + 12 +
+                    (flipped ? 12 : 0)) + 0.5) % 24, icelevel, glaciers);
+        pm_message("        -stars %d -saturation %d.",
+                   starfraction, starcolour);
+    }
 
-        /* Allow only 25% of random pictures to be crescents */
+    cp = makeCp(a, n, maxval);
 
-        if (!hourspec && ((rand() % 100) < 75)) {
-            flipped = sunvec[Z] < 0 ? TRUE : FALSE;
-            sunvec[Z] = abs(sunvec[Z]);
-        }
-        pm_message("%s: -seed %d -dimension %.2f -power %.2f -mesh %d",
-                   clouds ? "clouds" : "planet",
-                   rseed, fracdim, powscale, meshsize);
-        if (!clouds) {
-            pm_message(
-                "        -inclination %.0f -hour %d -ice %.2f -glaciers %.2f",
-                (siang * (180.0 / M_PI)),
-                (int) (((shang * (12.0 / M_PI)) + 12 +
-                        (flipped ? 12 : 0)) + 0.5) % 24, icelevel, glaciers);
-            pm_message("        -stars %d -saturation %d.",
-                       starfraction, starcolour);
-        }
-
-        /* Prescale the grid points into intensities. */
-
-        cp = (unsigned char *) malloc(n * n);
-        if (cp == NULL)
-            return;
-        ap = cp;
-        for (i = 0; i < n; i++) {
-            for (j = 0; j < n; j++) {
-                *ap++ = (255.0 * (Real(a, i, j) + 1.0)) / 2.0;
-            }
-        }
-
-        /* Fill the screen from the computed  intensity  grid  by  mapping
-           screen  points onto the grid, then calculating each pixel value
-           by bilinear interpolation from  the  surrounding  grid  points.
-           (N.b. the pictures would undoubtedly look better when generated
-           with small grids if  a  more  well-behaved  interpolation  were
-           used.)
-
-           Before  we get started, precompute the line-level interpolation
-           parameters and store them in an array so we don't  have  to  do
-           this every time around the inner loop. */
-
-#define UPRJ(a,size) ((a)/((size)-1.0))
-
-        for (j = 0; j < screenxsize; j++) {
-            double bx = (n - 1) * UPRJ(j, screenxsize);
+    /* Fill the screen from the computed  intensity  grid  by  mapping
+       screen  points onto the grid, then calculating each pixel value
+       by bilinear interpolation from  the  surrounding  grid  points.
+       (N.b. the pictures would undoubtedly look better when generated
+       with small grids if  a  more  well-behaved  interpolation  were
+       used.)
+       
+       Also compute the line-level interpolation parameters that
+       caller will need every time around his inner loop.  
+    */
 
+    MALLOCARRAY(u,   cols);
+    MALLOCARRAY(u1,  cols);
+    MALLOCARRAY(bxf, cols);
+    MALLOCARRAY(bxc, cols);
+    
+    if (u == NULL || u1 == NULL || bxf == NULL || bxc == NULL) 
+        pm_error("Cannot allocate %d element interpolation tables.", cols);
+    {
+        unsigned int j;
+        for (j = 0; j < cols; j++) {
+            double const bx = (n - 1) * uprj(j, cols);
+            
             bxf[j] = floor(bx);
             bxc[j] = bxf[j] + 1;
             u[j] = bx - bxf[j];
             u1[j] = 1 - u[j];
         }
-    } else {
-        pm_message("night: -seed %d -stars %d -saturation %d.",
-                   rseed, starfraction, starcolour);
+    }
+    *uP   = u;  *u1P  = u1;
+    *bxfP = bxf; *bxcP = bxc;
+    *cpP  = cp;
+}
+
+
+
+static void
+generateStarrySkyRow(pixel *      const pixels, 
+                     unsigned int const cols) {
+/*----------------------------------------------------------------------------
+  Generate a starry sky.  Note  that no FFT is performed;
+  the output is  generated  directly  from  a  power  law
+  mapping  of  a  pseudorandom sequence into intensities. 
+-----------------------------------------------------------------------------*/
+    unsigned int j;
+    
+    for (j = 0; j < cols; j++)
+        etoile(pixels + j);
+}
+
+
+
+static void
+generateCloudRow(pixel *         const pixels,
+                 unsigned int    const cols,
+                 double          const t,
+                 double          const t1,
+                 double *        const u,
+                 double *        const u1,
+                 unsigned char * const cp,
+                 unsigned int *  const bxc,
+                 unsigned int *  const bxf,
+                 int             const byc,
+                 int             const byf,
+                 pixval          const maxval) {
+
+    /* Render the FFT output as clouds. */
+
+    unsigned int j;
+
+    for (j = 0; j < cols; j++) {
+        double r;
+        pixval w;
+        
+        r = 0.0;  /* initial value */
+        /* Note that where t1 and t are zero, the cp[] element
+           referenced below does not exist.
+        */
+        if (t1 > 0.0)
+            r += t1 * u1[j] * cp[byf + bxf[j]] +
+                t1 * u[j]  * cp[byf + bxc[j]];
+        if (t > 0.0)
+            r += t * u1[j] * cp[byc + bxf[j]] +
+                t * u[j]  * cp[byc + bxc[j]];
+        
+        w = (r > 127.0) ? (maxval * ((r - 127.0) / 128.0)) : 0;
+        
+        PPM_ASSIGN(*(pixels + j), w, w, maxval);
+    }
+}
+
+
+
+static void
+makeLand(int *  const irP,
+         int *  const igP,
+         int *  const ibP,
+         double const r) {
+/*----------------------------------------------------------------------------
+  Land area.  Look up colour based on elevation from precomputed
+  colour map table.
+-----------------------------------------------------------------------------*/
+    static unsigned char pgnd[][3] = {
+        {206, 205, 0}, {208, 207, 0}, {211, 208, 0},
+        {214, 208, 0}, {217, 208, 0}, {220, 208, 0},
+        {222, 207, 0}, {225, 205, 0}, {227, 204, 0},
+        {229, 202, 0}, {231, 199, 0}, {232, 197, 0},
+        {233, 194, 0}, {234, 191, 0}, {234, 188, 0},
+        {233, 185, 0}, {232, 183, 0}, {231, 180, 0},
+        {229, 178, 0}, {227, 176, 0}, {225, 174, 0},
+        {223, 172, 0}, {221, 170, 0}, {219, 168, 0},
+        {216, 166, 0}, {214, 164, 0}, {212, 162, 0},
+        {210, 161, 0}, {207, 159, 0}, {205, 157, 0},
+        {203, 156, 0}, {200, 154, 0}, {198, 152, 0},
+        {195, 151, 0}, {193, 149, 0}, {190, 148, 0},
+        {188, 147, 0}, {185, 145, 0}, {183, 144, 0},
+        {180, 143, 0}, {177, 141, 0}, {175, 140, 0},
+        {172, 139, 0}, {169, 138, 0}, {167, 137, 0},
+        {164, 136, 0}, {161, 135, 0}, {158, 134, 0},
+        {156, 133, 0}, {153, 132, 0}, {150, 132, 0},
+        {147, 131, 0}, {145, 130, 0}, {142, 130, 0},
+        {139, 129, 0}, {136, 128, 0}, {133, 128, 0},
+        {130, 127, 0}, {127, 127, 0}, {125, 127, 0},
+        {122, 127, 0}, {119, 127, 0}, {116, 127, 0},
+        {113, 127, 0}, {110, 128, 0}, {107, 128, 0},
+        {104, 128, 0}, {102, 127, 0}, { 99, 126, 0},
+        { 97, 124, 0}, { 95, 122, 0}, { 93, 120, 0},
+        { 92, 117, 0}, { 92, 114, 0}, { 92, 111, 0},
+        { 93, 108, 0}, { 94, 106, 0}, { 96, 104, 0},
+        { 98, 102, 0}, {100, 100, 0}, {103,  99, 0},
+        {106,  99, 0}, {109,  99, 0}, {111, 100, 0},
+        {114, 101, 0}, {117, 102, 0}, {120, 103, 0},
+        {123, 102, 0}, {125, 102, 0}, {128, 100, 0},
+        {130,  98, 0}, {132,  96, 0}, {133,  94, 0},
+        {134,  91, 0}, {134,  88, 0}, {134,  85, 0},
+        {133,  82, 0}, {131,  80, 0}, {129,  78, 0}
+    };
+
+    unsigned int const ix = ((r - 128) * (ELEMENTS(pgnd) - 1)) / 127;
+    
+    *irP = pgnd[ix][0];
+    *igP = pgnd[ix][1];
+    *ibP = pgnd[ix][2];
+} 
+
+
+
+static void
+makeWater(int *  const irP,
+          int *  const igP,
+          int *  const ibP,
+          double const r,
+          pixval const maxval) {
+
+    /* Water.  Generate clouds above water based on elevation.  */
+
+    *irP = *igP = r > 64 ? (r - 64) * 4 : 0;
+    *ibP = maxval;
+}
+
+
+
+static void
+addIce(int *  const irP,
+       int *  const igP,
+       int *  const ibP,
+       double const r,
+       double const azimuth,
+       double const icelevel,
+       double const glaciers,
+       pixval const maxval) {
+
+    /* Generate polar ice caps. */
+    
+    double const icet = pow(fabs(sin(azimuth)), 1.0 / icelevel) - 0.5;
+    double const ice = MAX(0.0, 
+                           (icet + glaciers * MAX(-0.5, (r - 128) / 128.0)));
+    if  (ice > 0.125) {
+        *irP = maxval;
+        *igP = maxval;
+        *ibP = maxval;
+    }
+}
+
+
+
+static void
+limbDarken(int *          const irP,
+           int *          const igP,
+           int *          const ibP,
+           unsigned int   const col,
+           unsigned int   const row,
+           unsigned int   const cols,
+           unsigned int   const rows,
+           vector         const sunvec,
+           pixval         const maxval) {
+
+    /* With Gcc 2.95.3 compiler optimization level > 1, I have seen this
+       function confuse all the variables and ultimately generate a 
+       completely black image.  Adding an extra reference to 'rows' seems
+       to put things back in order, and the assert() below does that.
+       Take it out, and the problem comes back!  04.02.21.
+    */
+
+    /* Apply limb darkening by cosine rule. */
+
+    double const atthick  = 1.03;
+    double const atSatFac = 1.0;
+    double const athfac   = sqrt(atthick * atthick - 1.0);
+        /* Atmosphere thickness as a percentage of planet's diameter */
+
+    double const dy = 2 * ((double)rows/2 - row) / rows;
+    double const dysq = dy * dy;
+    /* Note: we are in fact normalizing this horizontal position by the
+       vertical size of the picture.  And we know rows >= cols.
+    */
+    double const dx   = 2 * ((double)cols/2 - col) / rows;
+    double const dxsq = dx * dx;
+
+    double const ds = MIN(1.0, sqrt(dxsq + dysq));
+    
+    /* Calculate atmospheric absorption based on the thickness of
+       atmosphere traversed by light on its way to the surface.  
+    */
+    double const dsq = ds * ds;
+    double const dsat = atSatFac * ((sqrt(atthick * atthick - dsq) -
+                                     sqrt(1.0 * 1.0 - dsq)) / athfac);
+
+    assert(rows >= cols);  /* An input requirement */
+
+    *irP = atSat(*irP, maxval/2, dsat);
+    *igP = atSat(*igP, maxval/2, dsat);
+    *ibP = atSat(*ibP, maxval,   dsat);
+    {
+        double const PlanetAmbient = 0.05;
+
+        double const sqomdysq = sqrt(1.0 - dysq);
+        double const svx = sunvec.x;
+        double const svy = sunvec.y * dy;
+        double const svz = sunvec.z * sqomdysq;
+        double const di = 
+            MAX(0, MIN(1.0, svx * dx + svy + svz * sqrt(1.0 - dxsq)));
+        double const inx = PlanetAmbient * 1.0 + (1.0 - PlanetAmbient) * di;
+
+        *irP *= inx;
+        *igP *= inx;
+        *ibP *= inx;
+    }
+}
+
+
+
+static void
+generatePlanetRow(pixel *         const pixelrow,
+                  unsigned int    const row,
+                  unsigned int    const rows,
+                  unsigned int    const cols,
+                  double          const t,
+                  double          const t1,
+                  double *        const u,
+                  double *        const u1,
+                  unsigned char * const cp,
+                  unsigned int *  const bxc,
+                  unsigned int *  const bxf,
+                  int             const byc,
+                  int             const byf,
+                  vector          const sunvec,
+                  pixval          const maxval) {
+
+    unsigned int const StarClose = 2;
+
+    double const azimuth    = asin(((((double) row) / (rows - 1)) * 2) - 1);
+    unsigned int const lcos = (rows / 2) * fabs(cos(azimuth));
+
+    unsigned int col;
+
+    for (col = 0; col < cols; ++col)
+        PPM_ASSIGN(pixelrow[col], 0, 0, 0);
+
+    for (col = cols/2 - lcos; col <= cols/2 + lcos; ++col) {
+        double r;
+        int ir, ig, ib;
+
+        r = 0.0;   /* initial value */
+        
+        /* Note that where t1 and t are zero, the cp[] element
+           referenced below does not exist.  
+        */
+        if (t1 > 0.0)
+            r += t1 * u1[col] * cp[byf + bxf[col]] +
+                t1 * u[col]  * cp[byf + bxc[col]];
+        if (t > 0.0)
+            r += t * u1[col] * cp[byc + bxf[col]] +
+                t * u[col]  * cp[byc + bxc[col]];
+
+        if (r >= 128) 
+            makeLand(&ir, &ig, &ib, r);
+        else 
+            makeWater(&ir, &ig, &ib, r, maxval);
+
+        addIce(&ir, &ig, &ib, r, azimuth, icelevel, glaciers, maxval);
+
+        limbDarken(&ir, &ig, &ib, col, row, cols, rows, sunvec, maxval);
+
+        PPM_ASSIGN(pixelrow[col], ir, ig, ib);
     }
 
-    pixels = ppm_allocrow(screenxsize);
-    for (i = 0; i < screenysize; i++) {
-        double t, t1, by, dy, dysq, sqomdysq, icet, svx, svy, svz,
-            azimuth;
-        int byf, byc, lcos;
+    /* Left stars */
 
-        if (!stars) {             /* Skip all this setup if just stars */
-            by = (n - 1) * UPRJ(i, screenysize);
-            dy = 2 * (((screenysize / 2) - i) / ((double) screenysize));
-            dysq = dy * dy;
-            sqomdysq = sqrt(1.0 - dysq);
-            svx = sunvec[X];
-            svy = sunvec[Y] * dy;
-            svz = sunvec[Z] * sqomdysq;
-            byf = floor(by) * n;
-            byc = byf + n;
-            t = by - floor(by);
-            t1 = 1 - t;
+    for (col = 0; (int)col < (int)(cols/2 - (lcos + StarClose)); ++col)
+        etoile(&pixelrow[col]);
+
+    /* Right stars */
+
+    for (col = cols/2 + (lcos + StarClose); col < cols; ++col)
+        etoile(&pixelrow[col]);
+}
+
+
+
+static void 
+genplanet(bool         const stars,
+          bool         const clouds,
+          float *      const a, 
+          unsigned int const cols,
+          unsigned int const rows,
+          unsigned int const n,
+          unsigned int const rseed) {
+/*----------------------------------------------------------------------------
+  Generate planet from elevation array.
+
+  If 'stars' is true, a is undefined.  Otherwise, it is defined.
+-----------------------------------------------------------------------------*/
+    pixval const maxval = PPM_MAXMAXVAL;
+
+    unsigned char *cp;
+    double *u, *u1;
+    unsigned int *bxf, *bxc;
+
+    pixel *pixelrow;
+    unsigned int row;
+
+    vector sunvec;
+
+    ppm_writeppminit(stdout, cols, rows, maxval, FALSE);
+
+    if (stars) {
+        pm_message("night: -seed %d -stars %d -saturation %d.",
+                   rseed, starfraction, starcolour);
+        cp = NULL; 
+        u = NULL; u1 = NULL;
+        bxf = NULL; bxc = NULL;
+    } else {
+        pm_message("%s: -seed %d -dimension %.2f -power %.2f -mesh %d",
+                   clouds ? "clouds" : "planet",
+                   rseed, fracdim, powscale, meshsize);
+        createPlanetStuff(a, n, &u, &u1, &bxf, &bxc, &cp, &sunvec, 
+                          cols, maxval);
+    }
+
+    pixelrow = ppm_allocrow(cols);
+    for (row = 0; row < rows; ++row) {
+        if (stars)
+            generateStarrySkyRow(pixelrow, cols);
+        else {
+            double const by = (n - 1) * uprj(row, rows);
+            int    const byf = floor(by) * n;
+            int    const byc = byf + n;
+            double const t = by - floor(by);
+            double const t1 = 1 - t;
+
+            if (clouds)
+                generateCloudRow(pixelrow, cols,
+                                 t, t1, u, u1, cp, bxc, bxf, byc, byf,
+                                 maxval);
+            else 
+                generatePlanetRow(pixelrow, row, rows, cols,
+                                  t, t1, u, u1, cp, bxc, bxf, byc, byf,
+                                  sunvec,
+                                  maxval);
         }
-
-        if (clouds) {
-
-            /* Render the FFT output as clouds. */
-
-            for (j = 0; j < screenxsize; j++) {
-                double r;
-                pixval w;
-
-                r = 0.0;  /* initial value */
-                /* Note that where t1 and t are zero, the cp[] element
-                   referenced below does not exist.
-                */
-                if (t1 > 0.0)
-                    r += t1 * u1[j] * cp[byf + bxf[j]] +
-                         t1 * u[j]  * cp[byf + bxc[j]];
-                if (t > 0.0)
-                    r += t * u1[j] * cp[byc + bxf[j]] +
-                         t * u[j]  * cp[byc + bxc[j]];
-        
-                w = (r > 127.0) ? (RGBQuant * ((r - 127.0) / 128.0)) : 0;
-
-                PPM_ASSIGN(*(pixels + j), w, w, RGBQuant);
-            }
-        } else if (stars) {
-
-            /* Generate a starry sky.  Note  that no FFT is performed;
-               the output is  generated  directly  from  a  power  law
-               mapping  of  a  pseudorandom sequence into intensities. */
-
-            for (j = 0; j < screenxsize; j++) {
-                etoile(pixels + j);
-            }
-        } else {
-            for (j = 0; j < screenxsize; j++) {
-                PPM_ASSIGN(*(pixels + j), 0, 0, 0);
-            }
-            azimuth = asin(((((double) i) / (screenysize - 1)) * 2) - 1);
-            icet = pow(abs(sin(azimuth)), 1.0 / icelevel) - 0.5;
-            lcos = (screenysize / 2) * abs(cos(azimuth));
-            rpix = pixels + (screenxsize / 2) - lcos;
-            for (j = (screenxsize / 2) - lcos;
-                 j <= (screenxsize / 2) + lcos; j++) {
-                double r, ice;
-                int ir, ig, ib;
-                static unsigned char pgnd[][3] = {
-                    {206, 205, 0}, {208, 207, 0}, {211, 208, 0},
-                    {214, 208, 0}, {217, 208, 0}, {220, 208, 0},
-                    {222, 207, 0}, {225, 205, 0}, {227, 204, 0},
-                    {229, 202, 0}, {231, 199, 0}, {232, 197, 0},
-                    {233, 194, 0}, {234, 191, 0}, {234, 188, 0},
-                    {233, 185, 0}, {232, 183, 0}, {231, 180, 0},
-                    {229, 178, 0}, {227, 176, 0}, {225, 174, 0},
-                    {223, 172, 0}, {221, 170, 0}, {219, 168, 0},
-                    {216, 166, 0}, {214, 164, 0}, {212, 162, 0},
-                    {210, 161, 0}, {207, 159, 0}, {205, 157, 0},
-                    {203, 156, 0}, {200, 154, 0}, {198, 152, 0},
-                    {195, 151, 0}, {193, 149, 0}, {190, 148, 0},
-                    {188, 147, 0}, {185, 145, 0}, {183, 144, 0},
-                    {180, 143, 0}, {177, 141, 0}, {175, 140, 0},
-                    {172, 139, 0}, {169, 138, 0}, {167, 137, 0},
-                    {164, 136, 0}, {161, 135, 0}, {158, 134, 0},
-                    {156, 133, 0}, {153, 132, 0}, {150, 132, 0},
-                    {147, 131, 0}, {145, 130, 0}, {142, 130, 0},
-                    {139, 129, 0}, {136, 128, 0}, {133, 128, 0},
-                    {130, 127, 0}, {127, 127, 0}, {125, 127, 0},
-                    {122, 127, 0}, {119, 127, 0}, {116, 127, 0},
-                    {113, 127, 0}, {110, 128, 0}, {107, 128, 0},
-                    {104, 128, 0}, {102, 127, 0}, { 99, 126, 0},
-                    { 97, 124, 0}, { 95, 122, 0}, { 93, 120, 0},
-                    { 92, 117, 0}, { 92, 114, 0}, { 92, 111, 0},
-                    { 93, 108, 0}, { 94, 106, 0}, { 96, 104, 0},
-                    { 98, 102, 0}, {100, 100, 0}, {103,  99, 0},
-                    {106,  99, 0}, {109,  99, 0}, {111, 100, 0},
-                    {114, 101, 0}, {117, 102, 0}, {120, 103, 0},
-                    {123, 102, 0}, {125, 102, 0}, {128, 100, 0},
-                    {130,  98, 0}, {132,  96, 0}, {133,  94, 0},
-                    {134,  91, 0}, {134,  88, 0}, {134,  85, 0},
-                    {133,  82, 0}, {131,  80, 0}, {129,  78, 0}
-                };
-
-                r = 0.0;   /* initial value */
-
-                /* Note that where t1 and t are zero, the cp[] element
-                   referenced below does not exist.  
-                */
-                if (t1 > 0.0)
-                    r += t1 * u1[j] * cp[byf + bxf[j]] +
-                         t1 * u[j]  * cp[byf + bxc[j]];
-                if (t > 0.0)
-                    r += t * u1[j] * cp[byc + bxf[j]] +
-                         t * u[j]  * cp[byc + bxc[j]];
-
-                if (r >= 128) {
-                    int ix = ((r - 128) * (ELEMENTS(pgnd) - 1)) / 127;
-
-                    /* Land area.  Look up colour based on elevation from
-                       precomputed colour map table. */
-
-                    ir = pgnd[ix][0];
-                    ig = pgnd[ix][1];
-                    ib = pgnd[ix][2];
-                } else {
-
-                    /* Water.  Generate clouds above water based on
-                       elevation.  */
-
-                    ir = ig = r > 64 ? (r - 64) * 4 : 0;
-                    ib = 255;
-                }
-
-                /* Generate polar ice caps. */
-
-                ice = 
-                    MAX(0.0, (icet + glaciers * MAX(-0.5, (r - 128) / 128.0)));
-                if  (ice > 0.125) {
-                    ir = ig = ib = 255;
-                }
-
-                /* Apply limb darkening by cosine rule. */
-
-                {   double dx = 2 * (((screenxsize / 2) - j) /
-                                     ((double) screenysize)),
-                        dxsq = dx * dx,
-                        ds, di, inx;
-                double dsq, dsat;
-                di = svx * dx + svy + svz * sqrt(1.0 - dxsq);
-#define         PlanetAmbient  0.05
-                if (di < 0)
-                    di = 0;
-                di = MIN(1.0, di);
-
-                ds = sqrt(dxsq + dysq);
-                ds = MIN(1.0, ds);
-
-                /* Calculate  atmospheric absorption  based on the
-                   thickness of atmosphere traversed by  light  on
-                   its way to the surface. */
-
-#define         AtSatFac 1.0
-                dsq = ds * ds;
-                dsat = AtSatFac * ((sqrt(Atthick * Atthick - dsq) -
-                                    sqrt(1.0 * 1.0 - dsq)) / athfac);
-#define         AtSat(x,y) x = ((x)*(1.0-dsat))+(y)*dsat
-                AtSat(ir, 127);
-                AtSat(ig, 127);
-                AtSat(ib, 255);
-
-                inx = PlanetAmbient + (1.0 - PlanetAmbient) * di;
-                ir *= inx;
-                ig *= inx;
-                ib *= inx;
-                }
-
-                PPM_ASSIGN(*rpix, ir, ig, ib);
-                rpix++;
-            }
-
-            /* Left stars */
-
-#define StarClose   2
-            for (j = 0; j < (screenxsize / 2) - (lcos + StarClose); j++) {
-                etoile(pixels + j);
-            }
-
-            /* Right stars */
-
-            for (j = (screenxsize / 2) + (lcos + StarClose);
-                 j < screenxsize; j++) {
-                etoile(pixels + j);
-            }
-        }
-        ppm_writeppmrow(stdout, pixels, screenxsize, RGBQuant, FALSE);
+        ppm_writeppmrow(stdout, pixelrow, cols, maxval, FALSE);
     }
     pm_close(stdout);
 
-    ppm_freerow(pixels);
-    if (!stars) {
-        free((char *) cp);
-        free((char *) u);
-        free((char *) u1);
-        free((char *) bxf);
-        free((char *) bxc);
+    ppm_freerow(pixelrow);
+    if (cp)  free(cp);
+    if (u)   free(u);
+    if (u1)  free(u1);
+    if (bxf) free(bxf);
+    if (bxc) free(bxc);
+}
+
+
+
+static void
+applyPowerLawScaling(float * const a,
+                     int     const meshsize,
+                     double  const powscale) {
+
+    /* Apply power law scaling if non-unity scale is requested. */
+    
+    if (powscale != 1.0) {
+        unsigned int i;
+        for (i = 0; i < meshsize; i++) {
+            unsigned int j;
+            for (j = 0; j < meshsize; j++) {
+                double const r = Real(a, i, j);
+                if (r > 0)
+                    Real(a, i, j) = pow(r, powscale);
+            }
+        }
     }
 }
 
-/*  PLANET  --  Make a planet.  */
 
-static Boolean planet()
-{
-    float *a = (float *) 0;
-    int i, j;
-#ifdef VMS
-    double rmin = HUGE_VAL, rmax = -HUGE_VAL, rmean, rrange;
-#else
-    double rmin = 1e50, rmax = -1e50, rmean, rrange;
-#endif
 
-    if (!seedspec) {
-        initseed();
+static void
+computeExtremeReal(const float * const a,
+                   int           const meshsize,
+                   double *      const rminP,
+                   double *      const rmaxP) {
+    
+    /* Compute extrema for autoscaling. */
+
+    double rmin, rmax;
+    unsigned int i;
+
+    rmin = hugeVal;
+    rmax = -hugeVal;
+
+    for (i = 0; i < meshsize; i++) {
+        unsigned int j;
+        for (j = 0; j < meshsize; j++) {
+            double r = Real(a, i, j);
+            
+            rmin = MIN(rmin, r);
+            rmax = MAX(rmax, r);
+        }
     }
+    *rminP = rmin;
+    *rmaxP = rmax;
+}
+
+
+
+static void
+replaceWithSpread(float * const a,
+                  int     const meshsize) {
+/*----------------------------------------------------------------------------
+  Replace the real part of each element of the 'a' array with a
+  measure of how far the real is from the middle; sort of a standard
+  deviation.
+-----------------------------------------------------------------------------*/
+    double rmin, rmax;
+    double rmean, rrange;
+    unsigned int i;
+
+    computeExtremeReal(a, meshsize, &rmin, &rmax);
+
+    rmean = (rmin + rmax) / 2;
+    rrange = (rmax - rmin) / 2;
+
+    for (i = 0; i < meshsize; i++) {
+        unsigned int j;
+        for (j = 0; j < meshsize; j++) {
+            Real(a, i, j) = (Real(a, i, j) - rmean) / rrange;
+        }
+    }
+}
+
+
+
+static bool
+planet(unsigned int const cols,
+       unsigned int const rows,
+       bool         const stars,
+       bool         const clouds) {
+/*----------------------------------------------------------------------------
+   Make a planet.
+-----------------------------------------------------------------------------*/
+    float * a;
+    bool error;
+    unsigned int rseed;        /* Current random seed */
+    
+    if (seedspec)
+        rseed = seedarg;
+    else 
+        rseed = initseed();
+    
     initgauss(rseed);
-
-    if (!stars) {
-
+    
+    if (stars) {
+        a = NULL;
+        error = FALSE;
+    } else {
         spectralsynth(&a, meshsize, 3.0 - fracdim);
-        if (a == (float *) 0) {
-            return FALSE;
-        }
+        if (a == NULL) {
+            error = TRUE;
+        } else {
+            applyPowerLawScaling(a, meshsize, powscale);
+                
+            replaceWithSpread(a, meshsize);
 
-        /* Apply power law scaling if non-unity scale is requested. */
-
-        if (powscale != 1.0) {
-            for (i = 0; i < meshsize; i++) {
-                for (j = 0; j < meshsize; j++) {
-                    double r = Real(a, i, j);
-
-                    if (r > 0) {
-                        Real(a, i, j) = pow(r, powscale);
-                    }
-                }
-            }
-        }
-
-        /* Compute extrema for autoscaling. */
-
-        for (i = 0; i < meshsize; i++) {
-            for (j = 0; j < meshsize; j++) {
-                double r = Real(a, i, j);
-
-                rmin = MIN(rmin, r);
-                rmax = MAX(rmax, r);
-            }
-        }
-        rmean = (rmin + rmax) / 2;
-        rrange = (rmax - rmin) / 2;
-        for (i = 0; i < meshsize; i++) {
-            for (j = 0; j < meshsize; j++) {
-                Real(a, i, j) = (Real(a, i, j) - rmean) / rrange;
-            }
+            error = FALSE;
         }
     }
-    genplanet(a, meshsize);
-    if (a != (float *) 0) {
-        free((char *) a);
-    }
-    return TRUE;
+    if (!error)
+        genplanet(stars, clouds, a, cols, rows, meshsize, rseed);
+
+    if (a != NULL)
+        free(a);
+
+    return !error;
 }
 
-/*  MAIN  --  Main program.  */
 
-int main(argc, argv)
-    int argc;
-    char *argv[];
-{
+
+
+int 
+main(int argc, char ** argv) {
+
+    bool success;
     int i;
     const char * const usage = "\n\
 [-width|-xsize <x>] [-height|-ysize <y>] [-mesh <n>]\n\
 [-clouds] [-dimension <f>] [-power <f>] [-seed <n>]\n\
 [-hour <f>] [-inclination|-tilt <f>] [-ice <f>] [-glaciers <f>]\n\
 [-night] [-stars <n>] [-saturation <n>]";
-    Boolean dimspec = FALSE, meshspec = FALSE, powerspec = FALSE,
+    bool dimspec = FALSE, meshspec = FALSE, powerspec = FALSE,
         widspec = FALSE, hgtspec = FALSE, icespec = FALSE,
         glacspec = FALSE, starspec = FALSE, starcspec = FALSE;
 
+    int cols, rows;     /* Dimensions of our output image */
 
     ppm_init(&argc, argv);
     i = 1;
+    
     while ((i < argc) && (argv[i][0] == '-') && (argv[i][1] != '\0')) {
 
         if (pm_keymatch(argv[i], "-clouds", 2)) {
@@ -907,7 +1130,7 @@ int main(argc, argv)
                 pm_error("already specified a random seed");
             }
             i++;
-            if ((i == argc) || (sscanf(argv[i], "%u", &rseed) != 1))
+            if ((i == argc) || (sscanf(argv[i], "%u", &seedarg) != 1))
                 pm_usage(usage);
             seedspec = TRUE;
         } else if (pm_keymatch(argv[i], "-xsize", 2) ||
@@ -965,12 +1188,10 @@ int main(argc, argv)
        calculated based on height.  
     */
 
-    screenxsize = MAX(screenysize, screenxsize);
-    screenxsize = (screenxsize + 1) & (~1);
-    exit(planet() ? 0 : 1);
+    cols = (MAX(screenysize, screenxsize) + 1) & (~1);
+    rows = screenysize;
+
+    success = planet(cols, rows, stars, clouds);
+
+    exit(success ? 0 : 1);
 }
-
-
-
-
-
