@@ -12,6 +12,10 @@
   (in addition to being the library for handling the PBM format).
 
 **************************************************************************/
+#define _SVID_SOURCE
+    /* Make sure P_tmpdir is defined in GNU libc 2.0.7 (_XOPEN_SOURCE 500
+       does it in other libc's.
+    */
 #define _XOPEN_SOURCE 500    /* Make sure ftello, fseeko are defined */
 #define _LARGEFILE_SOURCE 1  /* Make sure ftello, fseeko are defined */
 #define _LARGEFILE64_SOURCE 1 
@@ -79,34 +83,37 @@ pm_perror(const char reason[] ) {
         pm_error( "%s", e );
     }
 
+
+
 void GNU_PRINTF_ATTR
-pm_message(const char format[], ... ) {
+pm_message(const char format[], ...) {
 
     va_list args;
 
-    va_start( args, format );
+    va_start(args, format);
 
-    if ( pm_showmessages )
-        {
-        fprintf( stderr, "%s: ", pm_progname );
-        (void) vfprintf( stderr, format, args );
-        fputc( '\n', stderr );
-        }
-    va_end( args );
+    if (pm_showmessages) {
+        fprintf(stderr, "%s: ", pm_progname);
+        vfprintf(stderr, format, args);
+        fputc('\n', stderr);
     }
+    va_end(args);
+}
+
+
 
 void GNU_PRINTF_ATTR
-pm_error(const char format[], ... ) {
+pm_error(const char format[], ...) {
     va_list args;
 
-    va_start( args, format );
+    va_start(args, format);
 
-    fprintf( stderr, "%s: ", pm_progname );
-    (void) vfprintf( stderr, format, args );
-    fputc( '\n', stderr );
-    va_end( args );
-    exit( 1 );
-    }
+    fprintf(stderr, "%s: ", pm_progname);
+    vfprintf(stderr, format, args);
+    fputc('\n', stderr);
+    va_end(args);
+    exit(1);
+}
 
 
 /* Variable-sized arrays. */
@@ -139,7 +146,7 @@ pm_allocarray(int const cols, int const rows, int const size )  {
 
    We have two ways of allocating the space: fragmented and
    unfragmented.  In both, the row index (plus the extra element) is
-   in one block of memory.  In the fragemented format, each row is
+   in one block of memory.  In the fragmented format, each row is
    also in an independent memory block, and the extra row pointer is
    NULL.  In the unfragmented format, all the rows are in a single
    block of memory called the row heap and the extra row pointer is
@@ -458,8 +465,7 @@ showNetpbmHelp(const char progname[]) {
             char line[80+1];
             fgets(line, sizeof(line), netpbmConfigFile);
             if (line[0] != '#') {
-                int rc;
-                rc = sscanf(line, "docurl=%s", docurl);
+                sscanf(line, "docurl=%s", docurl);
             }
         }
         if (docurl == NULL)
@@ -931,17 +937,22 @@ pm_readmagicnumber(FILE * const ifP) {
 #define PM_MAX_BUF_INC 65536   /* Don't allocate more memory in larger blocks
                                    than this. */
 
-char *pm_read_unknown_size(FILE * const file, long * const nread) {
+char *
+pm_read_unknown_size(FILE * const file, 
+                     long * const nread) {
     long nalloc;
-    register int val;
+    int val;
     char* buf;
+    bool eof;
 
     *nread = 0;
     if ((buf=malloc(PM_BUF_SIZE)) == NULL)
         pm_error("Cannot allocate memory");
     nalloc = PM_BUF_SIZE;
 
-    while(1) {
+    eof = FALSE;  /* initial value */
+
+    while(!eof) {
         if (*nread >= nalloc) { /* We need a larger buffer */
             if (nalloc > PM_MAX_BUF_INC)
                 nalloc += PM_MAX_BUF_INC;
@@ -951,11 +962,13 @@ char *pm_read_unknown_size(FILE * const file, long * const nread) {
                 pm_error("Cannot allocate %ld bytes of memory", nalloc);
         }
 
-        if ((val = getc(file)) == EOF)
-            return (buf);
-
-        buf[(*nread)++] = val;
+        val = getc(file);
+        if (val == EOF)
+            eof = TRUE;
+        else 
+            buf[(*nread)++] = val;
     }
+    return buf;
 }
 
 
@@ -969,10 +982,12 @@ pm_tell2(FILE *       const fileP,
    'fileposSize' bytes long.  Abort the program if error, including if
    *fileP isn't a file that has a position.
 -----------------------------------------------------------------------------*/
-    /* Note: ftello may actually by ftello64 or ftell, depending on the 
-       capabilities of the underlying C library.
+    /* Note: FTELLO() is either ftello() or ftell(), depending on the
+       capabilities of the underlying C library.  It is defined in
+       pm_config.h.  ftello(), in turn, may be either ftell() or
+       ftello64(), as implemented by the C library.
     */
-    pm_filepos filepos = ftello(fileP);
+    pm_filepos const filepos = FTELLO(fileP);
     if (filepos < 0)
         pm_error("ftello() to get current file position failed.  "
                  "Errno = %s (%d)\n", strerror(errno), errno);
@@ -1015,10 +1030,12 @@ pm_seek2(FILE *             const fileP,
    if *fileP isn't a seekable file.
 -----------------------------------------------------------------------------*/
     if (fileposSize == sizeof(pm_filepos)) 
-        /* Note: fseeko may actually by fseeko64 or fseek, depending on the 
-           capabilities of the underlying C library.
+        /* Note: FSEEKO() is either fseeko() or fseek(), depending on the
+           capabilities of the underlying C library.  It is defined in
+           pm_config.h.  fseeko(), in turn, may be either fseek() or
+           fseeko64(), as implemented by the C library.
         */
-        fseeko(fileP, *fileposP, SEEK_SET);
+        FSEEKO(fileP, *fileposP, SEEK_SET);
     else if (fileposSize == sizeof(long)) {
         long const fileposLong = *(long *)fileposP;
         fseek(fileP, fileposLong, SEEK_SET);
@@ -1113,13 +1130,15 @@ pm_check(FILE *               const file,
     int rc;
 
 #ifdef LARGEFILEDEBUG
-    pm_message("pm_filepos received by pm_check() is %u bytes",
+    pm_message("pm_filepos received by pm_check() is %u bytes.",
                sizeof(pm_filepos));
 #endif
-    /* Note: ftello may actually by ftello64 or ftell, depending on the 
-       capabilities of the underlying C library.
+    /* Note: FTELLO() is either ftello() or ftell(), depending on the
+       capabilities of the underlying C library.  It is defined in
+       pm_config.h.  ftello(), in turn, may be either ftell() or
+       ftello64(), as implemented by the C library.
     */
-    curpos = ftello(file);
+    curpos = FTELLO(file);
     if (curpos >= 0) {
         /* This type of file has a current position */
             
