@@ -9,16 +9,16 @@
 ** implied warranty.
 */
 
-#define _BSD_SOURCE 1
-    /* Make sure strdup() is in string.h */
+#define _BSD_SOURCE 1      /* Make sure strdup() is in string.h */
+#define _XOPEN_SOURCE 500  /* Make sure strdup() is in string.h */
 
 #include <string.h>
+#include <math.h>
 #include "ppm.h"
 #include "colorname.h"
 #include "mallocvar.h"
 
 
-
 static void
 computeHexTable(int hexit[]) {
     int i;
@@ -44,176 +44,286 @@ computeHexTable(int hexit[]) {
 
 
 
+static long
+invRgbnorm(pixval       const rgb,
+           pixval       const maxval,
+           unsigned int const hexDigits) {
+
+    long retval;
+
+    switch (hexDigits) {
+    case 1:
+        retval = (long)((double) rgb * 15 / maxval + 0.5);
+        break;
+    case 2:
+        retval = (long) ((double) rgb * 255 / maxval + 0.5);
+        break;
+    case 3:
+        retval = (long) ((double) rgb * 4095 / maxval + 0.5);
+        break;
+    case 4:
+        retval = (long) ((double) rgb * 65535UL / maxval + 0.5);
+        break;
+    default:
+        pm_error("Internal error in invRgbnorm()");
+    }
+    return retval;
+}
+
+
+
+static pixval
+rgbnorm(long         const rgb, 
+        pixval       const maxval, 
+        unsigned int const hexDigits, 
+        bool         const closeOk,
+        const char * const colorname) {
+/*----------------------------------------------------------------------------
+   Normalize the color (r, g, or b) value 'rgb', which was specified with
+   'hexDigits' digits, to a maxval of 'maxval'.  If the number of digits isn't
+   valid, issue an error message and identify the complete color 
+   color specification in error as 'colorname'.
+
+   For example, if the user says 0ff/000/000 and the maxval is 100,
+   then rgb is 0xff, n is 3, and our result is 
+   0xff / (16**3-1) * 100 = 6.
+
+-----------------------------------------------------------------------------*/
+    pixval retval;
+
+    switch (hexDigits) {
+    case 1:
+        retval = (pixval)((double) rgb * maxval / 15 + 0.5);
+        break;
+    case 2:
+        retval = (pixval) ((double) rgb * maxval / 255 + 0.5);
+        break;
+    case 3:
+        retval = (pixval) ((double) rgb * maxval / 4095 + 0.5);
+        break;
+    case 4:
+        retval = (pixval) ((double) rgb * maxval / 65535L + 0.5);
+        break;
+    default:
+        pm_error("color specifier '%s' has too many digits", colorname);
+    }
+
+    if (!closeOk && invRgbnorm(retval, maxval, hexDigits) != rgb)
+        pm_message("WARNING: Component 0x%lx of color '%s' "
+                   "cannot be represented with maxval %u", 
+                   rgb, colorname, maxval);
+
+    return retval;
+}
+
+
+
 static void
-parseNewHexX11(const char colorname[], long const lmaxval,
-               long * const rP, long * const gP, long * const bP) {
+parseNewHexX11(const char       colorname[], 
+               pixval     const maxval,
+               bool       const closeOk,
+               pixel *    const colorP) {
 
     int hexit[256];
 
     const char* cp;
-    long r,g,b;
+    pixval r,g,b;
+    pixval rNorm, gNorm, bNorm;
+
     int i;
 
     computeHexTable(hexit);
 
     cp = colorname + 4;
-    r = g = b = 0;
-    for ( i = 0; *cp != '/'; ++i, ++cp )
+
+    for (i = 0, r = 0; *cp != '/'; ++i, ++cp)
         r = r * 16 + hexit[(int)*cp];
-    r = pm_rgbnorm( r, lmaxval, i, colorname );
-    for ( i = 0, ++cp; *cp != '/'; ++i, ++cp )
+    rNorm = rgbnorm(r, maxval, i, closeOk, colorname);
+
+    for (i = 0, g = 0,++cp; *cp != '/'; ++i, ++cp )
         g = g * 16 + hexit[(int)*cp];
-    g = pm_rgbnorm( g, lmaxval, i, colorname );
-    for ( i = 0, ++cp; *cp != '\0'; ++i, ++cp )
+    gNorm = rgbnorm(g, maxval, i, closeOk, colorname);
+
+    for (i = 0, b = 0, ++cp; *cp != '\0'; ++i, ++cp )
         b = b * 16 + hexit[(int)*cp];
-    b = pm_rgbnorm( b, lmaxval, i, colorname );
-    if ( r < 0 || r > lmaxval || g < 0 || g > lmaxval 
-         || b < 0 || b > lmaxval )
-        pm_error("invalid color specifier - \"%s\"", colorname );
+    bNorm = rgbnorm(b, maxval, i, closeOk, colorname);
 
-    *rP = r; *gP = g; *bP = b;
+    PPM_ASSIGN(*colorP, rNorm, gNorm, bNorm);
 }
 
 
 
 static void
-parseNewDecX11(const char colorname[], long const lmaxval,
-            long * const rP, long * const gP, long * const bP) {
+parseNewDecX11(const char       colorname[], 
+               long       const maxval,
+               bool       const closeOk,
+               pixel *    const colorP) {
 
+    float const epsilon = 1.0/65536.0;
     float fr, fg, fb;
+    pixval rNorm, gNorm, bNorm;
 
-    if ( sscanf( colorname, "rgbi:%f/%f/%f", &fr, &fg, &fb ) != 3 )
-        pm_error("invalid color specifier - \"%s\"", colorname );
-    if ( fr < 0.0 || fr > 1.0 || fg < 0.0 || fg > 1.0 
-         || fb < 0.0 || fb > 1.0 )
-        pm_error( "invalid color specifier - \"%s\" - "
-                  "values must be between 0.0 and 1.0", colorname );
-    *rP = fr * lmaxval;
-    *gP = fg * lmaxval;
-    *bP = fb * lmaxval;
+    if (sscanf( colorname, "rgbi:%f/%f/%f", &fr, &fg, &fb) != 3)
+        pm_error("invalid color specifier '%s'", colorname);
+    if (fr < 0.0 || fr > 1.0 || fg < 0.0 || fg > 1.0 
+        || fb < 0.0 || fb > 1.0)
+        pm_error("invalid color specifier '%s' - "
+                 "values must be between 0.0 and 1.0", colorname );
+
+    rNorm = fr * maxval + 0.5;
+    gNorm = fg * maxval + 0.5;
+    bNorm = fb * maxval + 0.5;
+
+    if (!closeOk) {
+        if (fabs((double)rNorm/maxval - fr) > epsilon ||
+            fabs((double)gNorm/maxval - fg) > epsilon ||
+            fabs((double)bNorm/maxval - fb) > epsilon)
+            pm_message("WARNING: Color '%s' cannot be represented "
+                       "precisely with maxval %lu", colorname, maxval);
+    }
+    PPM_ASSIGN(*colorP, rNorm, gNorm, bNorm);
 }
 
 
 
-
 static void
-parseOldX11(const char colorname[], long const lmaxval,
-            long * const rP, long * const gP, long * const bP) {
+parseOldX11(const char       colorname[], 
+            pixval     const maxval,
+            bool       const closeOk,
+            pixel *    const colorP) {
 
     int hexit[256];
     long r,g,b;
-
+    pixval rNorm, gNorm, bNorm;
+    
     computeHexTable(hexit);
 
-    switch ( strlen( colorname ) - 1 /* (Number of hex digits) */ )
-    {
+    switch (strlen(colorname) - 1 /* (Number of hex digits) */) {
     case 3:
         r = hexit[(int)colorname[1]];
         g = hexit[(int)colorname[2]];
         b = hexit[(int)colorname[3]];
-        r = pm_rgbnorm( r, lmaxval, 1, colorname );
-        g = pm_rgbnorm( g, lmaxval, 1, colorname );
-        b = pm_rgbnorm( b, lmaxval, 1, colorname );
+        rNorm = rgbnorm(r, maxval, 1, closeOk, colorname);
+        gNorm = rgbnorm(g, maxval, 1, closeOk, colorname);
+        bNorm = rgbnorm(b, maxval, 1, closeOk, colorname);
         break;
 
     case 6:
-        r = ( hexit[(int)colorname[1]] << 4 ) + hexit[(int)colorname[2]];
-        g = ( hexit[(int)colorname[3]] << 4 ) + hexit[(int)colorname[4]];
-        b = ( hexit[(int)colorname[5]] << 4 ) + hexit[(int)colorname[6]];
-        r = pm_rgbnorm( r, lmaxval, 2, colorname );
-        g = pm_rgbnorm( g, lmaxval, 2, colorname );
-        b = pm_rgbnorm( b, lmaxval, 2, colorname );
+        r = (hexit[(int)colorname[1]] << 4 ) + hexit[(int)colorname[2]];
+        g = (hexit[(int)colorname[3]] << 4 ) + hexit[(int)colorname[4]];
+        b = (hexit[(int)colorname[5]] << 4 ) + hexit[(int)colorname[6]];
+        rNorm = rgbnorm(r, maxval, 2, closeOk, colorname);
+        gNorm = rgbnorm(g, maxval, 2, closeOk, colorname);
+        bNorm = rgbnorm(b, maxval, 2, closeOk, colorname);
         break;
 
     case 9:
-        r = ( hexit[(int)colorname[1]] << 8 ) +
-            ( hexit[(int)colorname[2]] << 4 ) +
-            hexit[(int)colorname[3]];
-        g = ( hexit[(int)colorname[4]] << 8 ) + 
-            ( hexit[(int)colorname[5]] << 4 ) +
-            hexit[(int)colorname[6]];
-        b = ( hexit[(int)colorname[7]] << 8 ) + 
-            ( hexit[(int)colorname[8]] << 4 ) +
-            hexit[(int)colorname[9]];
-        r = pm_rgbnorm( r, lmaxval, 3, colorname );
-        g = pm_rgbnorm( g, lmaxval, 3, colorname );
-        b = pm_rgbnorm( b, lmaxval, 3, colorname );
+        r = (hexit[(int)colorname[1]] << 8) +
+            (hexit[(int)colorname[2]] << 4) +
+            (hexit[(int)colorname[3]] << 0);
+        g = (hexit[(int)colorname[4]] << 8) + 
+            (hexit[(int)colorname[5]] << 4) +
+            (hexit[(int)colorname[6]] << 0);
+        b = (hexit[(int)colorname[7]] << 8) + 
+            (hexit[(int)colorname[8]] << 4) +
+            (hexit[(int)colorname[9]] << 0);
+        rNorm = rgbnorm(r, maxval, 3, closeOk, colorname);
+        gNorm = rgbnorm(g, maxval, 3, closeOk, colorname);
+        bNorm = rgbnorm(b, maxval, 3, closeOk, colorname);
         break;
 
     case 12:
-        r = ( hexit[(int)colorname[1]] << 12 ) + 
-            ( hexit[(int)colorname[2]] << 8 ) +
-            ( hexit[(int)colorname[3]] << 4 ) + hexit[(int)colorname[4]];
-        g = ( hexit[(int)colorname[5]] << 12 ) + 
-            ( hexit[(int)colorname[6]] << 8 ) +
-            ( hexit[(int)colorname[7]] << 4 ) + hexit[(int)colorname[8]];
-        b = ( hexit[(int)colorname[9]] << 12 ) + 
-            ( hexit[(int)colorname[10]] << 8 ) +
-            ( hexit[(int)colorname[11]] << 4 ) + hexit[(int)colorname[12]];
-        r = pm_rgbnorm( r, lmaxval, 4, colorname );
-        g = pm_rgbnorm( g, lmaxval, 4, colorname );
-        b = pm_rgbnorm( b, lmaxval, 4, colorname );
+        r = (hexit[(int)colorname[1]] << 12) + 
+            (hexit[(int)colorname[2]] <<  8) +
+            (hexit[(int)colorname[3]] <<  4) + hexit[(int)colorname[4]];
+        g = (hexit[(int)colorname[5]] << 12) + 
+            (hexit[(int)colorname[6]] <<  8) +
+            (hexit[(int)colorname[7]] <<  4) + hexit[(int)colorname[8]];
+        b = (hexit[(int)colorname[9]] << 12) + 
+            (hexit[(int)colorname[10]] << 8) +
+            (hexit[(int)colorname[11]] << 4) + hexit[(int)colorname[12]];
+        rNorm = rgbnorm(r, maxval, 4, closeOk, colorname);
+        gNorm = rgbnorm(g, maxval, 4, closeOk, colorname);
+        bNorm = rgbnorm(b, maxval, 4, closeOk, colorname);
         break;
 
     default:
-        pm_error("invalid color specifier - \"%s\"", colorname );
+        pm_error("invalid color specifier '%s'", colorname);
     }
-    if ( r < 0 || r > lmaxval || g < 0 || g > lmaxval 
-         || b < 0 || b > lmaxval )
-        pm_error("invalid color specifier - \"%s\"", colorname );
-    
-    *rP = r; *gP = g; *bP = b;
+    PPM_ASSIGN(*colorP, rNorm, gNorm, bNorm);
 }
 
 
 
 
 static void
-parseOldX11Dec(const char colorname[], long const lmaxval,
-               long * const rP, long * const gP, long * const bP) {
+parseOldX11Dec(const char       colorname[], 
+               pixval     const maxval,
+               bool       const closeOk,
+               pixel *    const colorP) {
+
+    float const epsilon = 1.0/65536.0;
 
     float fr, fg, fb;
+    pixval rNorm, gNorm, bNorm;
 
-    if ( sscanf( colorname, "%f,%f,%f", &fr, &fg, &fb ) != 3 )
-        pm_error("invalid color specifier - \"%s\"", colorname );
-    if ( fr < 0.0 || fr > 1.0 || fg < 0.0 || fg > 1.0 
-         || fb < 0.0 || fb > 1.0 )
-        pm_error( "invalid color specifier - \"%s\" - "
-                  "values must be between 0.0 and 1.0", colorname );
-    *rP = fr * lmaxval;
-    *gP = fg * lmaxval;
-    *bP = fb * lmaxval;
+    if (sscanf(colorname, "%f,%f,%f", &fr, &fg, &fb) != 3)
+        pm_error("invalid color specifier '%s'", colorname);
+    if (fr < 0.0 || fr > 1.0 || fg < 0.0 || fg > 1.0 
+        || fb < 0.0 || fb > 1.0)
+        pm_error("invalid color specifier '%s' - "
+                 "values must be between 0.0 and 1.0", colorname );
+
+    rNorm = fr * maxval + 0.5;
+    gNorm = fg * maxval + 0.5;
+    bNorm = fb * maxval + 0.5;
+
+    if (!closeOk) {
+        if (fabs((float)rNorm/maxval - fr) > epsilon ||
+            fabs((float)gNorm/maxval - fg) > epsilon ||
+            fabs((float)bNorm/maxval - fb) > epsilon)
+            pm_message("WARNING: Color '%s' cannot be represented "
+                       "precisely with maxval %u", colorname, maxval);
+    }
+    PPM_ASSIGN(*colorP, rNorm, gNorm, bNorm);
 }
 
 
 
-
 pixel
-ppm_parsecolor(const char * const colorname, pixval const maxval) {
+ppm_parsecolor2(const char * const colorname,
+                pixval       const maxval,
+                int          const closeOk) {
 
-    pixel p;
-    long lmaxval, r, g, b;
+    pixel color;
     
-    lmaxval = maxval;
-    if ( strncmp( colorname, "rgb:", 4 ) == 0 )
+    if (strncmp(colorname, "rgb:", 4) == 0)
         /* It's a new-X11-style hexadecimal rgb specifier. */
-        parseNewHexX11(colorname, lmaxval, &r, &g, &b);
-    else if ( strncmp( colorname, "rgbi:", 5 ) == 0 )
+        parseNewHexX11(colorname, maxval, closeOk, &color);
+    else if (strncmp(colorname, "rgbi:", 5) == 0)
         /* It's a new-X11-style decimal/float rgb specifier. */
-        parseNewDecX11(colorname, lmaxval, &r, &g, &b);
-    else if ( colorname[0] == '#' )
+        parseNewDecX11(colorname, maxval, closeOk, &color);
+    else if (colorname[0] == '#')
         /* It's an old-X11-style hexadecimal rgb specifier. */
-        parseOldX11(colorname, lmaxval, &r, &g, &b);
-    else if ( ( colorname[0] >= '0' && colorname[0] <= '9' ) ||
-              colorname[0] == '.' )
+        parseOldX11(colorname, maxval, closeOk, &color);
+    else if ((colorname[0] >= '0' && colorname[0] <= '9') ||
+             colorname[0] == '.')
         /* It's an old-style decimal/float rgb specifier. */
-        parseOldX11Dec(colorname, lmaxval, &r, &g, &b);
+        parseOldX11Dec(colorname, maxval, closeOk, &color);
     else 
         /* Must be a name from the X-style rgb file. */
-        pm_parse_dictionary_name(colorname, lmaxval, &r, &g, &b);
+        pm_parse_dictionary_name(colorname, maxval, closeOk, &color);
+    
+    return color;
+}
 
-    PPM_ASSIGN( p, r, g, b );
-    return p;
+
+
+pixel
+ppm_parsecolor(const char * const colorname, 
+               pixval       const maxval) {
+
+    return ppm_parsecolor2(colorname, maxval, TRUE);
 }
 
 
