@@ -17,6 +17,7 @@
 
 #include "ppm.h"
 #include "ppmdraw.h"
+#include "mallocvar.h"
 
 #define DDA_SCALE 8192
 
@@ -430,25 +431,26 @@ typedef struct
 static int oldclip;
 
 char*
-ppmd_fill_init( )
-    {
+ppmd_fill_init( ) {
     fillobj* fh;
 
-    fh = (fillobj*) malloc( sizeof(fillobj) );
-    if ( fh == 0 )
-	pm_error( "out of memory allocating a fillhandle" );
+    MALLOCVAR(fh);
+    if (fh == NULL)
+        pm_error("out of memory allocating a fillhandle");
     fh->n = 0;
-    fh->coords = (coord*) malloc( SOME * sizeof(coord) );
-    if ( fh->coords == 0 )
-	pm_error( "out of memory allocating a fillhandle" );
     fh->size = SOME;
+    MALLOCARRAY(fh->coords, fh->size);
+    if (fh->coords == NULL)
+        pm_error("out of memory allocating a fillhandle");
     fh->curedge = 0;
-
+    
     /* Turn off line clipping. */
-    oldclip = ppmd_setlineclip( 0 );
+    oldclip = ppmd_setlineclip(0);
     
     return (char*) fh;
-    }
+}
+
+
 
 void
 ppmd_fill_drawproc(pixel** const pixels, 
