@@ -163,28 +163,84 @@ pm_closew (FILE* const f);
 
 
 int
-pm_readbigshort ( FILE* const in, short* const sP );
+pm_readbigshort(FILE *  const in, 
+                short * const sP);
+
+static __inline__ int
+pm_readbigshortu(FILE*            const in, 
+                 unsigned short * const sP) {
+    return pm_readbigshort(in, (short *) sP);
+}
 
 int
-pm_writebigshort ( FILE* const out, short const s );
+pm_writebigshort(FILE * const out, 
+                 short  const s);
+
+static __inline__ int
+pm_writebigshortu(FILE *          const out, 
+                  unsigned short  const s) {
+    return pm_writebigshort(out, (short) s);
+}
 
 int
-pm_readbiglong ( FILE* const in, long* const lP );
+pm_readbiglong(FILE * const in, 
+               long * const lP);
+
+static __inline__ int
+pm_readbiglongu(FILE *          const in, 
+                unsigned long * const lP) {
+    return pm_readbiglong(in, (long *) lP);
+}
 
 int
-pm_writebiglong ( FILE* const out, long const l );
+pm_writebiglong(FILE * const out, 
+                long   const l);
+
+static __inline__ int
+pm_writebiglongu(FILE *        const out, 
+                 unsigned long const l) {
+    return pm_writebiglong(out, (long) l);
+}
 
 int
-pm_readlittleshort ( FILE* const in, short* const sP );
+pm_readlittleshort(FILE  * const in, 
+                   short * const sP);
+
+static __inline__ int
+pm_readlittleshortu(FILE  *          const in, 
+                    unsigned short * const sP) {
+    return pm_readlittleshort(in, (short *) sP);
+}
 
 int
-pm_writelittleshort ( FILE* const out, short const s );
+pm_writelittleshort(FILE * const out, 
+                    short  const s);
+
+static __inline__ int
+pm_writelittleshortu(FILE *          const out, 
+                     unsigned short  const s) {
+    return pm_writelittleshort(out, (short) s);
+}
 
 int
-pm_readlittlelong ( FILE* const in, long* const lP );
+pm_readlittlelong(FILE * const in, 
+                  long * const lP);
+
+static __inline__ int
+pm_readlittlelongu(FILE *          const in, 
+                   unsigned long * const lP) {
+    return pm_readlittlelong(in, (long *) lP);
+}
 
 int
-pm_writelittlelong ( FILE* const out, long const l );
+pm_writelittlelong(FILE * const out, 
+                   long   const l);
+
+static __inline__ int
+pm_writelittlelongu(FILE *        const out, 
+                    unsigned long const l) {
+    return pm_writelittlelong(out, (long) l);
+}
 
 int 
 pm_readmagicnumber(FILE * const ifP);
