@@ -67,32 +67,76 @@ typedef struct
 
 /* Declarations of routines. */
 
-void ppm_init ARGS(( int* argcP, char* argv[] ));
+void ppm_init(int * argcP, char* argv[]);
 
 #define ppm_allocarray( cols, rows ) ((pixel**) pm_allocarray( cols, rows, sizeof(pixel) ))
 #define ppm_allocrow( cols ) ((pixel*) pm_allocrow( cols, sizeof(pixel) ))
 #define ppm_freearray( pixels, rows ) pm_freearray( (char**) pixels, rows )
 #define ppm_freerow( pixelrow ) pm_freerow( (char*) pixelrow )
 
-pixel** ppm_readppm ARGS(( FILE* file, int* colsP, int* rowsP, pixval* maxvalP ));
-void ppm_readppminit ARGS(( FILE* file, int* colsP, int* rowsP, pixval* maxvalP, int* formatP ));
-void ppm_readppmrow ARGS(( FILE* file, pixel* pixelrow, int cols, pixval maxval, int format ));
-
-void ppm_writeppm ARGS(( FILE* file, pixel** pixels, int cols, int rows, pixval maxval, int forceplain ));
-void ppm_writeppminit ARGS(( FILE* file, int cols, int rows, pixval maxval, int forceplain ));
-void ppm_writeppmrow ARGS(( FILE* file, pixel* pixelrow, int cols, pixval maxval, int forceplain ));
+pixel**
+ppm_readppm(FILE *   const fileP, 
+            int *    const colsP, 
+            int *    const rowsP, 
+            pixval * const maxvalP);
 
 void
-ppm_check(FILE * file, const enum pm_check_type check_type, 
-          const int format, const int cols, const int rows, const int maxval,
+ppm_readppminit(FILE *   const fileP, 
+                int *    const colsP, 
+                int *    const rowsP, 
+                pixval * const maxvalP, 
+                int *    const formatP);
+
+void
+ppm_readppmrow(FILE*  const fileP, 
+               pixel* const pixelrow, 
+               int    const cols, 
+               pixval const maxval, 
+               int    const format);
+
+void
+ppm_writeppm(FILE *  const fileP, 
+             pixel** const pixels, 
+             int     const cols, 
+             int     const rows, 
+             pixval  const maxval, 
+             int     const forceplain);
+
+void
+ppm_writeppminit(FILE*  const fileP, 
+                 int    const cols, 
+                 int    const rows, 
+                 pixval const maxval, 
+                 int    const forceplain);
+
+void
+ppm_writeppmrow(FILE *  const fileP, 
+                pixel * const pixelrow, 
+                int     const cols, 
+                pixval  const maxval, 
+                int     const forceplain);
+
+void
+ppm_check(FILE *               const fileP, 
+          enum pm_check_type   const check_type, 
+          int                  const format, 
+          int                  const cols, 
+          int                  const rows, 
+          pixval               const maxval,
           enum pm_check_code * const retval_p);
 
 void
-ppm_nextimage(FILE *file, int * const eofP);
+ppm_nextimage(FILE * const fileP, 
+              int *  const eofP);
 
 pixel 
 ppm_parsecolor(const char* colorname, pixval maxval);
 
+pixel
+ppm_parsecolor2(const char * const colorname,
+                pixval       const maxval,
+                int          const closeOk);
+
 char*
 ppm_colorname(const pixel* const colorP, 
               pixval       const maxval, 
@@ -126,6 +170,10 @@ ppm_freecolornames(const char ** colornames);
 	( (int) PPM_GETG(p) * (newmaxval) + (oldmaxval) / 2 ) / (oldmaxval), \
 	( (int) PPM_GETB(p) * (newmaxval) + (oldmaxval) / 2 ) / (oldmaxval) )
 
+#define PPM_DISTANCE(p1,p2) ( \
+  SQR(PPM_GETR(p1)-PPM_GETR(p2)) + \
+  SQR(PPM_GETG(p1)-PPM_GETR(p2)) + \
+  SQR(PPM_GETB(p1)-PPM_GETB(p2)) )
 
 /* Luminance, Chrominance macros. */
 
