@@ -29,6 +29,7 @@
 #include <limits.h>
 #include <errno.h>
 
+#include "nstring.h"
 #include "shhopt.h"
 
 /*-----------------------------------------------------------------------+
@@ -435,6 +436,9 @@ optParseOptions(int *argc, char *argv[], optStruct opt[], int allowNegNum)
     optEntry *opt_table;  /* malloc'ed array */
 
     opt_table = optStructTblToEntryTbl(opt);
+    if (opt_table == NULL)
+        optFatal("Memory allocation failed (trying to allocate space for "
+                 "new-format option table)");
 
     /*
      *  Loop through all arguments.
@@ -449,7 +453,7 @@ optParseOptions(int *argc, char *argv[], optStruct opt[], int allowNegNum)
 	    break;
 	}
 
-	if (allowNegNum && argv[ai][0] == '-' && isdigit(argv[ai][1])) {
+	if (allowNegNum && argv[ai][0] == '-' && ISDIGIT(argv[ai][1])) {
 	    ++ai;
 	    continue;
 	} else if (strncmp(argv[ai], "--", 2) == 0) {
@@ -777,7 +781,7 @@ optParseOptions3(int * const argc_p, char *argv[], const optStruct3 opt,
                 /* A dash by itself is not considered an option. */
                 ++ai;
                 tokens_consumed = 0;
-            } else if (opt.allowNegNum && isdigit(argv[ai][1])) {
+            } else if (opt.allowNegNum && ISDIGIT(argv[ai][1])) {
                 /* It's a negative number parameter, not an option */
                 ++ai;
                 tokens_consumed = 0;
