@@ -28,6 +28,7 @@
 #include "pam.h"
 #include "pammap.h"
 #include "shhopt.h"
+#include "mallocvar.h"
 
 #define MAXCOLORS 32767u
 #define MAXDEPTH 3
@@ -75,7 +76,7 @@ parseCommandLine (int argc, char ** argv,
    Note that the strings we return are stored in the storage that
    was passed to us as the argv array.  We also trash *argv.
 -----------------------------------------------------------------------------*/
-    optEntry *option_def = malloc( 100*sizeof( optEntry ) );
+    optEntry *option_def;
         /* Instructions to optParseOptions3 on how to parse our options.
          */
     optStruct3 opt;
@@ -85,6 +86,8 @@ parseCommandLine (int argc, char ** argv,
     unsigned int spreadbrightness, spreadluminosity;
     unsigned int center, meancolor, meanpixel;
 
+    MALLOCARRAY_NOFAIL(option_def, 100);
+
     option_def_index = 0;   /* incremented by OPTENT3 */
     OPTENT3(0,   "spreadbrightness", OPT_FLAG,   
             NULL,                       &spreadbrightness, 0);
@@ -249,7 +252,7 @@ static boxVector
 newBoxVector(int const colors, int const sum, int const newcolors) {
 
     boxVector bv;
-    bv = (boxVector) malloc(sizeof(struct box) * newcolors);
+    MALLOCARRAY(bv, newcolors);
     if (bv == NULL)
         pm_error("out of memory allocating box vector table");
 
@@ -859,12 +862,12 @@ colormapToImage(struct pam * const inpamP,
 
    May rearrange the contents of 'colormap'.
 -----------------------------------------------------------------------------*/
-    outpamP->size = sizeof(*outpamP);
-    outpamP->len = sizeof(*outpamP);
-    outpamP->format = inpamP->format;
-    outpamP->plainformat = FALSE;
-    outpamP->depth = inpamP->depth;
-    outpamP->maxval = colormapMaxval;
+    outpamP->size             = sizeof(*outpamP);
+    outpamP->len              = PAM_STRUCT_SIZE(tuple_type);
+    outpamP->format           = inpamP->format;
+    outpamP->plainformat      = FALSE;
+    outpamP->depth            = inpamP->depth;
+    outpamP->maxval           = colormapMaxval;
     outpamP->bytes_per_sample = inpamP->bytes_per_sample;
     strcpy(outpamP->tuple_type, inpamP->tuple_type);
 
@@ -898,7 +901,7 @@ main(int argc, char * argv[] ) {
 
     ifP = pm_openr_seekable(cmdline.inputFilespec);
 
-    pnm_readpaminit(ifP, &inpam, sizeof(inpam));
+    pnm_readpaminit(ifP, &inpam, PAM_STRUCT_SIZE(tuple_type));
     
     computeColorMapFromInput(&inpam,
                              cmdline.allcolors, cmdline.newcolors, 
@@ -923,8 +926,6 @@ main(int argc, char * argv[] ) {
 
     pm_close(stdout);
 
-    exit(0);
+    return 0;
+    
 }
-
-
-
